#ifndef GATEWAY_H_
#define GATEWAY_H_

/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */



static const char* const Gateway_h_Version =
    "$Id: Gateway.h,v 1.20 2001/01/23 22:22:34 cktam Exp $";

#include "MgAgent.hxx"
#include "common.h"

class Endpoint;
class Hardware;


const int GW_ENDPOINT_MAX = 24;
const int GW_NUM_RTP_PORTS = 100;
const int GW_INV_ENDPOINT_INDEX = -1;


/** Global configuration data as specified in configuration file
 */
typedef struct
{
    string deviceName;
    int gatewayPort;
    string agentHostname;
    int agentPort;
    int numEndpoints;
    string endpointId[GW_ENDPOINT_MAX];
    int cntsPerEndpt;
    int firstRtpPort;  // start of range of rtp ports
}

GatewayConfigData;


/** Base class for all gateways.  Specifies the MGCP gateway API.
 ** Defines a basic level of service provided by all MGCP gateways.
 */
class Gateway
{
    public:

        /** Creates a gateway and configures the gateway according to
         ** configuration file
         */
        Gateway (void);

        /** Deletes a gateway, including deallocating all heap memory
         ** allocated by the gateway.
         */
        ~Gateway (void);

        /** Starts the main processing loop of the gateway
         */
        void
        gatewayMain (void);

        /** Takes an rtp port from the free list. The return value is the
         ** port number of the new port, or an errorcode if no port is
         ** available.
         */
        int
        newRtpPort (void);

        /** Returns an rtp port to the free list
         */
        void
        deleteRtpPort (int port);

        /** Creates and returns a unique connection id
         */
        string
        newConnectionId (void);

        /** Accessors to private data
         */
        inline const string
        getDomainName (void)
        {
            return myDomainName;
        };

    private:
        void
        createHardware();

        void
        createEndpoints();


        /** Associates a new call agent with the gateway. Returns a ptr to
         ** the new call agent if successful and returns NULL
         ** otherwise. The gateway may be associated with at most one call
         ** agent, so any previously existing call agent association is
         ** removed if and only if the association with the new call agent
         ** is successful. Also spawns the mgcp thread for the agent.
         */
        MgAgent*
        setAgent (string agentHostname, int agentPort);

        /** Searches the endpoint id array for a match. If a match is found,
         ** returns the index into the array, otherwise returns an errorcode.
         */
        int
        getEndpointIndex (string id);

        /** Forwards an mgcp command to the appropriate endpoint by adding
         ** the command to the endpoints input queue
         */
        void
        forwardCommand (CMD cmd, MgCommandType type,
                        Fifo < EndpointMsg > * outputQ);

        /** Deletes the call agent and rejoins the the mgcp thread of the
         ** agent with the gateway thread. 
         */
        void
        deleteAgent (void);

        /** Deletes the hardware, flushes and deletes the hardware queue,
         ** and rejoins the the hardware thread with the gateway thread.
         */
        void
        deleteHardware (void);

        /** Deletes the endpoint, flushes and deletes the endpoint queue,
         ** and rejoins the the endpoint thread with the gateway thread.
         */
        void
        deleteEndpoint (int index);

        /// call agent associated with gateway
        MgAgent* myAgent;

        /// thread to process agent's mgcp stack
        VThread myAgentThread;

        /// hardware associated with gateway
        Hardware* myHardware;

        /// hardware thread
        VThread myHardwareThread;

        /// endpoint(s) associated with gateway
        Endpoint* myEndpoint[GW_ENDPOINT_MAX];

        /// endpoint threads, one per endpoint
        VThread myEndpointThread[GW_ENDPOINT_MAX];

        /// thread to monitor interthread communication via fifo class
        VThread myMonitorThread;

        /// message queue for each endpoint
        Fifo < EndpointMsg > * endpointQ[GW_ENDPOINT_MAX];

        /// message queue for hardware
        Fifo < HardwareMsg > * hardwareQ;

        /// number of endpoints currently active
        int activeEndpoints;

        /// port provided by gateway to receive commands from call agent
        int myPort;

        /// rtp port to be used for next audio session on this gateway
        int currentRtpPort;

        /// endpoint ids of active endpoints
        string myEndpointId[GW_ENDPOINT_MAX];

        /// domain name for gateway and all its endpoints and connections
        string myDomainName;

        /// mutex variables to make access to shared variables safe
        VMutex cidMutex;
        VMutex rtpMutex;

};

#endif // GATEWAY_H_
