
/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */
static const char* const Connection_cxx_Version =
    "$Id: Connection.cxx,v 1.19 2000/12/18 23:42:25 bko Exp $";
#include "MgAgent.h"
#include "Hardware.h"
#include "Gateway.h"
#include "Connection.h"

extern Gateway* theGateway;

// strings used for debug purposes to display state info
char* ConnectionStateString[ConnectionOpen + 1] =
    {
        "Closed",
        "Half Open",
        "Open"
    };

Connection::Connection (MgAgent* agent, Fifo < HardwareMsg > * outputQ, int endpointIndex)
        : myLocalSD(),
        myRemoteSD()
{
    myAgent = agent;
    myHardwareQ = outputQ;
    myState = ConnectionClosed;
    myEndpointIndex = endpointIndex;

    // set persitent fields of local SD, non persistent fields are set
    // by method setMyLocalSD() for each session
    myLocalSD.setSessionName("Vovida Gateway");
    myLocalSD.setUserName("-");
    myLocalSD.setVersion(0);
    myLocalSD.setAddress(theGateway->getDomainName());

} // end Connection::Connection()


Connection::~Connection (void)
{} // end Connection::~Connection()



MgcpResponseCode
Connection::open (CRCX crcx)
{
    MgcpResponseCode responseCode = TransactionExecuted;

    switch (myState)
    {

        case ConnectionClosed:
        {
            // allocate new connection info
            myId = theGateway->newConnectionId();
            setMyLocalSD();

            // store mandatory connection parameters
            myCallId = crcx->getCallId();
            myMode = crcx->getConnectionMode();

            // get remote SD
            myRemoteSD = crcx->getRemoteConnectionDescriptor();

            // request audio session
            requestAudioSession( crcx->getLocalConnectionOptions() );

            // set response parameters and send reply
            crcx->setConnectionId(myId);
            crcx->setLocalConnectionDescriptor(myLocalSD);
        }
        break;

        case ConnectionHalfOpen:
        case ConnectionOpen:
        {
            cpLog(LOG_ERR, "Connection already in use");
            //responseCode = ReasonIncorrectConnectionId;
        }

        break;

    } // end switch

    return responseCode;

} // end Connection:open()


MgcpResponseCode
Connection::modify (MDCX mdcx)
{
    MgcpResponseCode responseCode = TransactionExecuted;

    switch (myState)
    {

        case ConnectionClosed:
        {
            cpLog(LOG_ERR, "Connection closed, cannot modify");
            //responseCode = ReasonIncorrectConnectionId;
        }

        break;

        case ConnectionHalfOpen:
        case ConnectionOpen:
        {
            if (myState == ConnectionOpen)
            {
                // cannot modify audio session so must first ask hardware
                // to close existing one then request a new one
                closeAudioSession();
            }

            // complete connection using remote SD provided by agent
            myRemoteSD = mdcx->getRemoteConnectionDescriptor();

            requestAudioSession( mdcx->getLocalConnectionOptions());
        }
        break;

    } // end switch

    return responseCode;

} // end Connection::modify()


MgcpResponseCode
Connection::close (DLCX dlcx)
{
    MgcpResponseCode responseCode = ConnectionDeleted;

    switch (myState)
    {

        case ConnectionClosed:
    {}

        break;

        case ConnectionHalfOpen:
        case ConnectionOpen:
        {
            closeAudioSession();
        }
        break;

    } // end switch

    return responseCode;

} // end Connection::close()


void
Connection::setMyLocalSD (void)
{
    myLocalSD.setSessionId(atoi(myId.c_str()));
    myLocalSD.setPort(theGateway->newRtpPort());
} // end Connection::setMyLocalSD()


void
Connection::requestAudioSession (MgcpConnectionOptions options)
{
    HardwareMsg msg;

    // build msg to send to hardware to create audio session
    msg.type = HardwareAudioType;
    msg.request.type = AudioStart;
    msg.request.mode = myMode.c_str();
    msg.request.localPort = myLocalSD.getPort();
    msg.endpointIndex = myEndpointIndex;

    if (options.getEchoCancellation())
    {
        msg.request.echoCancellation = true;
    }
    else
    {
        msg.request.echoCancellation = false;
    };

    if (myRemoteSD.isValidDescriptor())
    {
        // remote end of connection is known, create complete
        // connection by setting remote connection info
        msg.request.remotePort = myRemoteSD.getPort();
        msg.request.remoteHost = myRemoteSD.getAddress()->c_str();
        changeState(ConnectionOpen);
    }
    else
    {
        // don't know remote info yet, create half open connection
        // TBD, should mode be forced to receive only?
        msg.request.remotePort = 0;
        msg.request.remoteHost = "";
        changeState(ConnectionHalfOpen);
        // TBD don't open session yet, need RTP API to be improved to
        // allow modify session; for now force return
        return ;
    }
    myHardwareQ->add(msg);

} // end Connection::requestAudioSession()


void
Connection::closeAudioSession (void)
{
    HardwareMsg msg;

    // build msg to send to hardware to close audio session
    msg.type = HardwareAudioType;
    msg.request.type = AudioStop;
    msg.endpointIndex = myEndpointIndex;


    myHardwareQ->add(msg);
    changeState(ConnectionClosed);

} // end Connection::closeAudioSession()

void
Connection::changeState (ConnectionState newState)
{
    cpLog(LOG_DEBUG, "%s: %s -> %s", myId.c_str(),
          ConnectionStateString[myState],
          ConnectionStateString[newState]);
    myState = newState;
} // end Connection::changeState()
