
/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */

#include "LGInfo.hxx"
#include "LGParam.hxx"
#include <time.h>

/// statistics initialization
LGInfo::Status LGInfo::status = LGInfo::STOPPED;
LGInfo::Atomic LGInfo::nCalls = 0;
LGInfo::Atomic LGInfo::nAccepts = 0;
LGInfo::Atomic LGInfo::nSuccesses = 0;
LGInfo::Atomic LGInfo::nFails = 0;
LGInfo::Atomic LGInfo::avgSetupTime = 0;
LGInfo::Atomic LGInfo::minSetupTime = 0;
LGInfo::Atomic LGInfo::maxSetupTime = 0;
int *        LGInfo::start = 0;
LGInfo::Atomic LGInfo::failTypes[] = {0,0,0,0,0,0,0,0};
static char *  Reasons[] = {
    "INVITE_REJECTED",
    "INVITE_REJECTED_408",
    "INVITE_REJECTED_503",
    "BYE_REJECTED   ",
    "INVITE_TIMEOUT ",
    "BYE_TIMEOUT    ",
    "REGISTRATION_REJECTED",
    "REGISTRATION_TIMEOUT"
};
Mutex         LGInfo::lock;
#if USE_VMUTEX_LOCKING
#define ELLOCK lock.lock()
#define ELUNLOCK lock.unlock()
#else
#define ELLOCK
#define ELUNLOCK
#endif

void
LGInfo::callAccept(int timeDiff)
{
    ELLOCK;
    nAccepts++;
    if(!minSetupTime || minSetupTime > timeDiff) minSetupTime = timeDiff;
    if(maxSetupTime < timeDiff) maxSetupTime = timeDiff;
/**
 * three options here:
 * 1. accumulate timeDiff in ms - problem: timeDiff can be in secs, hence
 *    may overflow after large calls
 * 1.5 ----- use float!!!
 * 2. keep current avg - problem: needs to * w/ nAccepts, hence same as 1
 *    only w/ more work!
 * 3. acuumulate timeDiff in base "typical time" (~100ms?)
 *    [actually its ~10ms!]
 */
#define FACTOR 10                      // needed below in print method

    avgSetupTime += (timeDiff/FACTOR); // using 3 above
    ELUNLOCK;
}

void
LGInfo::callSuccess(void)
{
    ELLOCK;
    nSuccesses++;
    ELUNLOCK;
}

void
LGInfo::callFail(LGInfo::Cause cause)
{
    ELLOCK;
    failTypes[cause]++;
    nFails++;
    ELUNLOCK;
}

void
LGInfo::callStart()
{
    if(!start)
    {
	lock.lock();
	if(!start)
	{
	    start = new int(time(NULL));
	    status = LGInfo::RUNNING;
	}
	lock.unlock();
    }
    ELLOCK;
    nCalls++;
    ELUNLOCK;
}

void
LGInfo::stop()
{
    status = LGInfo::STOPPED;
}

void
LGInfo::print(ostream& os)
{
    int totSec  = (start? (time(NULL) - *start) : 0);

    os << "Test duration     = " << (totSec/3600) << "Hr : " <<
	((totSec%3600)/60) << "Min : " << (totSec%60) << "Sec ["
       << (status == LGInfo::RUNNING ? "RUNNING" : "STOPPED") << "]"
       << endl;
    os << "Calls/Sec [CPS]   = " << (totSec ? ((1.0*nCalls)/totSec) : 0)
       << endl;
    if(LGParam::Function != LGParam::TERMINATING)
	os << "Call Setup Rate   = " << (totSec ? ((1.0*nAccepts)/totSec) : 0)
	   << "/sec" << endl;
    os << "Setup time        = avg[" <<
	(nAccepts ? (avgSetupTime*FACTOR/nAccepts) : 0)
       << "ms]   min[" << minSetupTime << "ms]   max[" <<
	maxSetupTime << "ms]" << endl;

    float csr;
    //    if(LGParam::Function == LGParam::TERMINATING)
    //	csr=((nSuccesses+nFails) ? ((100.0*((float)nSuccesses))/
    //       ((float)(nAccepts))) : 0);
//    else
	csr=((nSuccesses+nFails) ? ((100.0*((float)nSuccesses))/
				       ((float)(nSuccesses+nFails))) : 0);

    os << "Call Success Rate = " << csr << "%" << endl;
    os << "Calls Attempted   = " << nCalls << endl;
    os << "Calls Completed   = " << (nSuccesses+nFails) << endl;
    os << "Calls Accepted    = " << nAccepts << endl;
    os << "Calls Successful  = " << nSuccesses << endl;
    os << "Calls Failed      = " << nFails << endl;
    if(nFails)
    {
	os << "Failure Reasons:\n";
	for(int i = LGInfo::INVITE_REJECTED; i<LGInfo::EOC; i++)
	{
	    if(failTypes[i])
		os << "\t" << Reasons[i] << " = " << failTypes[i] << "\n";
	}
	os << endl;
    }
}
