
/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */
static const char* const OSPIntfVersion = "$Id: OSPIntf.cxx,v 1.22 2001/01/20 01:33:06 wbreu Exp $";



#include <sys/types.h>
#include <stdio.h>

#include "cpLog.h"
#include "LockHelper.hxx"
#include "OSPErrStrings.hxx"
#include "PolicyTransaction.hxx"
#include "OSPIntf.hxx"
#include "OSPCfg.hxx"
#include "PolicyCfg.hxx"

// static singleton:
OSPIntf* OSPIntf::_ospIntfIns = 0;

///
OSPIntf& OSPIntf::Instance( void )
{
    if ( _ospIntfIns == 0)
    {
        _ospIntfIns = new OSPIntf;
        assert( _ospIntfIns );

    }
    return *_ospIntfIns;
}

void OSPIntf::destroy( void )
{
    cpLog( LOG_DEBUG, "OSPIntf::destroy() invoked ");
    delete OSPIntf::_ospIntfIns;
    OSPIntf::_ospIntfIns = 0;
}

void OSPIntf::lock( void )
{
    cpLog( LOG_DEBUG, "Locking OSPIntf" );
    _itsMutex.lock();
}

void OSPIntf::unlock( void )
{
    cpLog( LOG_DEBUG, "UnLocking OSPIntf" );
    _itsMutex.unlock();
}


///
OSPIntf::OSPIntf()
{
    cpLog(LOG_DEBUG, "OSPIntf ctor invoked");


    _ospProvider = 0;

    createProvider();

    // NOTE: new OSPCfg() to spawn an OSP Provider and 
    //       establish connection to the OSP server.

    // _ospCfg knows how to set up connection w/ OSP Server
    // it will call _ospProvider::connect() to est. connection
}


int
OSPIntf::createProvider()
{
    int errorcode = 0;
    
    if ((errorcode = OSPPInit()) != 0)
    {
        cpLog(LOG_ALERT,
              "OSPPInit failed, won't new OSPProvider, err = %s",
              OSPErrStrings::Instance().dumpErr( errorcode ));
        // should assert!
        return -1;
    }

    // NOTE: new OSPCfg() to spawn an OSP Provider and 
    //       establish connection to the OSP server.
    //
    // _ospCfg knows how to set up connection w/ OSP Server
    // it will call _ospProvider::connect() to est. connection
    OSPCfg& cfg = OSPCfg::Instance();  // get ready to setup the ospprovider
    OSPProvider * prov = new OSPProvider;
    assert( prov );

    const char ** servpts = (const char **)cfg.servicepoints;

    errorcode = 1;
    int loopCnt = 0;
    while ( true )
    {

        errorcode = prov->connect(
                        cfg.num_serv_points,
                        servpts,
                        (cfg.auditurl.c_str()),
                        (const OSPTPRIVATEKEY *) & cfg.privatekey,
                        /* Local certificate */
                        cfg.localcerts[0],
                        /* CA certificates */
                        cfg.NUM_CA_CERTS,
                        (const void **)cfg.certs,
                        cfg.localValidation,
                        cfg.sslLifeTime,
                        cfg.httpMaxConnections,
                        cfg.httpPersistence,
                        cfg.httpRetryDelay,
                        cfg.httpRetryLimit,
                        cfg.httpTimeout,
                        (cfg.custid.c_str()),
                        (cfg.devid.c_str()));

        cpLog(LOG_DEBUG, "connect() returns |%s|",
              OSPErrStrings::Instance().dumpErr(errorcode));
	if (errorcode == 0)
        {
           break;
        } 
        loopCnt++;
        cpLog(LOG_DEBUG, "OSPProvider::connect tyring.... attempt %d", loopCnt);
        if (loopCnt == 5)
        {
            cpLog(LOG_DEBUG, "OSPProvider::connect tyring.... giving up after %d attempts", loopCnt);
            break;
        }
        sleep(1);
    }

    _ospProvider = prov;
    return errorcode;
}

///
bool OSPIntf::operator==( const OSPIntf& rhs )
{
    if (( this->_ospProvider.getPtr() != 0) &&
            (this->_ospProvider.getPtr() == rhs._ospProvider.getPtr()))
        return true;
    else
        return false;
}   // OSPIntf::operator==

///
// Requirement: caller of findTrans should lock
//              _itsMutex before invoking findTrans()
bool OSPIntf::findTrans( const string& callId ) const
{
    bool ret = true;

    cpLog(LOG_DEBUG, "findTrans() cid %s", callId.c_str());

    ACTIVE_TRANS_MAP::const_iterator t = activeTrans.find(callId);
    if (t == activeTrans.end())
    {
        ret = false;
    }
    else
    {
        cpLog( LOG_DEBUG, "found transaction %s", callId.c_str());
    }
    return ret;

}   // OSPIntf::findTrans

///
void OSPIntf::sendReq( OSPAuthInfo& ai, PolicyTransaction& pol ) throw (
    OSPException &)
{
    cpLog(LOG_ALERT, "sendReq() invoked");

    if (PolicyCfg::Instance().isOspDataChanged())
    {
        cpLog(LOG_ALERT, "***** OSPIntf::sendReq, policy CFG changed");
        OSPCfg::Instance().getServerConfiguration();
        _ospProvider->setHttpMaxConnections(OSPCfg::Instance().httpMaxConnections);
        _ospProvider->setHttpPersistence(OSPCfg::Instance().httpPersistence);
        _ospProvider->setHttpRetryDelay(OSPCfg::Instance().httpRetryDelay);
        _ospProvider->setHttpRetryLimit(OSPCfg::Instance().httpRetryLimit);
        _ospProvider->setHttpTimeout(OSPCfg::Instance().httpTimeout);
        _ospProvider->setLocalValidation(OSPCfg::Instance().localValidation);
        _ospProvider->setSslLifetime(OSPCfg::Instance().sslLifeTime);
        PolicyCfg::Instance().ospDataChanged(false);
    }

    if ((_ospProvider == 0) ||
            (_ospProvider->isProviderReady() == false))
    {
	string errstr = "sendReq: _ospProvider is not well formed";
        cpLog(LOG_ALERT, errstr.c_str());
	throw OSPException( errstr, __FILE__, __LINE__, 0 );
    }

    OSPTrans * aTrans = 0;
    int errCode = 0;

    aTrans = new OSPTrans( this->_ospProvider->getHandle() );
    if (aTrans == 0)
    {
        throw OSPException( "sendReq: fails to new a transaction",
                            __FILE__, __LINE__, 0 );
    }

    Sptr<OSPAuthRespInfo> ar = new OSPAuthRespInfo;
    assert( ar != 0 );

    errCode = aTrans->sendAuthReq( ai, *ar );
    ar->setStatus( errCode );     // status has not been update yet
    ar->setTid( aTrans->get_transactionID() );

    if (errCode != 0 )
    {
        pol.setID( ar->getTid() );
        pol.setStatus( ar->getStatus() );

        delete aTrans;    // don't keep it around
    }
    else
    {
        // update pol with the info found in the authorization response:
        pol.setID( ar->getTid() );
        pol.setStatus( ar->getStatus() );
        // wbr: 5/29/00 - leave call Id alone - should not be changed
        // pol.setCallId( ar->getCallId() );
        pol.setValidAfterTs( ar->getValidAfterTs() );
        pol.setValidUntilTs( ar->getValidUntilTs() );
        pol.setDuration( ar->getTimeLimit() );

        // convert binary token into base 64 encoding:
        unsigned int tsize = 0;
        cpLog(LOG_DEBUG, "calling encodeToken");
        if (ar->encodeToken() == true)
        {
            tsize = ar->getB64Token().length();
            pol.setTokenSize( tsize );
            pol.setToken( ar->getB64Token().c_str(), tsize );
            cpLog(LOG_DEBUG,
                  "encodeToken() OK: binary token size(%u), b64 token size(%u)", ar->getTokenSize(), tsize);
        }

#if 0
        // for unit testing, copy token into a file:
        FILE *out_file = fopen( "./authToken.out", "w");
        if (out_file == 0)
        {
            cpLog( LOG_ALERT, "failed to open ./authToken.out " );
        }
        else
        {
            int count = fwrite( ar->getB64Token().c_str(), 1, tsize, out_file );
            cpLog( LOG_DEBUG, "saved token to file, count = %d, tokenSize =%d",
                   count, tsize );
            fclose( out_file );
        }
#endif

        // add new transaction to the active transaction map:
        {
            string cid = pol.getCallId();
            LockHelper lockAid(_itsMutex);
            activeTrans[ cid ] = aTrans;
        }
    }


}   // OSPIntf::sendReq()


///
void OSPIntf::sendValReq( OSPValInfo& vi, PolicyTransaction& pol ) throw (
    OSPException &)
{
    cpLog(LOG_DEBUG, "sendValReq() invoked");

    if ((_ospProvider == 0) ||
            (_ospProvider->isProviderReady() == false))
    {
	string errstr = "sendValReq: _ospProvider is not well formed";
        cpLog(LOG_ALERT, errstr.c_str());
        throw OSPException( errstr,
                            __FILE__, __LINE__, 0 );
    }

    OSPTrans * aTrans = 0;
    int errCode = 0;

    aTrans = new OSPTrans( this->_ospProvider->getHandle() );
    if (aTrans == 0)
    {
        throw OSPException( "sendValReq: fails to new a transaction",
                            __FILE__, __LINE__, 0 );
    }

    vi.dump();
    errCode = aTrans->sendValReq( vi );
    pol.setID( aTrans->get_transactionID() );
    pol.setStatus( errCode );

    if (errCode == 0 )
    {
        // update pol with the info updated in the OSPValInfo:

        pol.setDuration( vi.getTimeLimit() );
        pol.setValidationSts( vi.getAuthFlag() );

        // add new transaction to the active transaction map:
        {
            string cid = pol.getCallId();
            LockHelper lockAid(_itsMutex);
            activeTrans[ cid ] = aTrans;
        }
    }
    else
    {
        delete aTrans;    // don't keep it around
    }

}   // OSPIntf::sendValReq()

///
void OSPIntf::sendUsage( OSPUsageInfo & ui, PolicyTransaction& pol ) throw (
    OSPException &)
{
    if ((_ospProvider == 0) ||
            (_ospProvider->isProviderReady() == false))
    {
        string buf = "sendUsage: _ospProvider is not ready";
        cpLog(LOG_ALERT, buf.c_str());
	throw OSPException( buf, __FILE__, __LINE__, 0 );
    }

    int errCode = -1;
    string callId = pol.getCallId();
    OSPTransactionHandle tid = ui.getID();     // get transaction id

    if (tid == OSPInvalidTransHandle)
    {
        cpLog(LOG_DEBUG, "Invalid id, %d, from Usage Report", tid);
    }
    else
    {
        LockHelper lockAid( _itsMutex );
        ACTIVE_TRANS_MAP::iterator t = activeTrans.find(callId);
        if (t == activeTrans.end())
        {
	    char buf[512];
	    sprintf(buf, "sendUsage: cannot find transaction id %d for callid %s",
                    tid, callId.c_str() );
            cpLog(LOG_ALERT, buf);
            throw OSPException( buf, __FILE__, __LINE__, tid );
        }
        else   // found transaction
        {
            //
            // get call duration from policy transaction:
            // (don't care about the loss packets/fraction statistics for now)

            // send Usage Report to OSP Server
            errCode = t->second->sendUsage( ui );

            // update result in policy Transaction:
            pol.setStatus( errCode );
            if (errCode != 0)
            {
                cpLog(LOG_DEBUG, "sendUsage error = |%s|",
                      OSPErrStrings::Instance().dumpErr( errCode ));
            }

            // remove active Transaction from activeTrans map:
            activeTrans.erase( t );
        }
        // end locking
    }

}   // OSPIntf:sendUsage()

///
int OSPIntf::abandonTrans( const string & reason, PolicyTransaction & pol )
{
    int id = pol.getID();     // get transaction id
    int errCode = -1;
    const char* cid = pol.getCallId();

    cpLog(LOG_DEBUG, "abandonTrans() - id = %d, callId = %s",
          pol.getID(), pol.getCallId());

    if ((_ospProvider == 0) ||
            (_ospProvider->isProviderReady() == false))
    {
        cpLog(LOG_ALERT, "abandonTrans: _ospProvider is not ready");
        return -1;
    }

    Sptr < OSPTrans > aTrans = 0;

    //locate transaction from the active transaction map
    {   //**************** begin locking:
        LockHelper lockAid( _itsMutex );
        if (findTrans( cid ) == false)
        {
            cpLog(LOG_DEBUG,
                  "cannot locate tranaction, tid = %d, callid = %s in abandonTrans()",
                  id, cid );
        }
        else
        {
            // map reason of failure
            int failReason = OSPC_FAIL_REMOTE_NORESP;
            aTrans = activeTrans[cid];
            // record the failure reason
            errCode = aTrans->abandonTrans(failReason);
        }
        //***************** end locking ******************
    }

    // send a 0 usage report to get the transaction in the right
    // state, otherwise, the OSP transaction (in the SDK) is
    // not allowed to be deleted:

    Sptr<OSPUsageInfo> u = new OSPUsageInfo( id, 0, 0, -1, 0, -1 );
    assert( u != 0);
    sendUsage( *u, pol);

    return ( 0 );

}   // OSPIntf::abandonTrans()

// end of file
