
/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */
static const char* const OSPIntf_hxx_Version =
    "$Id: OSPIntf.hxx,v 1.14 2001/01/20 01:33:06 wbreu Exp $";
#ifndef OSPINTF_HXX
#define OSPINTF_HXX




#include <sys/types.h>
#include <map>

#include "VMutex.h"

#include "osp.h"
#include "OSPDefines.hxx"
#include "OSPException.hxx"
#include "OSPProvider.hxx"
#include "PolicyTransaction.hxx"
#include "OSPAuthInfo.hxx"
#include "OSPValInfo.hxx"
#include "OSPTrans.hxx"
#include "OSPTrans.hxx"
#include "Sptr.hxx"


/** OSP Interface object for the OSP client (C toolkit)
 */
class OSPIntf
{

    public:
        /**  Active Transaction Table keyed by call id:
	 */
        typedef map < string, Sptr < OSPTrans > > ACTIVE_TRANS_MAP;

        /**
         * The constructor instantiates an OSPCfg to handle the
         * OSP configuration requirements, and an OSPProvider
         * object.
	 */
        static OSPIntf& Instance( void );

        /** self-destruct at program exits:
	 */
        static void destroy( void );

	/** add mutexing
	 */
        void lock( void );

	/** remove mutexing
	 */
        void unlock( void );

	/** setProvider checks if _ospProvider is well formed before
	 *  this OSPIntf object can be used
	 */  
        void setProvider( Sptr < OSPProvider > ospp ) throw (OSPException &)
        {
            assert( ospp.getPtr() );
            if ( _ospProvider.getPtr() != 0)
            {
                throw OSPException( "OSP Provider already exists",
                                    __FILE__, __LINE__, 0 );
            }
            // required ospp to be in the Ready state:
            if (ospp->isProviderReady() == false)
            {
                throw OSPException( "OSP Provider is not well formed",
                                    __FILE__, __LINE__, 0 );
            }
            _ospProvider = ospp;
        }

        /** Equal operator
	 */
        bool operator==( const OSPIntf& rhs );

        /** send Authorization Request
	 */
        void sendReq(OSPAuthInfo & ai, PolicyTransaction& pol ) throw (
            OSPException& );

        /**  send Authorization Validation Request
	 */
        void sendValReq(OSPValInfo & vi, PolicyTransaction& pol ) throw (
            OSPException& );

        /** send Usage Indication
	 */
        void sendUsage( OSPUsageInfo & ui, PolicyTransaction& pol ) throw (
            OSPException& );

        /** record the failure reason in the transaction, then send
	 *  a 0 usage report to OSP server, and remove transaction from
         * the active transaction map:
	 */
        int abandonTrans( const string & reason, PolicyTransaction & pol );

    protected:
	/** constructor
	 */
        OSPIntf();
	
        /// destructor
        virtual ~OSPIntf() {}

        /**    locate a transaction in the activeTrans map give a
	 *     unique call id
	 */
        bool findTrans( const string& callId ) const;

    private:

        /** instanstatiate OSP client: get an OSP handle from
	 *  OSP toolkit
	 */
        int createProvider();

        /** make obj of this class thread safe
	 */
        VMutex _itsMutex;

        /** copy constructor - illegal for use
	 */
        OSPIntf( const OSPIntf& rhs );
         
        /** assignment operator - illegal for use
	 */
        OSPIntf& operator=( const OSPIntf& rhs );

        /** smart pointer containing pointer to OSP client object,
	 *  _ospProvider
	 */
        Sptr < OSPProvider > _ospProvider;

        /** associative array of all outstanding OSP transactions.  Each
         *  transaction has a unique transactionID assigned by the OSP
         *  Provider.
	 */
        ACTIVE_TRANS_MAP activeTrans;

        /** static pointer to OSPIntf singleton object
	 */
        static OSPIntf* _ospIntfIns;
};


#endif
