
/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */
static const char* const OSPToken_cxx_Version =
    "$Id: OSPToken.cxx,v 1.11 2001/01/20 01:33:06 wbreu Exp $";
#include "osp.h"
#include "ospdatatypes.h"
#include "ospb64.h"
#include "cpLog.h"
#include "OSPToken.hxx"



/// class usage in libOSP:
//  During Auth. request/ response, the token is returned in the
//  OSPAuthRespInfo object.  This token is in the binary format.
//  This method is called to convert the binary token into a base64
//  token (stored in the Policy Transaction) for safer carriage over
//  the SIP message.
//
//  000906: make b64 token as a quoted string for token carriage
//          over the SIP INVITE header
bool OSPToken::encodeToken( const unsigned char* btok, unsigned int btokSize,
                            string& b64TokenStr )
{
    if ((btok == 0) || (btokSize <= 0))
    {
        return false;
    }
    unsigned int b64TokenSize = (btokSize * 2) + 2;
    char* b64Token= new char[ b64TokenSize ];
    unsigned int b64TokSize = b64TokenSize;    // will be updated w/ real size
    memset( b64Token, 0, b64TokenSize );
    int err = OSPPBase64Encode( btok, btokSize,
                                reinterpret_cast < unsigned char* > (b64Token), &b64TokSize );
    if (err == OSPC_ERR_NO_ERROR )
    {
        // put token between double quotes:
        b64TokenStr = '\"';
        b64TokenStr.append( b64Token, b64TokSize );
        b64TokenStr += "=\"";      /* just in case */

        cpLog(LOG_DEBUG, "Base 64 Encoded token (size=%d ) = \n%s",
              b64TokSize, b64TokenStr.c_str() );
       delete [] b64Token;
       return true;
    }
    cpLog(LOG_ERR, "Unable to base64 encode token, err = %d",
          err );
    delete [] b64Token;
    return false;

}

///  convert a base 64 encoded token into binary data:
//   Usage: When an incoming SIP INVITE is received, the OSP
//   token is extracted and placed in a COPS message for OSP
//   valadition.  Remember this token is in previously encoded
//   in base64 format, therefore, before token validation, it is
//   decoded back in a binary form (and set in the OSPValInfo object.
//
//   000906: check if token is a quoted string, if so, strip quotes
//           before decoding.
bool OSPToken::decodeToken( const char* b64Token,
                            unsigned int b64TokenSize, string& btok )
{
    if ((b64Token == 0) || (b64TokenSize == 0))  // empty token
        return false;

    unsigned int tokenSize = b64TokenSize;
    unsigned int uqTokenSize = b64TokenSize;    // size of unquoted token
    const char* startToken = b64Token;
    char*  token= new char[ tokenSize + 1 ];

    /* if token is embedded in quotes, strip them first */
    if (b64Token[0] == '\"')
    {
        startToken = &(b64Token[1]);     // skip beginning quote
        uqTokenSize = b64TokenSize - 2;

    }
    memset( token, 0, tokenSize + 1 );
    int err = OSPPBase64Decode(
                  startToken, uqTokenSize,
                  reinterpret_cast < unsigned char* > (token), &tokenSize );
    if (err == OSPC_ERR_NO_ERROR )
    {
        btok.assign( token, tokenSize );
        cpLog(LOG_DEBUG, "Base 64 token decoded, tokenSize=%u (len=%d )",
              tokenSize, btok.length() );
        delete [] token;
        return true;
    }
    cpLog(LOG_ERR, "Unable to decode  base64 token, err = %d",
          err );
    delete [] token;
    return false;

}
