
/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */
static const char* const ospTranCxxVersion = "$Id: OSPTrans.cxx,v 1.12 2001/01/20 01:33:06 wbreu Exp $";
#include <stdio.h>
#include <fcntl.h>

#include "global.h"
#include <iostream>
#include <string>
#include <cassert>

#include "cpLog.h"
#include "osptransapi.h"     // header file to interface w/ osplib
#include "OSPErrStrings.hxx"
#include "OSPTrans.hxx"

// associate this OSPTransaction with a specific OSPProvider
OSPTrans::OSPTrans( int pH )
{
    int errCode = 0;

    cpLog(LOG_DEBUG, "OSPTrans ctor, provider handlle = %d", pH);

    _transactionId = OSPInvalidTransHandle;
    _providerHandle = 0;
    _maxSize = 0;      /* max size of _detailLog */
    _logSize = 0;      /* actual size of trans log entered */
    _detailLog = 0;   /* storage for transaction details */

    if ((errCode = OSPPTransactionNew( pH, &_transactionId)) == 0)
    {
        _providerHandle = pH;
    }
    

    cpLog(LOG_DEBUG,
          "OSPTrans ctor, providerHandle=%d, transId=%d, err=%s",
          pH, _transactionId,
          OSPErrStrings::Instance().dumpErr(errCode));
}

OSPTrans::~OSPTrans()
{
    int errCode = 0;

    if (_detailLog) delete [] _detailLog;

    // call osplib OSPPTransactionDelete();
    if ((errCode = OSPPTransactionDelete(this->_transactionId)) != 0)
    {
        // this could really happen. e.g. program exits causing
        // OSPIntf::destroy() to annihilate the active transaction
        // map.  However, depending on the transaction state, the
        // SDK may reject the OSPPTransactionDelete().
        cpLog(LOG_ALERT,
              "Memory Leak: failure to delete OSP transactionId=%d, errCode=%d",
              _transactionId, OSPErrStrings::Instance().dumpErr(errCode));
    }
    cpLog(LOG_DEBUG, "OSPTrans for providerHandle=%d deleted", _providerHandle);
}

bool OSPTrans::operator==( const OSPTrans& rhs )
{
    if ((this->_providerHandle == rhs._providerHandle) &&
            (this->_transactionId == rhs._transactionId))
        return true;
    else
        return false;
}

int OSPTrans::sendAuthReq( OSPAuthInfo& authReq, OSPAuthRespInfo& authResp )
{
    int errCode = 0;
    unsigned int numDest = authReq.getNumDest();
    OSPCallIdList * cl_ptr = authReq.getCallIds();

    resetLog();
    errCode = OSPPTransactionRequestAuthorisation( _transactionId,
              authReq.getSource(), authReq.getSourceDevice(),
              authReq.getCallingNum(), authReq.getCalledNum(),
              authReq.getUser(), authReq.getNumOfCallIds(),
              &cl_ptr, authReq.getPreferDests(),
              &numDest, &_logSize, (void *) _detailLog );

    // application wishes to store a copy of the authorization
    // response obtained from the OSP Server:
    if (_detailLog && _logSize > 0)
    {
        cpLog(LOG_DEBUG, "sendAuthReq detail log, size=%d:\n |%s|",
              _logSize, _detailLog);
    }

    cpLog(LOG_DEBUG, "sendAuthReq returns = %s",
          OSPErrStrings::Instance().dumpErr( errCode ));

    if (errCode)
    {
        cpLog(LOG_ALERT, "sendAuthReq error = %s",
              OSPErrStrings::Instance().dumpErr( errCode ));
    }
    else   // RequestAuthorization is sent successfully:
    {
        authReq.setNumDest( numDest );

        // retrieve info. for first authorized destination

        errCode = OSPPTransactionGetFirstDestination( _transactionId,
                  authResp.getTimeStampSize(),
                  authResp.getValidAfterTs(), authResp.getValidUntilTs(),
                  authResp.getTimeLimitLoc(),
                  authResp.getCallIdSizeLoc(), authResp.getCallId(),
                  authResp.getCalledNumSize(), authResp.getCalledNum(),
                  authResp.getDestSize(), authResp.getDestination(),
                  authResp.getDestDeviceSize(), authResp.getDestDevice(),
                  authResp.getTokenSizeLoc(), authResp.getToken() );
        cpLog(LOG_DEBUG,
              "OSPPTransactionGetFirstDestination returns = %s",
              OSPErrStrings::Instance().dumpErr( errCode ));
    }

    return errCode;
}

/// For incoming call, validate authorization token carried
//  in the SIP invite message:
int OSPTrans::sendValReq( OSPValInfo& info )
{
    int errCode = 0;
    unsigned int authSts = 0;    // default to unauthorized
    unsigned int timeLimit = 0;  // 0 = no time limit exists

    resetLog();
    errCode = OSPPTransactionValidateAuthorisation( _transactionId,
              info.getSource(), info.getDestination(),
              info.getSourceDevice(), info.getDestDevice(),
              info.getCallingNum(), info.getCalledNum(),
              info.getCallIdSize(), info.getCallId(),
              info.getTokenSize(), static_cast < const void * > (info.getToken()),
              &authSts, &timeLimit,
              &_logSize, (void *) _detailLog );

    // application wishes to store a copy of the authorization
    // response obtained from the OSP Server:
    if (_detailLog && _logSize > 0)
    {
        cpLog(LOG_DEBUG, "sendValReq detail log, size=%d:\n |%s|",
              _logSize, _detailLog);
    }

    cpLog(LOG_DEBUG, "OSPPTransactionValidateAuthorization returns = %s",
          OSPErrStrings::Instance().dumpErr( errCode ));

    if (errCode)
    {
        cpLog(LOG_ALERT, "Callid = %s, transactionId: %d ", info.getCallId(), _transactionId);
        cpLog(LOG_ALERT, "sendValReq error = %s",
              OSPErrStrings::Instance().dumpErr( errCode ));
    }
    else   // validateAuthorization is sent successfully:
    {
        info.setAuthFlag( authSts );
        info.setTimeLimit( timeLimit );
        cpLog(LOG_DEBUG,
              "Token authorization status = %d, timeLimit = %d",
              info.getAuthFlag(), info.getTimeLimit() );
    }
    return ( errCode );
}  // sendValReq()



int OSPTrans::sendUsage( const OSPUsageInfo& usage )
{
    int errCode = 0;

    errCode = OSPPTransactionReportUsage(
                  get_transactionID(),
                  usage.getDuration(),
                  usage.getLossPacketSent(), usage.getLossFractionSent(),
                  usage.getLossPacketReceived(), usage.getLossFractionReceived(),
                  &_logSize, (void *) _detailLog );

    return errCode;
}


// record failure of a call attempt for the current destination
int OSPTrans::abandonTrans( int reason )
{
    int errCode = 0;
    cpLog(LOG_DEBUG, "abandonTrans() - id = %d", get_transactionID());
    errCode = OSPPTransactionRecordFailure( get_transactionID(),
                                            static_cast < enum OSPEFAILREASON > (reason) );
    if (errCode != 0)
    {
        cpLog(LOG_ALERT, "abandonTrans() error = %d", errCode );
    }
    else
    {
        cpLog(LOG_DEBUG, "abandonTrans() OK");
    }

    return errCode;
}



