
/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */
#ifndef OSPTRANS_HXX
#define OSPTRANS_HXX

static const char* const opsTranshxxVersion = "$Id: OSPTrans.hxx,v 1.10 2001/01/20 01:33:06 wbreu Exp $";



#include <sys/types.h>
#include "osp.h"
#include "OSPDefines.hxx"
#include "OSPAuthInfo.hxx"
#include "OSPValInfo.hxx"
#include "OSPAuthRespInfo.hxx"
#include "OSPUsageInfo.hxx"


/// C++ wrapper for an OSP Transaction object
class OSPTrans
{

    public:
	/** constructor.
         *  associate this OSPTransaction with a specific OSPProvider
	 *  call osplib OSPPTransactionNew() and init _transactionID
	 */
        OSPTrans(int pH);

        /** destructor.
	 * call osplib OSPPTransactionDelete();
	 */
        virtual ~OSPTrans();

        ///
        bool operator==( const OSPTrans& rhs );

        /// retrieve transaction handle:
        OSPTransactionHandle get_transactionID( void )
        {
            return _transactionId;
        }

        /** 
         *   For origination system:
         *   send authorizationRequest msg. to OSP Server and
         *   return the first authorized destination
         */
        int sendAuthReq( OSPAuthInfo& authReq, OSPAuthRespInfo& authResp );

        /**
         *  For derminating system:
         *  send validateAuthorizationRequest msg. to OSP Server
         *  expect authorization status (0 = unauth >0 = auth)
         *     and time limit (0 = no time limit exists)
	 */
        int sendValReq( OSPValInfo& info );

        ///  send Usage Indication message to OSP Server
        int sendUsage( const OSPUsageInfo& usage );

        /// record failure of a call attempt for the current destination
        int abandonTrans( int reason );

	// For OSP v2.1
        // refresh a previously granted authorization (maybe
        // due to authorization is nearing its expiration)
        // applicable in many calling card services
        // int sendReAuthReq( const ospReAuthReqInfo reAuthReq );

        // For destination system (OSP v2.1):
        // validate a requested incoming call, based on the
        // call's info and autorization token.  Depending on
        // the number of authorization tokens a call request
        // may have, this method may be called mutliple times,
        // one for each auth token in the request
        // If Local Validation is configured at the provider,
        // then the provider validates the authorization without
        // network interaction.
        // int sendAuthVal( const ospAuthValInfo authVal );

	// For OSP v2.1:
        // re-validate an existing call, perhaps in order to
        // increase the time limit for the call, or applications
        // may invoke this method when an updated authorization
        // token is received for the call.
        // int sendReAuthVal( const ospReAuthValInfo reAuthVal);

        /** set up transaction logging.
         *  if detailed log for the transaction is desired:
         *  (currently not supported by current version of SDK,
         *   v2.5.2 )
	 */
        int setUpLog( int size )
        {
            if (size > 0)
            {
                if (_detailLog)
                {
                    delete [] _detailLog;
                }
                _detailLog = new char[ size ];
                assert( _detailLog );
                memset( _detailLog, 0, size );
                _maxSize = _logSize = size;
                cpLog(LOG_DEBUG, "setUpLog: transId=%d, setUpLog( %d bytes)",
                      _transactionId, _logSize);
                return 0;
            }
            return -1;
        }

        /// clear the log entry and reset _logSize
        void resetLog( void )
        {
            if (_detailLog)
            {
                memset( _detailLog, 0, _maxSize );
                _logSize = _maxSize;
            }
        }


    protected:
	/// max size of detailed log
        unsigned int _maxSize;     
	/// actual size of detailed log
        unsigned int _logSize;        
	/// pointer to detailed log (updated by OSP SDK)
        char * _detailLog;

    private:
        /// not used: default constructor
        OSPTrans();
         
        /// not used: copy constructor
        OSPTrans( const OSPTrans& rhs );
         
        /// not used: assignment constructor
        OSPTrans& operator=( const OSPTrans& rhs );

        /// OSP Provider handle - used to communicate w/ SDK
        int _providerHandle;
        /// OSP transaction id - used to communicate w/ SDK
        OSPTransactionHandle _transactionId;


};

#endif
