#ifndef POLICYTRANSACTION_HXX
#define POLICYTRANSACTION_HXX

/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */
static const char* const PolicyTransaction_hxx_Version =
    "$Id: PolicyTransaction.hxx,v 1.15 2001/01/20 01:33:06 wbreu Exp $";




#include <time.h>
#include <string>

#include "md5String.hxx"
#include "PolicyBadDataException.hxx"

/** Policy Transaction object used to exchange information
 *  between COPS and OSP messages
 */
class PolicyTransaction
{

    public:
        /// contructor
        PolicyTransaction(void);
        /// destructor
        virtual ~PolicyTransaction();

        /** concertrate data of this object to a c-style string:
         *  "CallId||TransactionId|123|....\n"
	 */
        string encode( void ) const;

        /** given a string object encoded in the above manner,
         *  reconstitue each keyword/value pairs and set up data in
         *  this object
	 */
        void decode( const string & str ) throw (PolicyBadDataException &);

        /*   Accessors and Mutators: */

	/// access transaction ID
        int getID( void ) const
        {
            return ( _Id );
        }
        /// set transaction ID
        void setID( int id )
        {
            _Id = id;
        }

        /// get transaction status
        int getStatus( void ) const
        {
            return ( _status );
        }
        /// set transaction status
        void setStatus( int sts )
        {
            _status = sts;
        }

        /// get unique call ID
        const char * getCallId( void )
        {
            return ( _callId.c_str() );
        }
        /// set call ID
        void setCallId( const char * ci )
        {
            assert( ci );
            _callId = ci;

            // workaround: always map callId to a 16-bytes md5String
            md5String md5Str;
            _md5CallId = md5Str.getMd5String( _callId );
        }

        /// get length of MD5 call ID
        int getMd5CallIdLen( void ) const
        {
            return ( _md5CallId.length() );
        }

        /// get MD5 digest of the call ID
        const char * getMd5CallId( void )
        {
            return ( _md5CallId.c_str() );
        }

        /// convert an OSP timestamp string to time in seconds since epoch
        time_t getTm( const char * ts ) const;

        ///  get "valid after" timestamp string:
        const char * getValidAfterTs( void )
        {
            return ( _validAfterTs.c_str() );
        }

        ///  get "valid after" time in seconds since epoch
        time_t getValidAfterTm( void )
        {
            return ( getTm( getValidAfterTs() ) );
        }

        /// set "valid after" time stamp
        void setValidAfterTs( const char * ts )
        {
            assert( ts );
            _validAfterTs = ts;
        }

        /// get "valid til" timestamp string:
        const char * getValidUntilTs( void )
        {
            return ( _validUntilTs.c_str() );
        }

        /// get "valid til" time in seconds since epoch
        time_t getValidUntilTm( void )
        {
            return ( getTm( getValidUntilTs() ));
        }

        /// set "valid til" timestamp string
        void setValidUntilTs( const char * ts )
        {
            assert( ts );
            _validUntilTs = ts;
        }

        /// get call duration in seconds
        unsigned int getDuration( void ) const
        {
            return ( _duration );
        }

        /// set duration
        void setDuration( unsigned int d )
        {
            _duration = d;
        }

        /// get size of authorization token
        unsigned int getTokenSize( void ) const
        {
            return ( _tokenSize );
        }

        /// set token size
        void setTokenSize( unsigned int s )
        {
            _tokenSize = s;
        }

        /// for C-style string token
        const char * getToken( void )
        {
            return ( _token.c_str() );
        }

	/// get _token string obj
        const string& getRawToken( void )
        {
            return ( _token );
        }

        // set C-style string as token
        void setToken( const char * tok )
        {
            _token = tok;
        }

        // for non C-style string, OK to have \0 in midst of string content:
        void setToken( const char * tok, unsigned int size )
        {
            _token.assign( tok, size);
        }

	/// get validation status
        unsigned int getValidationSts( void ) const
        {
            return (_valSts);
        }

	/// set validation status
        void setValidationSts( unsigned int sts )
        {
            _valSts = sts;
        }

        /**
         * Debug only:
	 */
        void print( void );

    private:
        /// equal operator - illegal to use
        bool operator==( const PolicyTransaction& rhs );
         
        /// copy constructor - illegal to use
        PolicyTransaction( const PolicyTransaction& rhs );
         
        /// assignment operator - illegal to use
        PolicyTransaction& operator=( const PolicyTransaction& rhs );

        /// transaction id
        int _Id;    // unique id (same as transactionID for OSP SDK)

        /// transaction status (error code)
        int _status;    // = errCode

	/// unique call id assoc'd w/ a call 
        string _callId;    // unique call id associated w/ the call

	/// call is authorized from this UTC timestamp
        string _validAfterTs;    // call is authorized from this UTC timestamp

	/// call is authorized until this UTC timestamp
        string _validUntilTs;    // call is authorized til this UTC timestamp

	/// call duration
        unsigned int _duration;      // available time limit in seconds

	/// size of OSP auth. token
        unsigned int _tokenSize;     // size of the authorization token

	/// H.235 security token in B64 format
        string _token;             // H.235 security token in B64 value

	/** validation status of token on incoming side
         * 0  = auth token not authorized
         * >0 = auth token authorized
	 */
        unsigned int _valSts;        // for incoming end: validation status


        /** work around for the TransNexus ClearIP OSP Server:
         *  no set method on purpose
         *  problem: SIP callId length is variable and unlimited, this
         *           causes OSP Server to return 400 bad request 
         *  fix:     masquerade the callId as a 16-bytes MD5 digest
         *           in OSP land
	 *           (this is not a problem w/ the SUN/Solaris OSP
	 *           Server)
	 */
        string _md5CallId;


};


#endif
