#ifndef PepAgent_hxx
#define PepAgent_hxx

/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */



static const char* const PepAgent_hxx_Version =
    "$Id: PepAgent.hxx,v 1.24.22.1 2002/05/15 01:24:05 sprajpat Exp $";

#include <string>

#include <VMutex.h>
#include "VNetworkException.hxx"
#include "VMissingDataException.hxx"
#include "VDuplicateIdException.hxx"
#include "Sptr.hxx"
#include "NetworkAddress.h"
#include "VThreadPool.hxx"
#include "PolRequest.hxx"
#include "Connection.hxx"
#include "Fifo.h"
#include "COPSData.hxx"

class COPSMsg;
class COPSReport;
class SipMsg;

/**Function prototype, a client would define the callback function as.
   PolRequest: The PolicyRequest object that holds the response
*/
typedef void (*PolCallBackFunc)(PolRequest& polReq);


/**
    PepAgent represents a COPS PEP.Responsible for making
    connection to the PDP and maintaining it
*/
class PepAgent
{
    public:
        static int PDPPort;
        ///
        static PepAgent& instance(const char* psHost = 0, int psPort = 0,
                                  const char* psAltHost = 0, int psAltPort = 0,
                                  int nThreads = 8);

        ///
        string className()
        {
            return "PepAgent";
        }

        ///
        void setClientInfo(const string& cHost, int cPort, COPS_ClientType cType = C_SIP);

        ///
        virtual ~PepAgent();

        ///
        void auth(const string& calledNumber,
                  const string& callingNumber,
                  const string& callId) throw (VDuplicateIdException&);

        ///
        void validateToken(const string& calledNumber,
                           const string& callingNumber,
                           const string& callId, const COPSData& token)
        throw (VDuplicateIdException&);

        ///
        void enableQoS(const string& callId,
                       const string& srcIp, int srcPort,
                       const string& destIp, int destPort);

        ///
        void disableQoS(const string& callId);

        ///
        void registerCallback(PolCallBackFunc func)
        {
            _callBackFunc = func;
        }

        ///
        string getDomainName() const
        {
            return _domainName;
        };

        ///
        string getHostName() const
        {
            return _hostName;
        };

        ///
        PolRequest& getPolReq(const string& callId) throw (VMissingDataException&) ;
        ///
        PolRequest& getPolReq(const string& calledNumber,
                              const string& callingNumber,
                              const string& callId) throw (VMissingDataException&) ;


        ///
        int getResponseTime() const
        {
            return _responseTime;
        };

        ///Set response time in seconds
        void setResponseTime(int respTime)
        {
            _responseTime = respTime;
        };

        ///
        int sendSyncWithRetries(const PolRequest& req, int nRetries = 3);

    protected:
        ///
        PepAgent(const char* psHost = 0, int psPort = 0,
                 const char* psAltHost = 0, int psAltPort = 0,
                 int nThreads = 8);

        ///
        Connection& getConnection(const string& hashKey);

        ///
        void notifyClient(PolRequest& pReq);

        ///
        void processCOPSMsg(const COPSMsg& msg, Connection& conn, const string& hashKey = "");

        ///
        void handleReportMsg(const COPSReportMsg& msg);

        ///
        void processData(Connection& conn, const string& hashKey = "");

        ///
        void checkForData();

        ///
        void checkForData(Connection& conn);

        ///
        void connect(const string& hashKey) throw (VNetworkException&);

        /**
           Frees the memory associated with singelton instance.
           gets register to atexit() function at the time of creation.
        */
        static void destroy(void);


        ///
        void queueForSend(PolRequest& req);
        ///
        void queueForResponse(PolRequest& req);
        ///
        void queueForUsage(PolRequest& req);

        ///
        void cleanUp(const PolRequest& req);

        ///
        static void garbageCollectFunc(void* args);

        ///
        static void sendPolicyRequest(void* args);

        ///
        static void sendUsage(void* args);

        ///
        static void checkUsageQueue(void* args);

        ///
        static void checkResponseQueue(void* args);

        ///
        static void checkSendQueue(void* args);

        ///
        string getConnectionLookupKey(const string& hashKey) const;

        ///
        string getActivePdp(const string& hashKey) const;

        ///
        static PepAgent* _instance;
	///
        Sptr < VThreadPool > _threadPool;

	///
        typedef map < COPSData, Sptr < PolRequest > > PolReqMap;
	///
        PolReqMap _reqMap;
	///
        Vocal::Fifo < PolRequest* > _sendQueue;
	///
        Vocal::Fifo < PolRequest* > _responseQueue;
	///
        Vocal::Fifo < PolRequest* > _usageQueue;

        //The key to the map would be a string containing
        //the threadid added to the pdpServer name that it need
        //to server "tid+<pdpHost>"

        ///MAP of connections to PDP
        typedef map < string, Connection > ConnectionMap;
	///
        ConnectionMap _connectionMap;

	///
        VMutex  _lock;
	///
        VMutex  _connlock;


	///
        u_int16_t _kaTimeVal;
	///
        int _responseTime;

	///
        PolCallBackFunc _callBackFunc;
	///
        string _domainName;
	///
        string _hostName;
	///
        u_int16_t _clientType;

        //Client information running the PepAgent
	///
        string _clientHost;
	///
        int _clientPort;

        friend class ReqStateUninitialized;
        friend class ReqStatePending;
        friend class ReqStateImplStart;
        friend class ReqStateReceived;
        friend class ReqStateSent;
        friend class PolRequest;
};

#endif
