/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */


static const char* const MrshlOpAck_cxx_Version =
    "$Id: MrshlOpAck.cxx,v 1.50.2.1 2002/05/15 00:59:28 bko Exp $";


#include "AckMsg.hxx"
#include "SipContact.hxx"
#include "Marshal.hxx"
#include "MrshlOpAck.hxx"
#include "MrshlState.hxx"
#include "MrshlOpStatus.hxx"
#include "CdrInterface.hxx"
#include "VCdrException.hxx"

using namespace Vocal;
    
MrshlOpAck::MrshlOpAck(): Operator()
{
}


MrshlOpAck::~MrshlOpAck()
{
}


const char* const
MrshlOpAck::name() const
{
    return "MrshlOpAck";
}


const Sptr < State >
MrshlOpAck::process(const Sptr < SipProxyEvent > event )
{
    string nextHost;
    Sptr < SipEvent > sipEvent;
    sipEvent.dynamicCast(event);

    Marshal &mrshl = Marshal::instance();

    if ( sipEvent == 0 )
    {
        cpLog( LOG_ERR, "process: sipEvent = 0" );
        return PROXY_DONE_WITH_EVENT;
    }

    Sptr < SipMsg > sipMsg = sipEvent->getSipMsg();
    Sptr < AckMsg > ackMsg;
    ackMsg.dynamicCast(sipMsg);

    if ( ackMsg == 0 )
    {
        return PROXY_CONTINUE;
    }

    // log reception of msg
    doCpLog();
    cpLog( LOG_DEBUG, "SIP message received:\n%s,",
	   sipMsg->encode().logData() );

    // check the number of Route, if the number is 0, the
    // ACK is generate by 4xx, don't forward it
    if ( ackMsg->getNumRoute() == 0 )
    {
        cpLog( LOG_INFO , "Route is empty, not for INVITE, don't forward" );
        return PROXY_DONE_WITH_EVENT;
    }

    // marshal the message
    unsigned int mrshlStatus = mrshl.marshalIncomingCommand( ackMsg );
    Data viaBranch; // application specific addition to branch
    if ( mrshlStatus == MRSHL_STATUS_INCOMING )
    {
        viaBranch = MRSHL_INCOMING_CONTACT;
    }
    else if ( mrshlStatus == MRSHL_STATUS_OUTGOING )
    {
        viaBranch = MRSHL_OUTGOING_CONTACT;
    }
    else // mrshlStatus == sip error response code
    {
        // return appropriate error/status msg, and exit
        Sptr<StatusMsg> errorMsg = new StatusMsg( *ackMsg, mrshlStatus );
        sipEvent->getSipStack()->sendReply( errorMsg );
        return PROXY_DONE_WITH_EVENT;
    }

    // start usage policy (if needed) for this call
    Data callId = ackMsg->getCallId().getCallIdData();    
    if ( mrshl.serverType() == MRSHL_TYPE_INTERNETWORK )
    {
        mrshl.policy().policyStart( callId );
    }

    // start billing for connected call
    cpLog( LOG_INFO, "Start billing call From: %s, To: %s, CallId: %s",
           ackMsg->getFrom().getUser().logData(),
           ackMsg->getTo().getUser().logData(),
           callId.logData() );
    try
    {
        CdrInterface::instance().callStarted( ackMsg->getFrom().getUser().convertString(),
                                              ackMsg->getTo().getUser().convertString(),
                                              callId.convertString() );
    }
    catch ( VCdrException&e )
    {
        cpLog( LOG_ERR, "WARNING:Cannot send record to CDR Server" );

        if ( mrshl.requireCdr() )
        {
            cpLog( LOG_ERR, "Billing is required, sending 402" );

            // return 402-Payment Required, and exit
            Sptr<StatusMsg> errorMsg = new StatusMsg( *ackMsg, 402 );
            sipEvent->getSipStack()->sendReply( errorMsg );
            return PROXY_DONE_WITH_EVENT;
        }
    }

    // create new ack msg based on received ack msg, the requestLine
    // is set automatically to the Route (if present) otherwise to To
    Sptr<AckMsg> newAckMsg = new AckMsg( *ackMsg , AckMsg::ackMsgForward );

    // add Via with branch, must be done AFTER setting requestLine
    mrshl.addViaWithBranch( *newAckMsg, viaBranch );

    newAckMsg->setNextHopIsProxy(mrshl.isNextHopAProxy(*newAckMsg));
    sipEvent->getSipStack()->sendAsync( newAckMsg );
    return PROXY_DONE_WITH_EVENT;
}

// End of File
