/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */
static const char* const MrshlOpBye_cxx_Version =
    "$Id: MrshlOpBye.cxx,v 1.54.2.1 2002/05/15 00:59:28 bko Exp $";

    
#include "Marshal.hxx"
#include "MrshlOpBye.hxx"
#include "ByeMsg.hxx"
#include "MrshlUserCache.hxx"
#include "CdrInterface.hxx"
#include "VCdrException.hxx"
#include "MrshlConfMapEntry.hxx"

using namespace Vocal;

MrshlOpBye::MrshlOpBye()
    : Operator()
{
}


MrshlOpBye::~MrshlOpBye()
{
}

    
const char* const
MrshlOpBye::name() const
{
    return "MrshlOpBye";
}


const Sptr < State >
MrshlOpBye::process(const Sptr < SipProxyEvent > event )
{
    Marshal &mrshl = Marshal::instance();

    Sptr < SipEvent > sipEvent;
    sipEvent.dynamicCast( event );

    // return if not a valid event
    if ( sipEvent == 0 )
    {
        cpLog( LOG_ERR, "process: SipEvent = 0" );
        return PROXY_DONE_WITH_EVENT;
    }

    // return if not a Bye message
    Sptr < SipMsg > sipMsg = sipEvent->getSipMsg();
    Sptr < ByeMsg > byeMsg;
    byeMsg.dynamicCast( sipMsg );

    if ( byeMsg == 0 )
    {
        return PROXY_CONTINUE;
    }

    // log reception of Bye message
    doCpLog();
    cpLog( LOG_DEBUG, "SIP message received:\n%s,",
	   sipMsg->encode().logData() );

    // marshal the message
    unsigned int mrshlStatus = mrshl.marshalIncomingCommand( byeMsg );
    Data viaBranch; // application specific portion of branch
    Data user; // originator of call ( used for conference only )

    if ( mrshlStatus == MRSHL_STATUS_INCOMING )
    {
	// check if handling this BYE message as a CANCEL
	if ( byeMsg->getNumRoute() == 0 )
	{
	    cpLog( LOG_DEBUG, "Processing BYE message as a CANCEL" );
	    mrshl.proxyCommandWithoutRoute( sipEvent, false );
	    return PROXY_DONE_WITH_EVENT;
	}

        viaBranch = MRSHL_INCOMING_CONTACT;
        user = byeMsg->getTo().getUser();
    }
    else if ( mrshlStatus == MRSHL_STATUS_OUTGOING )
    {
        viaBranch = MRSHL_OUTGOING_CONTACT;
        user = byeMsg->getFrom().getUser();
    }
    else // mrshlStatus == a sip error response code
    {
        // return appropriate error/status msg, and exit
        Sptr<StatusMsg> errorMsg = new StatusMsg( *byeMsg, mrshlStatus );
        sipEvent->getSipStack()->sendReply( errorMsg );
        return PROXY_DONE_WITH_EVENT;
    }


    Data callId = byeMsg->getCallId().getCallIdData();

    if( mrshl.serverType() == MRSHL_TYPE_INTERNETWORK )
    {
	// disable QoS in PepAgent
	// TODO: determine if responsible for enabling QoS
        mrshl.policy().disableQoS( byeMsg->getCallId() );

	// end/finish usage policy for this call if needed
	// note, safe to call it more than once for the same call
        mrshl.policy().policyFinish( callId );
    }
    else if ( mrshl.serverType() == MRSHL_TYPE_CONFERENCE )
    {
	// handle conference mapping
	// check conference map and remove caller from call
	MrshlConfCallMap &mrshlConfCallMap = mrshl.mrshlConfCallMap();
        MrshlConfCallMap::iterator iter = mrshlConfCallMap.find( callId );
	bool callFound = false;
    
	// check that a conference with this callId exists
        if ( iter != mrshlConfCallMap.end() )
	{
	    // remove user from conference list
	    MrshlConfMapEntry &confMapEntry = mrshlConfCallMap[ callId ];
	    if( confMapEntry.removeCallFromConf( user ) == true )
	    {
		callFound = true;
		unsigned int numParties( confMapEntry.getNumParticipants() );
		
		// if only one party now left, initiate BYE for them
		if ( numParties == 1 )
		{
		    cpLog( LOG_INFO, "1 party left, starting call termination" );
		    confMapEntry.disconnectLastCall( sipEvent );
		}
	    }
	}
    
	if ( callFound == false )
        {
	    cpLog( LOG_ERR, "CallId/user not found in conferences, returning 404" );
	    Sptr<StatusMsg> errorStatusMsg  = new StatusMsg( *byeMsg, 404 ); // 404 not found 
	    sipEvent->getSipStack()->sendReply( errorStatusMsg );
	    return PROXY_DONE_WITH_EVENT;
	}    
    }
    
    // stop cdr billing
    try
    {
        cpLog( LOG_INFO, "Stop billing From: %s, To: %s, CallId: %s",
               byeMsg->getFrom().getUser().logData(),
               byeMsg->getTo().getUser().logData(),
               callId.logData() );

        CdrInterface::instance().callEnded(
            byeMsg->getFrom().getUser().convertString(),
            byeMsg->getTo().getUser().convertString(),
            callId.convertString() );
    }
    catch ( VCdrException&e )
    {
        cpLog( LOG_ERR, "WARNING: Cannot send record to CDR Server" );
    }

    // create new BYE msg from received BYE msg, requestLine
    // is set automatically to the Route (if present) otherwise to To
    Sptr<ByeMsg> newByeMsg = new ByeMsg( *byeMsg , ByeMsg::byeMsgForward );

    // add Via with branch, must be done AFTER setting requestLine
    mrshl.addViaWithBranch( *newByeMsg, viaBranch );

    if(newByeMsg->getNumRoute() == 0)
    {
        //Check the user transport
        SipRequestLine& reqLine = newByeMsg->getMutableRequestLine();
        Sptr<BaseUrl> bUrl = reqLine.getUrl();
        Sptr<SipUrl> sUrl;
        sUrl.dynamicCast(bUrl);
        if(sUrl != 0)
        {
            //Get the marshal user data
            Data user = sUrl->getUserValue();
            MrshlUserCache & myUserCache = MrshlUserCache::instance();
            try
            {
                MrshlUserData tempUserData;
                myUserCache.getMrshlUserData( user.convertString(), tempUserData );
                if(tempUserData.transport == TCP)
                {
                    sUrl->setTransportParam(Data("tcp"));
                    reqLine.setUrl(sUrl);
                }
            }
            catch(VException& e)
            {
                //If user is not found , do nothing, however for debugging
                //purpose lets put the message
                cpLog(LOG_DEBUG, "Failed to get user data for (%s), reason %s",
                                user.logData(), e.getDescription().c_str());
            }
        }
        
    }

    newByeMsg->setNextHopIsProxy(mrshl.isNextHopAProxy(*newByeMsg));
    sipEvent->getSipStack()->sendAsync( newByeMsg );
    return PROXY_DONE_WITH_EVENT;
}


// End of File
