/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */


static const char* const MrshlOpOptions_cxx_Version =
    "$Id: MrshlOpOptions.cxx,v 1.26 2002/03/19 21:39:47 jason Exp $";


#include "Marshal.hxx"
#include "RoundRobinServers.hxx"
#include "MrshlOpOptions.hxx"
#include "SipVia.hxx"

using namespace Vocal;

MrshlOpOptions::MrshlOpOptions()
    : Operator()
{
}


MrshlOpOptions::~MrshlOpOptions()
{
}


const char* const
MrshlOpOptions::name() const
{
    return "MrshlOpOptions";
}


const Sptr < State >
MrshlOpOptions::process(const Sptr < SipProxyEvent > event )
{
    Marshal &mrshl = Marshal::instance();

    Sptr < SipEvent > sipEvent;
    sipEvent.dynamicCast( event );

    if ( sipEvent == 0 )
    {
        cpLog( LOG_ERR, "process: SipEvent = 0" );
        return PROXY_DONE_WITH_EVENT;
    }

    Sptr < SipMsg > sipMsg = sipEvent->getSipMsg();
    Sptr < OptionsMsg > optionsMsg;
    optionsMsg.dynamicCast( sipMsg );

    if ( optionsMsg == 0 )
    {
        return PROXY_CONTINUE;
    }

    // log reception of msg
    doCpLog();
    cpLog( LOG_DEBUG, "SIP message received:\n%s,",
	   sipMsg->encode().logData() );

    // marshal the message
    unsigned int mrshlStatus = mrshl.marshalIncomingCommand( optionsMsg );
    Data viaBranch; // marshal specific branch component
    bool sendBasedOnRoute = false;
    bool sendDirectlyToGateway = false;
    
    // In case the message is a Re-Options, RS is not queried and the
    // branches need to be set as in MrshlOpStatus::process302, otherwise
    // the ACK won't match with the 200 (branches are hard-coded in the
    // ACK operator
    //
    if ( mrshlStatus == MRSHL_STATUS_INCOMING )
    {
        if ( optionsMsg->getNumRoute() > 0 )
        {
            // Re-Options
            viaBranch = MRSHL_INCOMING_CONTACT;
	    sendBasedOnRoute = true;
        }
        else
        {
            viaBranch = MRSHL_INCOMING_REDIRECT;
        }
    }
    else if ( mrshlStatus == MRSHL_STATUS_OUTGOING )
    {
        if ( optionsMsg->getNumRoute() > 0 )
        {
            // Re-Options, send to Contact without going to RS
            viaBranch = MRSHL_OUTGOING_CONTACT;
            sendBasedOnRoute = true;
        }
        else if ( mrshl.serverType() == MRSHL_TYPE_USERAGENT )
        {
            // send to RS to get Contact
            viaBranch = MRSHL_OUTGOING_REDIRECT;
        }
	else
        {
            // for outgoing Options msgs, gateway marshals use their gateway
            // as the Contact without contacting RS
            viaBranch = MRSHL_OUTGOING_CONTACT;
	    sendDirectlyToGateway = true;
        }
    }
    else // mrshlStatus == a sip error response code
    {
        // return appropriate error/status msg, and exit
        StatusMsg errorMsg( *optionsMsg, mrshlStatus );
        sipEvent->getSipStack()->sendReply( errorMsg );
        return PROXY_DONE_WITH_EVENT;
    }

    // return 100 to sender
    StatusMsg statusMsg( *optionsMsg, 100 );
    sipEvent->getSipStack()->sendReply( statusMsg );

    // check if Options already has a route, if so, forward based on
    // this route and return
    if ( sendBasedOnRoute )
    {
        // create new options msg based on received options msg
        cpLog( LOG_INFO, "forward Options based on route" );
        Sptr<OptionsMsg> newOptionsMsg = new OptionsMsg(*optionsMsg,
                                                        OptionsMsg::optionsMsgForward );
        
	// add Via with branch, must be done AFTER setting requestLine
	mrshl.addViaWithBranch( *newOptionsMsg, viaBranch );

        sipEvent->getSipStack()->sendAsync( newOptionsMsg );
        return PROXY_DONE_WITH_EVENT;
    }

    Sptr < SipUrl > routeSipUrl;
    routeSipUrl.dynamicCast( optionsMsg->getRequestLine().getUrl() );
    if ( routeSipUrl == 0 )
    {
        cpLog( LOG_ERR, "process: routeSipUrl = 0" );
        return PROXY_DONE_WITH_EVENT;
    }        

    // prep RequestUri
    if ( sendDirectlyToGateway )
    {
        // set RequestUri to gateway
       routeSipUrl->setHost( mrshl.gatewayHost() );
       routeSipUrl->setPort( mrshl.gatewayPort() );
    }
    else
    {
        // set RequestUri to redirect server
        RoundRobinServers& rServers = RoundRobinServers::instance( SERVER_RS );
        Data lHost; 
        int  lPort;
        rServers.getHostNPort(sipMsg->getFrom().getUrl()->encode(), lHost, lPort);
        routeSipUrl->setHost( lHost);
        routeSipUrl->setPort( lPort);
    }

    // set RequestUri
    SipRequestLine requestLine( optionsMsg->getRequestLine() );
    requestLine.setUrl( routeSipUrl );
    optionsMsg->setRequestLine( requestLine );
    
    // add Via with branch, must be done AFTER setting requestLine
    mrshl.addViaWithBranch( *optionsMsg, viaBranch );

    // do we need any policy stuff here for options? JF

    // note, for INVITEs, Internetwork Marshals would check with policy
    // server prior to sending; not sure if this is needed for OPTIONS,
    // so for now, this check is removed for the sake of simplicity
    sipEvent->getSipStack()->sendAsync( *optionsMsg );
    return PROXY_DONE_WITH_EVENT;
}

// End of File
