/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */

static const char* const MrshlOpStatus_cxx_Version =
    "$Id: MrshlOpStatus.cxx,v 1.79.2.1 2002/05/15 00:59:28 bko Exp $";


#include "AckMsg.hxx"
#include "CancelMsg.hxx"
#include "CdrInterface.hxx"
#include "Marshal.hxx"
#include "MrshlConfMapEntry.hxx"
#include "MrshlOpInvite.hxx"
#include "MrshlOpStatus.hxx"
#include "MrshlState.hxx"
#include "RoundRobinServers.hxx"
#include "SipCallId.hxx"
#include "SipCommand.hxx"
#include "SipContact.hxx"
#include "SipVia.hxx"
#include "State.hxx"
#include "SystemInfo.hxx"
#include "VCdrException.hxx"
#include "MrshlConfMapEntry.hxx"
#include "MrshlConfParticipantInfo.hxx"

using namespace Vocal;

MrshlOpStatus::MrshlOpStatus()
    : Operator()
{
}


MrshlOpStatus::~MrshlOpStatus()
{
}


const char* const
MrshlOpStatus::name() const
{
    return "MrshlOpStatus";
}


const Sptr < State >
MrshlOpStatus::process( const Sptr < SipProxyEvent > event )
{
    Marshal &mrshl = Marshal::instance();

    Sptr < SipEvent > sipEvent;
    sipEvent.dynamicCast(event);

    if ( sipEvent == 0 )
    {
        cpLog( LOG_ERR, "process: SipEvent = 0" );
        return PROXY_DONE_WITH_EVENT;
    }

    Sptr < SipMsg > sipMsg = sipEvent->getSipMsg();
    Sptr < StatusMsg > statusMsg;
    statusMsg.dynamicCast( sipMsg );

    if ( statusMsg == 0 )
    {
        return PROXY_CONTINUE;
    }

    // log reception of msg
    doCpLog();
    cpLog( LOG_DEBUG, "SIP message received:\n%s,",
	   sipMsg->encode().logData() );

    int statusCode = statusMsg->getStatusLine().getStatusCode();
    switch (statusCode)
    {
        case 100:
        {
            // ignore 100
            cpLog( LOG_DEBUG, "Received 100, no processing required" );
        }
        break;

        case 181:
        case 182:
	{
            // forward based on Via unless 2xx already forwarded
            // maybe this is already done in stack
            cpLog( LOG_DEBUG, "Received 181/182, forward based on Via" );
            forwardBasedOnVia( sipEvent, statusMsg );
        }
        break;

        case 180:
        case 183:
        {
            // enable QoS in PepAgent
            // TODO: determine if responsible for enabling QoS
            if( mrshl.serverType() == MRSHL_TYPE_INTERNETWORK )
            {
                mrshl.policy().enableQoS( sipEvent, statusMsg );
            }

            // start billing for ringtime and
            // forward based on Via unless 2xx already forwarded
            // maybe this is already done in stack
            cpLog( LOG_DEBUG, "Received 180/183, bill for ringtime" );
            process180183( sipEvent, statusMsg );
        }
        break;

        case 200:
        {
            // forward based on Via unless 2xx already forwarded
            // maybe this is already done in stack
            cpLog( LOG_DEBUG, "Received 2xx, need to look at method" );
	    process200( sipEvent, statusMsg );
	}
        break;

        case 302:
        {
            // For INVITEs,
            // send ACK to Request URI of matching request, and forward
            // upstream using Via, except if from redirect server,
            // in which case special processing is required
            cpLog( LOG_DEBUG, "Received 302" );
            process302( sipEvent, statusMsg );
        }
        break;

        default:
        {
	    // other 3xx or 4xx, 5xx, 6xx, handle as Failure
	    cpLog( LOG_ERR, "Received %d", statusCode );
	    processDefault( sipEvent, statusMsg );
        }
        break;

    } // end of switch (statusCode)

    return PROXY_DONE_WITH_EVENT;
}



void
MrshlOpStatus::forwardBasedOnVia( Sptr < SipEvent > sipEvent,
                                  Sptr < StatusMsg > statusMsg )
{
    cpLog( LOG_DEBUG, "Forward status" );

    // don't try to forward if ours is the last Via
    if ( statusMsg->getNumVia() <= 1 )
    {
        cpLog( LOG_ERR, "No Via after ours, cannot forward STATUS" );
    }

    else
    {
        // remove own Via header
        statusMsg->removeVia( 0 );

        // forward status message to next Via (done automatically by stack)
        sipEvent->getSipStack()->sendReply( statusMsg );
    }
}



void
MrshlOpStatus::ackBasedOnCommand( Sptr < SipEvent > sipEvent,
				  Sptr < StatusMsg > statusMsg )
{
    // retrieve COMMAND associated with sipEvent
    Sptr < SipCommand > savedCommand = sipEvent->getCommand();
    if ( savedCommand == 0 )
    {
        // error, no corresponding COMMAND, return without sending ACK
        cpLog( LOG_ERR, "No COMMAND associated with STATUS" );
        return ;
    }

    // send ACK to receiver of corresponding INVITE by setting Request URI
    // of ACK to that of the INVITE
    Sptr<AckMsg> ack = new AckMsg( *statusMsg );
    SipRequestLine& ackRequestLine = ack->getMutableRequestLine();
    ackRequestLine.setMethod(Data(ACK_METHOD));
    ackRequestLine.setUrl(savedCommand->getRequestLine().getUrl());
    ack->setNextHopIsProxy(Marshal::instance().isNextHopAProxy(*ack));
    sipEvent->getSipStack()->sendAsync( ack );
}



void
MrshlOpStatus::process180183( Sptr < SipEvent > sipEvent,
                              Sptr < StatusMsg > statusMsg )
{
    Marshal &mrshl = Marshal::instance();
    bool ableToBill = true;

    // start billing for ringtime
    cpLog( LOG_DEBUG,
           "Start billing ring From: %s, To: %s, CallId: %s",
           statusMsg->getFrom().getUser().logData(),
           statusMsg->getTo().getUser().logData(),
           statusMsg->getCallId().getCallIdData().logData() );
    try
    {
        CdrInterface::instance().ringStarted(
            statusMsg->getFrom().getUser().convertString(),
            statusMsg->getTo().getUser().convertString(),
            statusMsg->getCallId().getCallIdData().convertString() );
    }
    catch ( VCdrException&e )
    {
        cpLog( LOG_ERR, "WARNING: Cannot send to CDR Server" );
        ableToBill = false;
    }

    // cancel call if billing is required and not able to bill
    if ( mrshl.requireCdr() && !ableToBill )
    {
        // retrieve associated INVITE in order to Cancel
        Sptr < InviteMsg > savedInvite = sipEvent->getInvite();
        if ( savedInvite == 0 )
        {
            // error, no corresponding INVITE
            cpLog( LOG_ERR, "No INVITE associated with STATUS" );
            cpLog( LOG_ERR, "Cannot bill, but cannot cancel either" );
        }
        else
        {
            // return 402-Payment Required to caller
            cpLog( LOG_ALERT, "Billing is required, sending 402" );
            Sptr<StatusMsg> errorMsg = new StatusMsg( *savedInvite, 402 );
            sipEvent->getSipStack()->sendReply( errorMsg );

            // send Cancel to callee
            Sptr<CancelMsg> cancelMsg = new CancelMsg( *savedInvite );
            sipEvent->getSipStack()->sendAsync( cancelMsg );
        }
    }
    else
    {
	    if ( mrshl.serverType() != MRSHL_TYPE_CONFERENCE )
    	{
    	    // forward based on Via unless 2xx already forwarded
    	    // maybe this is already done in stack
    	    cpLog( LOG_DEBUG, "forward based on Via" );
    	    forwardBasedOnVia( sipEvent, statusMsg );
    	}
    	else
    	{
            cpLog( LOG_INFO, "180/183 not forwarded for conference calls" );
    	}
    }
}



void
MrshlOpStatus::process200( Sptr < SipEvent > sipEvent,
                           Sptr < StatusMsg > statusMsg )
{
    Marshal &mrshl = Marshal::instance();

    if ( statusMsg->getCSeq().getMethod() == CANCEL_METHOD )
    {
	cpLog( LOG_DEBUG, "CANCEL is hop by hop, don't forward" );
	return;
    }
    else if ( statusMsg->getCSeq().getMethod() == REGISTER_METHOD )
    {
	// remove self from contact list, marshal is always
	// the first contact
	if ( statusMsg->getNumContact() < 1 )
	{
	    cpLog( LOG_ERR, "No Contacts in 200" );
	}
	else
	{
	    cpLog( LOG_DEBUG, "remove self from Contact" );
	    statusMsg->removeContact( 0 );
	}
    }    
    else if ( mrshl.serverType() == MRSHL_TYPE_CONFERENCE )
    {
	// check command type and handle accordingly
	SipCallId callId = statusMsg->getCallId();
	MrshlConfCallMap &mrshlConfCallMap = mrshl.mrshlConfCallMap();
	MrshlConfMapEntry& confMapEntry = mrshlConfCallMap[ callId ];
		
	if ( statusMsg->getCSeq().getMethod() == INVITE_METHOD )
	{
	    // save status msg in participant's info
	    Data user = statusMsg->getFrom().getUser();
	    confMapEntry.setStatusMsg( user, ( *statusMsg ));
	}
	else if ( statusMsg->getCSeq().getMethod() == BYE_METHOD )
	{
	    // check if conference is finished
	    unsigned int numParties( confMapEntry.getNumParticipants() );
		    
	    if ( numParties == 0 )
	    {
		bool lastUserDisconnect = confMapEntry.getLastUserDisconnect();
			
		cpLog( LOG_INFO, "Removing conference from conf map" );
		mrshlConfCallMap.erase( callId );
			
		// if this was call disconnected by marshal, no further processing
		// required
		if ( lastUserDisconnect == true )
		{
		    return;
		}
	    }
	}
    }
 
    forwardBasedOnVia( sipEvent, statusMsg );
}



void
MrshlOpStatus::process302( Sptr < SipEvent > sipEvent,
                           Sptr < StatusMsg > statusMsg )
{
    if ( statusMsg->getCSeq().getMethod() == INVITE_METHOD )
    {
        // return ACK to receiver of corresponding Command
        cpLog( LOG_DEBUG, "ACK 302 response for INVITE" );
        ackBasedOnCommand( sipEvent, statusMsg );
    }

    // retrieve command associated with 302
    Sptr <SipCommand > savedCommand = sipEvent->getCommand();
    if ( savedCommand == 0 )
    {
        // error, no corresponding COMMAND, forward and hope someone
        // else knows what to do with it
        cpLog( LOG_ERR, "No COMMAND associated with 302, try forwarding" );
        forwardBasedOnVia( sipEvent, statusMsg );
        return ;
    }

    Data callId = statusMsg->getCallId().getCallIdData();

    // if not from redirect server, simply forward and return
    Sptr < SipUrl > inviteSipUrl;
    inviteSipUrl.dynamicCast( savedCommand->getRequestLine().getUrl() );
    if ( inviteSipUrl == 0 )
    {
	cpLog( LOG_ERR, "process302: inviteSipUrl = 0" );
	return;
    }

    RoundRobinServers& rServers = RoundRobinServers::instance( SERVER_RS );
    Marshal &mrshl = Marshal::instance();
    
    if ( !rServers.isExistingServer( inviteSipUrl->getHost(),
				     inviteSipUrl->getPort().convertInt() ) )
    {
        if ( mrshl.serverType() == MRSHL_TYPE_INTERNETWORK )
        {
            cpLog( LOG_DEBUG, "Abort policy" );
            mrshl.policy().policyAbort( callId );
        }

        cpLog( LOG_DEBUG, "302 not from RS, forward" );
        forwardBasedOnVia( sipEvent, statusMsg );
        return ;
    }

    // create branch for Via
    Data viaBranch; // application specific portion of branch
    if ( mrshl.hasMarshalKey( savedCommand ) )
    {
        // incoming, set branch
        cpLog( LOG_DEBUG, "incoming, set branch to incoming contact" );
        viaBranch = MRSHL_INCOMING_CONTACT;
    }
    else
    {
        // outgoing, set branch
        cpLog( LOG_DEBUG, "outgoing, set branch to outgoing contact" );
        viaBranch = MRSHL_OUTGOING_CONTACT;
    }
    
    // get contact to which to send Invite from the 302 msg
    Sptr<SipContact> contact = getContactFromList(
	statusMsg->getContactList() );

    // return 404 if no contact
    if ( contact == 0 )
    {
        // unexpectedly empty Contact list (incoming/outgoing),
        // or cannot find ourselves of next Contact (outgoing),
        // return 404 to sender of COMMAND
        cpLog( LOG_ERR, "Empty Contact list, returning 404 to sender" );
        Sptr<StatusMsg> errorStatusMsg  = new StatusMsg( *savedCommand, 404 );  // 404 Not Found
        errorStatusMsg->removeVia( 0 );
        sipEvent->getSipStack()->sendReply( errorStatusMsg );
        return ;
    }
    
    // If serverType is Conference, check message info and add
    // to global map
    unsigned int branch = viaBranch.convertInt();
    Data confAccessNumber = "";
    if (( mrshl.serverType() == MRSHL_TYPE_CONFERENCE ) &&
	( branch == MRSHL_OUTGOING_CONTACT  ) &&
	( statusMsg->getCSeq().getMethod() == INVITE_METHOD ) )
    {
	Sptr < InviteMsg > savedInvite;
	savedInvite.dynamicCast( savedCommand );

        // search for the callId in the conference map
        const SipCallId& callId = savedInvite->getCallId();
	MrshlConfCallMap &mrshlConfCallMap = mrshl.mrshlConfCallMap();
        MrshlConfCallMap::iterator iter = mrshlConfCallMap.find( callId );
	
        if ( iter == mrshlConfCallMap.end() )
        {
            // new conference
            // get an available access number to use for the conference
            if ( mrshl.accessNumbers().empty() == true)
            {
		// none available, return 486 (busy here)
                cpLog( LOG_ERR, "No available conference access numbers" );
                Sptr<StatusMsg> errorStatusMsg  = new StatusMsg( *savedInvite, 486 );
		errorStatusMsg->removeVia( 0 );
                sipEvent->getSipStack()->sendReply( errorStatusMsg );
		return ;
	    }
    
            cpLog( LOG_INFO, "New ad-hoc conference!, adding to conf map" );
            MrshlConfMapEntry confEntry;
            mrshlConfCallMap[ callId ] = confEntry;
	    
	    confAccessNumber = mrshl.accessNumbers().front();
            mrshl.accessNumbers().pop_front();
            mrshlConfCallMap[ callId ].setBridgeAccessNumber( confAccessNumber );
            cpLog( LOG_INFO, "Access number %s will be used",
                   confAccessNumber.logData() );
	    
	}
        else
        {
            cpLog( LOG_INFO, "Existing adhoc conference with %d participants!",
                   mrshlConfCallMap[ callId ].getNumParticipants() );

            // get the access number in use for this conference
            confAccessNumber = mrshlConfCallMap[ callId ].getBridgeAccessNumber();
            cpLog( LOG_INFO, "Uses access number %s",
                   confAccessNumber.logData() );
        }

        // add user to list of active calls for this conference
        MrshlConfParticipantInfo confCallInfo;
        confCallInfo.setUser( savedInvite->getFrom().getUser() );

        // save inviteMsg in participant's info
        confCallInfo.setInviteMsg( *savedInvite );
        mrshlConfCallMap[ callId ].addCallToConf( confCallInfo );
    }

    // set request line ( changing sipUrl for conference if required ) 
    SipRequestLine& commandRequestLine = savedCommand->getMutableRequestLine();
    commandRequestLine.setUrl(contact->getUrl());

    if( confAccessNumber.length() != 0 )
    {
	cpLog( LOG_INFO, "setting url number to %s", confAccessNumber.logData() );
	Sptr < SipUrl > newUrl( inviteSipUrl );
	newUrl->setUserValue( confAccessNumber, TEL_USER );
	commandRequestLine.setUrl( newUrl );
    }  
    

    // replace Via with new branch, must be done AFTER setting requestLine
    savedCommand->removeVia( 0 );
    mrshl.addViaWithBranch( *savedCommand, viaBranch );

    // authorize usage policy, if needed
    if ( ( !( mrshl.hasMarshalKey( savedCommand ) ) ) && 
         ( mrshl.serverType() == MRSHL_TYPE_INTERNETWORK ) &&
         ( statusMsg->getCSeq().getMethod() == INVITE_METHOD ) )
    {
	Sptr < InviteMsg > savedInvite;
	savedInvite.dynamicCast( savedCommand );

        if (savedInvite != 0)
        {
           if ( mrshl.policy().policyAuthorize(
                   sipEvent, savedInvite ) == false )
           {
              cpLog( LOG_INFO, "Policy rejected auth request for callId %s",
                     callId.logData() );

              Sptr<StatusMsg> errorStatusMsg  = new StatusMsg( *savedInvite, 401 );
              errorStatusMsg->removeVia( 0 );
              sipEvent->getSipStack()->sendReply( errorStatusMsg );
           }
        }
    }
    else
    {
        // send Invite to Contact
        cpLog( LOG_DEBUG, "send Invite to Contact ... \n%s" , savedCommand->encode().logData());
        savedCommand->setNextHopIsProxy(Marshal::instance().isNextHopAProxy(*savedCommand));
        sipEvent->getSipStack()->sendAsync( savedCommand );
    }
}



void
MrshlOpStatus::processDefault( Sptr < SipEvent > sipEvent,
                           Sptr < StatusMsg > statusMsg )
{
    Marshal &mrshl = Marshal::instance();

    if ( mrshl.serverType() == MRSHL_TYPE_INTERNETWORK )
    {
	// abort policy
	cpLog( LOG_DEBUG, "Abort policy" );
	Data callId = statusMsg->getCallId().getCallIdData();
	mrshl.policy().policyAbort( callId );
    }
    else if ( mrshl.serverType() == MRSHL_TYPE_CONFERENCE )
    {
	// remove user and conference if necessary
	SipCallId callId = statusMsg->getCallId();
	MrshlConfCallMap &mrshlConfCallMap = mrshl.mrshlConfCallMap();
	MrshlConfMapEntry& confMapEntry = mrshlConfCallMap[ callId ];

	Data user = statusMsg->getFrom().getUser();
	confMapEntry.removeCallFromConf( user );
    
	unsigned int numParticipants( confMapEntry.getNumParticipants() );
	if ( numParticipants == 0 )
	{
	    cpLog( LOG_INFO, "Removing conference from conf map" );
	    mrshlConfCallMap.erase( callId );
	}
    }

    if ( statusMsg->getCSeq().getMethod() == INVITE_METHOD )
    {
        // send ACK to Request URI of matching request
        cpLog( LOG_DEBUG, "ACK %d response for INVITE",
               statusMsg->getStatusLine().getStatusCode() );
        ackBasedOnCommand( sipEvent, statusMsg );
    }
    // Forward upstream using Via
    cpLog( LOG_DEBUG, "Forward based on Via)" );
    forwardBasedOnVia( sipEvent, statusMsg );
}


Sptr<SipContact>
MrshlOpStatus::getContactFromList( const SipContactList& contactList,
                                   bool getFirstContact )
{
    // check for empty contact list
//    if ( contactList.size() == 0)
    if ( contactList.empty() )
    {
        // empty Contact list, always an error
        cpLog( LOG_ERR, "Unexpectedly empty Contact list" );
        return 0;
    }

    SipContactList::iterator iter = contactList.begin();

    if ( getFirstContact )
    {
        // returning first Contact, no error checking is needed
        // first contact could be ourself for hairpin call case
        // and this should be ok
        cpLog( LOG_DEBUG, "Return first Contact in list" );
        return *iter;
    }

    // traverse Contact list
    Data hostAddress;
    unsigned short hostPort;
#if defined(__SUNPRO_CC)
    while ( !(iter == contactList.end()) )
#else
    while ( iter != contactList.end() )
#endif
    {
        Sptr < SipUrl > iterSipUrl;
	iterSipUrl.dynamicCast( ( *iter )->getUrl() );
	if (iterSipUrl == 0 )
	{
	    cpLog( LOG_ERR, "getContactFromList: iterSipUrl = 0" );
	    return 0;
	}

        hostAddress = iterSipUrl->getHost();
        hostPort = iterSipUrl->getPort().convertInt();
        if ( ( hostAddress == theSystem.gethostAddress() ) &&
                ( hostPort == Marshal::instance().sipPort() ) )
        {
            // found self in list, return the next Contact
            cpLog( LOG_DEBUG, "found self in Contact list" );

            ++iter;
            if ( iter == contactList.end() )
            {
                // no Contact after us, this is an error
                cpLog( LOG_ERR, "we are last contact" );
                return 0;
            }

            cpLog( LOG_DEBUG, "next hop is: %s:%d",
                   iterSipUrl->getHost().logData(),
                   atoi( iterSipUrl->getPort().logData() ) );
            return *iter;
        }
        ++iter;
    }

    // end of Contact list, always an error
    cpLog( LOG_ERR, "Unexpectedly reached end of Contact list" );
    return 0;
}


// End of File
