#ifndef _MRSHLPOLICY_HXX_
#define _MRSHLPOLICY_HXX_

/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */


static const char* const mrshlpolicy_hxx_Version =
    "$Id: MrshlPolicy.hxx,v 1.16 2002/03/19 21:39:47 jason Exp $";


#include <map>

#include "Sptr.hxx"
#include "Data.hxx"
#include "SipEvent.hxx"
#include "InviteMsg.hxx"
#include "VRWLock.hxx"

#ifndef NO_POLICY
#include "PepAgent.hxx"
#include "VDuplicateIdException.hxx"
#endif

namespace Vocal
{

/**
 * Map storing all the corresponding SipEvent for all InviteMsgsSipEvents
 * which are pending based on the result of requests made to the policy
 * server. Call ID is the key field, SipEvent is the Invite message.
 */
typedef map < Data, Sptr < Vocal::SipEvent > > PendingInviteMap;

///

/**
 * MrshlPolicy, encapsulates all the Internet Marshal/OSP functions
 * and manages the PendingInviteMap. The class uses the pepLib 
 * (Policy Enforcement Point) for all of the OSP call authorizations
 * and token validations.
 */
class MrshlPolicy
{
    public:

        /**
         * Default constructor
         */
        MrshlPolicy();

        /**
         * Destructor
         */
        ~MrshlPolicy();

        /**
         * Initialize object data/connections
         * @param int responseTime timeout for policy server connection
         * @return void
         */
        void initialize( const int responseTime );

        /**
         * Is this call ID in the PendingInviteMap?
         * @param Data& callId
         * @return bool
         */
        bool inList( const Data &callId );

        /**
         * Add a new invite to the PendingInviteMap
         * @param Data& callId
         * @param Sptr<SipEvent> sipEvt
         * @return void
         */
        void addInvite( const Data &callId, const Sptr < Vocal::SipEvent > sipEvt );

        /**
         * Remove an invite from the PendingInviteMap
         * @param Data& callId
         * @return bool
         */
        bool removeInvite( const Data &callId );

        /**
         * Abort policy for cancel and status SIP messages
         * @param Data& callId
         * @return void
         */
        void policyAbort( const Data &callId );

        /**
         * Start policy for a new ack message
         * @param Data& callId
         * @return void
         */
        void policyStart( const Data &callId );

        /**
         * Finish an existing policy when a bye message received
         * @param Data& callId
         * @return void
         */
        void policyFinish( const Data &callId );

        /**
         * Authorize call, get a token from policy server
         * @param Sptr <SipEvent> sipEvent
         * @param Sptr <InviteMsg> inviteMsg
         * @return bool true if the policy server returns a token
         */
        bool policyAuthorize( const Sptr < Vocal::SipEvent > sipEvent,
                              Sptr < InviteMsg > inviteMsg );

        /**
         * Validate a received token. Will also remove the token
         * from the message.
         * @param Sptr <SipEvent> sipEvent
         * @param Sptr <InviteMsg> inviteMsg
         * @return bool true if the token is valid
         */
        bool policyValidate( const Sptr < Vocal::SipEvent > sipEvent,
                             Sptr < InviteMsg > inviteMsg )
            #ifndef NO_POLICY                             
            throw (VDuplicateIdException&);
            #else
            ;
            #endif            

        /**
         * Enable the Qos
         * @param Sptr <SipEvent> sipEvent
         * @param Sptr <StatusMsg> statusMsg
         * @return bool true if the Qos is enabled successfully
         */
        bool enableQoS( const Sptr<Vocal::SipEvent> sipEvent,
                        const Sptr<StatusMsg> statusMsg );

        /**
         * Disable the Qos
         * @param SipCallId callId
         * @return bool true if the Qos is disabled successfully
         */
        bool disableQoS( const SipCallId &callId );

#ifndef NO_POLICY

        /**
         * This is the callback used by the pep agent
         * The callback finds the associated invite message and
         * either authorizes invite and adds the token to the SIP
         * message or receives a validation of the existing token
         * for incoming calls
         * @param PolRequest& policyRequest
         * @return void
         */
        static void policyCallback( PolRequest& policyRequest );

#endif

    private:

        /// map < Data, Sptr < Vocal::SipEvent > >
        static PendingInviteMap myPendingInviteMap;

        /// lock for the _pendingInviteMap
        static VRWLock myLock;

        /// for Policy timeout
        int myResponseTime;

        /**
         * Temporary for sip osp, default false
         * The SIP message has a new header for the OSP token,
         * we are keeping the code which uses the content data
         * for compatibility with other systems
         */
        static bool myUseContentData;
};

}

// Local Variables:
// mode:c++
// c-file-style:"bsd"
// c-basic-offset:4
// c-file-offsets:((inclass . ++))
// indent-tabs-mode:nil
// End:

#endif // _MRSHLPOLICY_HXX_
