
/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */
static const char* const IPPlanContainer_cxx_Version =
    "$Id: IPPlanContainer.cxx,v 1.19 2002/02/25 05:51:19 sprajpat Exp $";
#include <cpLog.h>
#include <stdio.h>
#include "IPPlanContainer.hxx"
#include "ProvisionInterface.hxx"


Sptr < IPPlanContainer > IPPlanContainer::theIPContainer = 0;

/**
 * Return the singleton IP plan container or create it if
 * it does not already exist. 
 * @return IP plan container singleton
 */
IPPlanContainer&
IPPlanContainer::instance()
{
    if ( theIPContainer == 0 )
    {
        theIPContainer = new IPPlanContainer;
    }

    return (*theIPContainer);
}

/**
 * Constructor
 */
IPPlanContainer::IPPlanContainer()
{}
;

/**
 * Destructor
 */
IPPlanContainer::~IPPlanContainer()
{}
;

/**
 * Add IP plan elements one at a time
 * A new plan is created if the plan is not already in the container.
 * @param  planName  Unique name of plan to which the element is to be added.
 * @param  key  Regular expression
 * @param  contacts  List of contacts
 */
void
IPPlanContainer::add( const Data& planName, const Data& key, const vector < Data > & contacts )
{
    //  cpLog( LOG_DEBUG,"IPPlanContainer::add");
    /// find the plan in the container
    if (IPContainer.count(planName) == 0)
    {
        /// The dial plan does not exist, so create it
        Sptr < Plan > workingPlan = new Plan;
        workingPlan->addEntry(key, contacts);
        IPContainer[planName] = workingPlan;
    }
    else
    {
        IPContainer[planName]->addEntry(key, contacts);
    }

};

/**
 * Remove the specified IP plan
 * @param  planName  Identifier of plan to be removed.
 */
void
IPPlanContainer::remove( const Data& planName)
{
    /// search the container for the requested IP plan
    PlanIter i = IPContainer.find(planName);

    if (i != IPContainer.end() )
    {
        /// clear out the IP plan
        i->second->clear();
        /// remove the plan from the table
        IPContainer.erase(i);
    }

};

/**
 * Search plan using regular expression matching to find key.
 * Retrieve and return contact list.
 * Throw an exception if plan is not found.
 * @param  planName  Unique identifier of plan to be searched.
 * @param  key  String to be matched against regular expressions in plan.
 * @return  Contact list 
 */
bool
IPPlanContainer::search( const Data& planName, const Data& key, vector < Data > & planFound)
{
    //  cpLog( LOG_DEBUG,"IPPlanContainer::search");

    if (IPContainer.count(planName) == 0)
    {
        cpLog(LOG_DEBUG, "IP Plan is missing for %s", planName.logData());
        return false;
    }

    if (true == IPContainer[planName]->search(key, planFound))
    {
        return true;
    }
    else
    {
        return false;
    }
};

/**
 * Look for and obtain a specific IP plan from the container
 * @param  planName  Identifier of plan to search for.
 * @param  workingPlan  The obtained plan.
 * @return true if found, false if not
 */
bool
IPPlanContainer::getPlan( const Data& planName, Sptr<Plan> workingPlan)
{
    /// search the container for the requested IP plan
    PlanIter i = IPContainer.find(planName);

    if (i != IPContainer.end() )
    {
        workingPlan = i->second;
        return true;
    }
    else
        return false;
};

/**
 * create a new plan and store it against the given name
 * @param  planName  Unique identifier of new plan.
 * @param  workingPlan  The plan object created.
 */
void
IPPlanContainer::create( const Data& planName, Sptr<Plan> workingPlan )
{
    Sptr < Plan > newPlan = new Plan;
    IPContainer.insert(pair < Data, Sptr < Plan > > (planName, newPlan));
    workingPlan = newPlan;
};


/**
 * Display the specified plan.
 * @param  planName  Unique identifier of plan to be displayed.
 */
void
IPPlanContainer::display( const Data& planName)
{
    cpLog( LOG_DEBUG, "IPPlanContainer::display");

    if (IPContainer.count(planName) == 0)
    {
        cpLog(LOG_DEBUG, "IP Plan is missing for %s", planName.logData());
        return ;
    }

    IPContainer[planName]->display();
}

/**
 * The Provisioning Server is notifying us of a change in an IP Plan.
 * Obtain the plan from the PS and store it.
 * @param  data  Contents of xml file
 * @param  fileName  Name of plan file
 * @param  deleteFlag  Indicator of whether file was deleted or not.
 */
void
updateIPPlan(const string& data, const string& fileName, bool deleteFlag)
{

    cpLog( LOG_DEBUG, "updateIPPlan triggered");

    /// Strip off the '.xml' extension
    string withOutXml = fileName;
    string::size_type pos = fileName.find(".xml");
    if (pos != string::npos)
        withOutXml = fileName.substr(0, pos);

    Data identifier(withOutXml);

    vector < VPlanElement > planHolder;

    try
    {
        /// Retrieve the IP Plan specific info from the PS
        ProvisionInterface::instance().getDialPlan(fileName, planHolder);
    }
    catch (VException& e)
    {
        /// failed to connect to the Provisioning Server
        cpLog(LOG_ALERT, "Failed to connect to Provisioning Server, reason %s", e.getDescription().c_str());
        return;
    }

    IPPlanContainer& ipContainer = IPPlanContainer::instance();

    /// Remove the current IP Plan
    ipContainer.remove(identifier);

    /// Create the Dial Plan with the new data
    for ( vector < VPlanElement > ::iterator i = planHolder.begin() ; i != planHolder.end() ; ++i)
    {
        Data key(i->getKey());
        ipContainer.add(identifier, key, i->getContactList());
    }

    ipContainer.display(identifier);

} /// end of updateIPPlan

