
/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */
static const char* const RSProxy_cxx_Version =
    "$Id: RSProxy.cxx,v 1.53 2002/04/11 23:47:41 bko Exp $";


#include "RSProxy.hxx"
#include "RsAgent.hxx"
#include "CommandLine.hxx"
#include "RSServerContainer.hxx"
#include "SystemInfo.hxx"

using namespace Vocal;

// static data declaratin
RSProxy* RSProxy::rsInstance = 0;

/**
 * Return the singleton RSProxy or create it if
 * it does not already exist.
 * @return reference singleton of RSProxy
 */
RSProxy&
RSProxy::instance(Sptr<RSBuilder> rsBuilder /*default=0*/,
                  unsigned short defaultSipPort /*default=5060*/)
{
    if (rsInstance == 0)
    {
        if (rsBuilder == 0)
        {
            cpLog (LOG_ERR, "RSBuilder has not been defined!!!");
            assert (0);
        }

        rsInstance = new RSProxy(rsBuilder, defaultSipPort);
    }

    return *rsInstance;
}

/**
 * Constructor
 * @param  rsBuilder  Redirect server builder object
 * @param  defaultRedundancyPort  Port to be used for syncronization
 * @param  defaultSipPort  Port to be used for SIP messaging
 */
RSProxy::RSProxy( Sptr < RSBuilder > rsBuilder, unsigned short defaultSipPort)
  : BasicProxy(rsBuilder, defaultSipPort, RSNAME), 
    itsState(RSINACTIVE), 
    localSipPort(defaultSipPort)
{

    if(CommandLine::instance()->getInt("HEARTBEAT"))
    {
        cpLog(LOG_INFO, "Initializing heartbeat mechanism");
        myHeartbeatRxThread = new HeartbeatRxThread( RSServerContainer::instance() );
        myHouseKeepingThread = new HouseKeepingThread( RSServerContainer::instance() );

        Data localHostPort(theSystem.gethostAddress());
        localHostPort += ":";
        localHostPort += localSipPort;
        cpLog(LOG_DEBUG, "Local host:port [%s]", localHostPort.logData());
        unsigned short redundancyPort = RSServerContainer::instance().getRSSyncPort(localHostPort);
        if (redundancyPort == 0)
        {
            /// redundancyPort is 0
            cpLog(LOG_ALERT, "********** ALERT: Cannot get RSSyncPort!!!!");
            cpLog(LOG_ALERT, "********** ALERT: RS and/or RSSyncPort");
            cpLog(LOG_ALERT, "********** ALERT: needs to be configured");
            /// Assert!!!! Cannot start RS without Sync Port!!!
            assert(0);
        }
        else
        {
            cpLog(LOG_DEBUG, "Redundancy port [%d]", redundancyPort);
        }

        Sptr < Fifo < Sptr < SipProxyEvent > > > redundancyTimerQueue = new Fifo < Sptr < SipProxyEvent > > ;
        assert (redundancyTimerQueue != 0);

        itsRedundancyRxThread = new RedundancyRxThread(redundancyTimerQueue, redundancyPort);
        /// share HeartbeatTxThread's UdpStack

        itsRedundancyWorkerThread = new RedundancyWorkerThread(redundancyTimerQueue, myHeartbeatTxThread->getTxUdpStack(), defaultSipPort, redundancyPort);
    }
    else
    {
        //declare this RS be active as no heartbeat is going 
        setRsState(RSACTIVE);
    }
    /// add instance()
    itsRsAgent = RsAgent::instance( defaultSipPort );
    itsRsAgent->run();

}


/**
 * Begin the RS proxy and its threads
 */
void
RSProxy::run()
{
    BasicProxy::run();

    if (itsRedundancyRxThread != 0) itsRedundancyRxThread->run();
    if (itsRedundancyWorkerThread != 0 ) itsRedundancyWorkerThread->run();
}

/**
 * Handle the ending of processes
 */
void
RSProxy::join()
{
    BasicProxy::join();
    BasicProxy::joinHeartbeatThread();

    if (itsRedundancyRxThread != 0) itsRedundancyRxThread->join();
    if (itsRedundancyWorkerThread != 0 ) itsRedundancyWorkerThread->join();
}

/**
 * Destructor
 */
RSProxy::~RSProxy()
{
    cpLog(LOG_DEBUG, "CALL destructor RSProxy");

}


/**
 * Obtain the SIP stack
 * @return  The SIP stack
 */
Sptr < SipTransceiver >
RSProxy::getSipStack() const
{
    return mySipStack;
}

/**
 * Obtain the current state of the RS
 * @return  The current state of the RS
 */
RSSTATE RSProxy::getRsState() const
{
    return itsState;
}

/**
 * Store the state of the RS
 * @param  newState  New state of the RS to be stored
 */
void RSProxy::setRsState( RSSTATE newState )
{
    itsState = newState;
}

/**
 * Obtain the port used for SIP messaging
 * @return  The SIP port
 */
unsigned short RSProxy::getLocalSipPort() const
{
    return localSipPort;
}

/**
 * Store the port to be used for SIP messaging
 * @param  sipPort  Port to be used for SIP messaging
 */
void RSProxy::setLocalSipPort(unsigned short sipPort)
{
    localSipPort = sipPort;
}

/**
 *  The sync port has been changed. Update the redundancy threads.
 * @param  oldSyncPort  The old port used for synchronization
 * @param  newSyncPort  The new port used for synchronization
 */
void RSProxy::updateRedundancyPort( unsigned short oldSyncPort, unsigned short newSyncPort)
{
    if(itsRedundancyWorkerThread != 0)
       itsRedundancyWorkerThread->setRedundancyPort( oldSyncPort, newSyncPort );
}
/// End of File
