#ifndef SIPCONTACTIF_HXX
#define SIPCONTACTIF_HXX

/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */

static const char* const SipContactIfVersion = "$Id: SipContactIf.hxx,v 1.26 2001/06/29 02:03:33 jason Exp $";


/**
 * This class deals with the processing of SIP messages.
 */

class SipContactIf
{
    public:
        /// Constructor
        SipContactIf();

        /**
         * Copy Constructor
         * @param  rhs  Object to copy from
         */
        SipContactIf(const SipContactIf& rhs);

        /**
         * Set object equal to specified object.
         * @param  rhs  Object to copy from
         * @return  This object
         */
        SipContactIf& operator=(const SipContactIf& rhs);

        /// Destructor
        virtual ~SipContactIf();

        /**
         * Update is triggered when a REGISTER message comes in to the RS.
         * The 'To' field is to be used as the key to search the Subscriber
         * container. The Term contacts are updated as well as the expire time.
         * Note: Update only deals with Subscribers
         * Note: The 'To' field only contains the user portion.
         * @param  to  Subscriber identifier to be updated.
         * @param  from  Identifier of individual who performed the registration.
         * @param  expires  Expire time in seconds until subscriber needs to be re-registered.
         * @param  newContacts  New terminating contact list to store
         * @param  existingContacts  Previously stored contact list 
         * @return Response code to be passed back in response to the Register.
         */
        int update( const Data& to, const Data& from,
                            Data& expires,
                            const Sptr < vector < Data > > & newcontacts,
                            vector < Data > & existingContacts );

        /**
         * Generate a contact list for the given originator and destination.
         * The contact will be generated accordingly:
         * If the Req Uri is found in the Subscriber container, then
         *    Originator's Calling Contacts
         *    Destination's Called Contacts
         *    Destination's Term Contacts
         *
         * Otherwise, if the Req Uri matches a key in one of the dial plans, then
         *    Originator's Calling Contacts
         *    Dial Plan Entry's Contacts
         * @param  orig  Originator's user field
         * @param  dest  Destination Sip address
         * @param  destUserIsPhone  Flag to indicate if destination's user field is a phone number or ip address
         * @param  contacts  Holder for the contact list to be returned
         * @return  Response code to be passed back in the status response to the Invite.
         */
         int generateContactList(const Data& orig, 
                                 const Data& dest, 
                                 bool destUserIsPhone, 
                                 bool & destIsRegistered,
                                 vector <Data> & contacts);


        /**
         * Generate a contact list for the given originator and destination but
         * only return the term contacts. This is used for options messages
         * being processed by the rs. 
         */
         int generateTermOnlyContactList(const Data& dest, 
                                         bool & destIsRegistered,
                                         vector <Data> & contacts);

        /**
         * Determine the location of the top Via in the contact list
         * @param  contacts  Contact list to be searched
         * @param  lastTowVias List of Via Sip addresses
         * @param  contactIter Location of the contact following the given Via
         * @return  True if Via was found, false otherwise
         */
         bool locateVia (vector <Data> & contacts, 
                         const vector < Data > & lastTwoVias, 
                         vector <Data>::iterator& contactIter);
         
        /**
         * Starting with the iterator location try to validate
         * a contact which will be used in a Status Response
         * to an INVITE message.
         * @param  contacts  Contact list to be searched
         * @param  contactIter Location of the first contact to be validated
         * @param  finalContact  Validated contact 
         * @return  Response code for Status Message
         */
         int obtainNextContact(const vector <Data> & contacts, 
                               vector <Data>::iterator& contactIter, 
                               bool destIsPhoneUser,
                               bool destIsRegistered,
                               Data & finalContact);
        /**
         * Starting with the iterator location try to validate
         * each contact which will then be placed in a vector to be 
         * returned and used in a Status Response to an INVITE message.
         * @param  contacts  Contact list to be searched
         * @param  contactIter Location of the first contact to be validated
         * @param  destIsPhoneUser If true user parm = phone, else user parm = ip
         * @param  destIsRegistered Dialled number is a registered user.
         * @param  finalContacts  Validated contacts to be returned
         * @return  Response code for Status Message
         */
        int
        SipContactIf::validateRemainingContacts(const vector <Data> & contacts,
                                vector <Data>::iterator& contactIter,
                                bool destIsPhoneUser,
                                bool destIsRegistered,
                                vector <Data> & finalContacts);

        /**
         *  Look for a wildcard within each contact and replace it
         *  with the given user name.
         *  @param  userName  String which will replace wildcards
         *  @param  contacts  Contact list to be modified
         */
         void replaceWildcard(const Data & userName, vector <Data> & contacts);

    protected:

        /**
         * strip off all but host:port of the contact
         * @param  contact  String to be manipulated
         * @return  Only the host:port portion of the passed in string.
         */
        string getHostPort(const Data& contact );

        /**
         * strip off all but user portion of the passed in SIP address
         * @param  sipAddress  String to be manipulated
         * @return  Only the user portion of the passed in string.
         */
        string getUser(const Data& sipAddress);

        /**
         * Strip off 'sip:' from the given string
         * @param  withSip  String to be manipulated
         * @return  String with 'sip:' removed
         */
        string stripOffSip(const Data& withSip);

        void prepareContact(Data & contact, const Data & userField, const bool isPhone);

#ifdef VOCAL_HAS_ENUM
        /*
         * Query ENUM (E164 to Url Mapping) to obtain a Sip url for the 
         * given E.164 number. 
         * @param  userField  E.164 number to be used in the query 
         * @param  resultingUrl  Corresponding Sip Url for given E.164 number.
         * @return  True if query returned a valid sip url, false otherwise
         */
        bool queryEnum(Data & userField, Data & resultingUrl);
#endif

    private:
        /**
         * Copy data from specified object into this object.
         * @param  rhs  Object from which to copy.
         */
        void copyRhsToThis(const SipContactIf& rhs);

};

#endif //SIPCONTACTIF_HXX
