#ifndef Subscriber_HXX
#define Subscriber_HXX

/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */

#include <vector>
#include "Data.hxx"
#include "Sptr.hxx"
#include "VTime.hxx"

static const char* const SubscriberVersion = "$Id: Subscriber.hxx,v 1.25 2001/01/31 01:53:20 marcl Exp $";

/**
   A subscriber is the local copy of all the data required to
   direct calls to a registered user. Stored within are the
   contact lists made up of feature servers which is obtained
   from provisioning. The Term contact list and expiry time are
   obtained from a Register message. 

   Multiple identifiers can be used to access this subscriber.
   One of the identifiers is the Master user and the rest are
   called Aliases. 
 */
class Subscriber
{
    public:

        /**
         * Constructor
         */
        Subscriber();

        /**
         * Constructor with data
         * @param  name  Master name
         * @param  calling  Contact list of subscriber's calling features.
         * @param  called  Contact list of subscriber's called features.
         * @param  staticFlag  Mark subscriber's static registration status.
         */
        Subscriber( const Data& name,
                    const Sptr < vector < Data > > calling,
                    const Sptr < vector < Data > > called,
                    const bool staticFlag);

        /**
         * Copy Constructor 
         * @param  rhs  Subscriber object to copy
         */
        Subscriber::Subscriber( const Subscriber& rhs );

        /**
         * Set this subscriber equal to the given subscriber.
         * @param  rhs  Subscriber object to copy.
         * @return Reference to this subscriber
         */
        Subscriber& Subscriber::operator=( const Subscriber& rhs );

        /**
         *  destructor
         */
        virtual ~Subscriber();

        /// Provisioned State constants
        enum provisionedState { MasterUnprovisioned = 0, AliasUnprovisioned, Provisioned};

        /**
         * Get the feature contact list used when this subscriber is making a call.
         * @param  contacts  List to which subscriber's calling contacts will be appended.
         */
        void appendCalling(vector < Data > & contacts ) const;

        /**
         * Get the feature contact list used when terminating on this subscriber.
         * @param  contacts  List to which subscriber's called contacts will be appended.
         */
        void appendCalled(vector < Data > & contacts ) const;

        /**
         * Get the contact list used when terminating on this subscriber.
         * @param  contacts  List to which subscriber's Term contacts will be appended.
         */
        void appendTerm(vector < Data > & contacts ) const;


        /**
         * Store the feature contacts used when this subscriber makes a call.
         * @param  calling  List of contacts for calls made from this subscriber.
         */
        void setCalling( const Sptr < vector < Data > > calling );

        /**
         * Store the feature contacts used when a call terminates on this subscriber.
         * @param  called  List of contacts for calls made to this subscriber.
         */
        void setCalled( const Sptr < vector < Data > > called );

        /**
         * Store the contacts used when a call terminates on this subscriber.
         * @param  term  List of contacts for calls made to this subscriber.
         */
        void setTerm( const Sptr < vector < Data > > term );

        /**
         * Erase the feature contacts used when this subscriber makes a call.
         */
        void clearCalling();

        /**
         * Erase the feature contacts used when a call is made to this subscriber.
         */
        void clearCalled();

        /**
         * Erase the contacts used when a call is made to this subscriber.
         */
        void clearTerm();

        /**
         * Get the amount of seconds remaining until registration becomes expired.
         * @return Seconds remaining before expiry.
         */
        const unsigned long getExpiry( ) const;

        /**
         * Store the expiry time remaining before re-registration.
         * @param  expire  Time in seconds before another register message is required.
         */
        void setExpiry( const unsigned long expire );

        /**
         * Store the current time to mark when this Subscriber was last registered.
         */
        void setRegTime();

        /**
         * Clear the expiry timer.
         */
        void clearTimer();

        /**
         * Check if subscriber's registration has expired and deregister if necessary.
         * @param  secondsRemaining  Amount of time in seconds before re-registration is required.
         * @return true if subscriber is expired, false otherwise.
         */
        bool expired(long & secondsRemaining );

        /**
         *  Check if subscriber has static registration
         *  @return  true if subscriber is statically registered, false otherwise.
         */
        bool isStaticReg() const
        {
            return StaticReg;
        }
        
        /**
         *  Set subscriber's static registration flag
         *  @param  flag  Subscriber's static registration status
         */
        void setStaticFlag(const bool flag)
        {
            StaticReg = flag;
        }

        /**
         *  Check if Subscriber has gotten data from provisioning yet
         *  @return  true if provisioning data has been obtained for this subscriber
         */
        bool isProvisioned() const;

        /**
         *  Mark the subscriber as having received data from provisioning
         *  @param  flag  Subscriber's provisioned status
         */
        void setProvisioned(const bool flag);

        /**
         * Check to see if subscriber is an alias.
         * @param  name  Subscriber's identifier.
         * @return true if subscriber is an alias, false otherwise.
         */
        bool isAlias(const Data& name) const;

        /**
         *  Mark the subscriber as being for an unprovisioned Alias or Master
         *  Once provisioned, it could be representing both.
         *  @param  flag  Subscriber's alias status
         */
        void setAlias(const bool flag);

        /**
         * Get the subscriber's master name
         * @return  Master name 
         */
        Data getMasterName() const
        {
            return masterName;
        }
       
        /**
         * Set the subscriber's master name
         * @param  name  Master name for this subscriber
         */
        void setMasterName( const Data name)
        {
            masterName = name;
        }

    private:

        /**
         * Copy the data in the given subscriber into this subscriber.
         * @param  rhs  Subscriber object to copy.
         */
        void Subscriber::copyRhsToThis( const Subscriber& rhs );

        /** Contacts from provisioning which are used when this
         *  subscriber originates a call.
         */
        Sptr < vector < Data > > CallingContacts;

        /** Contacts from provisioning which are used when a
         *  call terminates on this subscriber.
         */
        Sptr < vector < Data > > CalledContacts;

        /** Contacts from Registration which are used when a
         *  call terminates on this subscriber.
         */
        Sptr < vector < Data > > TermContacts;

        /// Time in seconds before subscriber's registration becomes expired.
        unsigned long int secTilExpired;

        /// Time the Subscriber became registered
        Sptr < VTime > RegTime;

        /** Flag representing whether or not subscriber is
         *  statically registered. Static registrations never expire.
         */
        bool StaticReg;

        /// Keeps track of whether or not subscriber's data has been obtained 
        /// from provisioning. Before provisioning is accessed a subscriber belongs
        /// to either an alias or a master user. Once provisioned, the subscriber 
        /// could be representing both.
        provisionedState ProvisionedState;

        /** A master user and its aliases all share the same subscriber
         *  object. This identifies the master user name.
         */
        Data masterName;

};

#endif
