#if !defined(VOCAL_DEFAULT_ADMISSION_POLICY_HXX)
#define VOCAL_DEFAULT_ADMISSION_POLICY_HXX

/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */


static const char* const DefaultAdmissionPolicy_hxx_Version = 
    "$Id: DefaultAdmissionPolicy.hxx,v 1.10 2001/11/07 08:57:15 icahoon Exp $";


#include "ptlib.h"
#include "h323pdu.h"
#include <string>
#include "AdmissionPolicy.hxx"
#include "Sptr.hxx"
#include "IPAddress.hxx"
#include "EndpointInfo.hxx"
#include "EndpointInfoMap.hxx"


/** Vovida Open Communication Application Library<br><br>
 */
namespace Vocal
{


/** H.323 Gatekeeper<br><br>
 */
namespace H323GK
{


using Vocal::SipH323Info::EndpointInfoSet;


/** Default policy for determining how to handle RAS admission and disengage
 *  messages.<br><br>
 *
 *  @see    Vocal::H323GK::AdmissionPolicy
 *  @see    Vocal::H323GK::RASPolicy
 *  @see    Vocal::H323GK::RASGKProtocol
 *  @see    Vocal::SipH323Info::GatekeeperInfo
 *  @see    Vocal::SipH323Info::EndpointInfoMap
 *  @see    Vocal::SipH323Info::EndpointInfo
 *  @see    Vocal::Services::Service
 *  @see    Vocal::Transport::IPAddress
 */
class DefaultAdmissionPolicy : public Vocal::H323GK::AdmissionPolicy
{

    public:


    	/** Construct given the controling service, the associated ras gk
	 *  protocol, the configuration information for the gatekeeper and
	 *  the table of registered endpoints.
	 */
    	DefaultAdmissionPolicy(
	    Service &,
	    RASGKProtocol &,
	    GatekeeperInfo &,
	    EndpointInfoMap &
	);


    	/** Virtual destructor
	 */
	virtual		    ~DefaultAdmissionPolicy();
			

    	/** Callback to handle an incoming admission request.
	 *
	 *  This will first validate the incoming arq. It will then give
	 *  the user an opportunity to process the incoming message by calling
	 *  the callback onAdmission. On confirmation, an ACF is sent.
	 *  On rejection, an ARJ is sent.
	 */
    	void        	    processAdmissionRequest(
	    	    	    	Sptr<H323RasPDU>    rasPDU, 
			    	Sptr<IPAddress>     remoteAddr);


    	/** Callback to handle an incoming disengage request.
	 *
	 *  This will first validate the incoming drq. It will then give
	 *  the user an opportunity to process the incoming message by calling
	 *  the callback onDisengage. On confirmation, an DCF is sent.
	 *  On rejection, an DRJ is sent.
	 */
    	void   	    	    processDisengageRequest(
	    	    	    	Sptr<H323RasPDU>    rasPDU,
			    	Sptr<IPAddress>     remoteAddr);
			    

    	/** Callback to handle an incoming disengage response.
	 *
	 *  This will give the user an opportunity to process the incoming 
	 *  message by calling the callback onDisengageResponse.
	 */
    	void   	    	    processDisengageResponse(
	    	    	    	Sptr<H323RasPDU>    rasPDU,
			    	Sptr<IPAddress>     remoteAddr);
	    


    	/** Builds and sends a disengage request.
    	 *  Will finalize before sending.
	 */
    	void	    	    disengage(
				H225_DisengageReason 	    &	reason,
				Sptr<EndpointInfo>  	    	endpointInfo,
			    	Sptr<IPAddress>     	    	remoteAddr);


    	/** Write the policy to an ostream. 
	 */
    	ostream &           writeTo(ostream & out) const;


    protected:


	/** Called on receipt of an admission request. 
	 *
    	 *  On wait, the processing stops. It is the responsibility of
    	 *  the derived class to retain message state.
	 *
    	 *  On confirm, the processing continues processing the request.
	 *
    	 *  On reject, a admission reject is sent, and the user needs
    	 *  to supply a reason.
	 */
    	virtual Proceed	    onAdmission(
				Sptr<H323RasPDU>       	    	rasPDU, 
				Sptr<IPAddress> 	    	remoteAddr,
    	    	    	    	Sptr<EndpointInfo>  	    	srcEptInfo,
				Sptr<EndpointInfo>  	    	destEptInfo,
	    	    	    	H225_AdmissionRequest	    &   arq,
				H225_AdmissionRejectReason  &	reason);


    	/** Called before the acf is sent.
	 */
    	virtual void	    finalizeAdmissionConfirm(
	    	    	    	H225_AdmissionRequest	&   arq,
	    	    	    	H225_AdmissionConfirm	&   acf,
				Sptr<EndpointInfo>  	    srcEndpointInfo,
				Sptr<EndpointInfo>  	    destEndpointInfo);


    	/** Called befire the arj is sent.
	 */
    	virtual void	    finalizeAdmissionReject(
	    	    	    	H225_AdmissionRequest	&   arq,
	    	    	    	H225_AdmissionReject    &   arj);

	/** Called on receipt of a disengage request. 
    	 *  
    	 *  On wait, the processing stops. It is the responsibility of
    	 *  the derived class to retain message state.
    	 *  
    	 *  On confirm, the processing continues processing the request.
    	 *  	 
    	 *  On reject, a disengage reject is sent, and the user needs
    	 *  to supply a reason.
	 */
    	virtual Proceed     onDisengage(
				Sptr<H323RasPDU>	    	rasPDU, 
				Sptr<IPAddress> 	    	remoteAddr,
    	    	    	    	Sptr<EndpointInfo>  	    	endpointInfo,
	    	    	    	H225_DisengageRequest 	    &   drq,
				H225_DisengageRejectReason  &   reason);
	

    	/** Called before the drq is sent.
	 */
    	virtual void	    finalizeDisengageRequest(
	    	    	    	H225_DisengageRequest   &   drq,
				Sptr<EndpointInfo>  	    endpointInfo);


    	/** Called before the dcf is sent.
	 */
    	virtual void	    finalizeDisengageConfirm(
	    	    	    	H225_DisengageRequest   &   drq,
	    	    	    	H225_DisengageConfirm   &   dcf,
				Sptr<EndpointInfo>  	    endpointInfo);


    	/** Called befire the drj is sent.
	 */
    	virtual void	    finalizeDisengageReject(
	    	    	    	H225_DisengageRequest   &   drq,
	    	    	    	H225_DisengageReject    &   drj);


	/** Called on receipt of a disengage response. 
	 */
    	virtual void	    onDisengageResponse(
    	    	    	    	Sptr<H323RasPDU>    	    rasPDU);


    	/** Returns the address where the response will be sent.
    	 *  If the arq contains a valid rasAddress, that will be used, 
    	 *  otherwise the remote address will be used.
	 */
    	Sptr<IPAddress>     responseAddress(
				Sptr<EndpointInfo>  	    endpointInfo,
			    	Sptr<IPAddress>     	    remoteAddr);


    	/** Ensures the admission request is sane. If the request
    	 *  is sane it returns true, otherwise it returns false.
    	 *  If true is returned, the srcEndpointInfo and destEndpointInfo 
    	 *  are populated. If false is returned, the reason is populated.
	 */	
    	virtual bool	    validateARQ(
	    	    	    	H225_AdmissionRequest	    &   arq,
    	    	    	    	Sptr<EndpointInfo>  	    &	srcEndpointInfo,
				Sptr<EndpointInfo>  	    &	destEndpointInfo,
				H225_AdmissionRejectReason  &	reason,
				Sptr<IPAddress>     	    &	remoteAddr);


    	/** Builds and sends the confirmation to the admission request.
    	 *  Will finalize before sending.
	 */
    	virtual void	    confirmARQ(
	    	    	    	H225_AdmissionRequest   &   arq, 
				Sptr<EndpointInfo>  	    srcEndpointInfo,
				Sptr<EndpointInfo>  	    destEndpointInfo,
			    	Sptr<IPAddress>     	    remoteAddr);


    	/** Builds and sends the rejection to the admission request.
    	 *  Will finalize before sending.
	 */
    	virtual void	    rejectARQ(
	    	    	    	H225_AdmissionRequest       &   arq, 
				H225_AdmissionRejectReason  &	reason,
			    	Sptr<IPAddress>     	    	remoteAddr);


    	/** Ensures the disengage request is sane. If the request
    	 *  is sane it returns true, otherwise it returns false. 
	 */	
    	virtual bool	    validateDRQ(
	    	    	    	H225_DisengageRequest	    &   drq,
    	    	    	    	Sptr<EndpointInfo>  	    &	endpointInfo,
				H225_DisengageRejectReason  &   reason);


    	/** Builds and sends the confirmation to the disengage request.
    	 *  Will finalize before sending.
	 */
    	void	    	    confirmDRQ(
	    	    	    	H225_DisengageRequest   &   drq, 
				Sptr<EndpointInfo>  	    endpointInfo,
			    	Sptr<IPAddress>     	    remoteAddr);


    	/** Builds and sends the rejection to the disengage request.
    	 *  Will finalize before sending.
	 */
    	void	    	    rejectDRQ(
	    	    	    	H225_DisengageRequest       &   drq,
				H225_DisengageRejectReason  &   reason,
			    	Sptr<IPAddress>     	    	remoteAddr);



    	/** Find an gateway that matches the given alias. 
	 */
	Sptr<EndpointInfo>  findGateway(H225_ArrayOf_AliasAddress & aliases);


        /** Given a set of gateways that can route given the same prefix,
         *  choose the destination gateway. The default behavior will be
         *  to choose the first gateway.
         */
        Sptr<EndpointInfo>  chooseGateway(const EndpointInfoSet &);
};


} // namespace H323GK
} // namespace Vocal


#endif // !defined(VOCAL_DEFAULT_ADMISSION_POLICY_HXX)
