/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */


static const char* const DefaultDiscoveryPolicy_cxx_Version = 
  "$Id: DefaultDiscoveryPolicy.cxx,v 1.16 2001/12/17 23:58:33 icahoon Exp $";


#include "DefaultDiscoveryPolicy.hxx"
#include "GatekeeperInfo.hxx"
#include "EndpointInfoMap.hxx"
#include "RASGKProtocol.hxx"
#include "VLog.hxx"


using Vocal::H323GK::DefaultDiscoveryPolicy;
using Vocal::H323GK::DiscoveryPolicy;
using Vocal::H323GK::RASGKProtocol;
using Vocal::SipH323Info::GatekeeperInfo;
using Vocal::SipH323Info::EndpointInfoMap;
using Vocal::SipH323Info::EndpointInfo;
using Vocal::Services::Service;
using Vocal::Transport::IPAddress;
using Vocal::Logging::VLog;


DefaultDiscoveryPolicy::DefaultDiscoveryPolicy(
    Service  	    	&   service,
    RASGKProtocol   	&   rasGkProtocol,
    GatekeeperInfo  	&   gatekeeperInfo,
    EndpointInfoMap 	&   endpointInfoMap
)
    :	DiscoveryPolicy(service, rasGkProtocol, gatekeeperInfo, endpointInfoMap)
{
}


DefaultDiscoveryPolicy::~DefaultDiscoveryPolicy()
{
}


void    	
DefaultDiscoveryPolicy::processMulticastDiscoveryRequest(
    Sptr<H323RasPDU>	    rasPDU, 
    Sptr<IPAddress> 	    remoteAddr
)
{
    const string    fn("DefaultDiscoveryPolicy::processMulticastDiscoveryRequest");
    VLog    	    log(fn);
    
    H225_GatekeeperRequest	&   grq = *rasPDU;

    // Setup the response address.
    //
    remoteAddr = responseAddress(grq, remoteAddr);

    H225_GatekeeperRejectReason     reason(H225_GatekeeperRejectReason::e_undefinedReason);;

    // Let's first see if we can make some sense of it.
    //
    if ( !validateGRQ(grq, reason) )
    {
    	// No rejection for multicast
	//
    	return;
    }

    // Let the derived class decide how to process it.
    //
    Proceed proceed = onMulticastDiscovery(rasPDU, remoteAddr, grq, reason);

    switch ( proceed )
    {
	case wait:
	{
	    // The gatekeeper is probably waiting for an external 
	    // event to drive the discovery forward. 
	    //
	    VDEBUG(log) << fn << ": waiting." << VDEBUG_END(log);
	    return;
	}

	case confirm:
	{
	    // Confirm
	    //
	    confirmGRQ(grq, remoteAddr);

    	    VDEBUG(log) << fn << ": Multicast discovery, address = " 
		       << *remoteAddr << VDEBUG_END(log);
	    break;
	}
	default:
	{
    	    rejectGRQ(grq, reason, remoteAddr);

	    break;
	}
    }
}


void    	
DefaultDiscoveryPolicy::processUnicastDiscoveryRequest(
    Sptr<H323RasPDU>	    rasPDU, 
    Sptr<IPAddress> 	    remoteAddr
)
{
    const string    fn("DefaultDiscoveryPolicy::processUnicastDiscoveryRequest");
    VLog    	    log(fn);
    
    H225_GatekeeperRequest	&   grq = *rasPDU;

    // Setup the response address.
    //
    remoteAddr = responseAddress(grq, remoteAddr);

    H225_GatekeeperRejectReason     reason(H225_GatekeeperRejectReason::e_undefinedReason);;

    // Let's first see if we can make some sense of it.
    //
    if ( !validateGRQ(grq, reason) )
    {
    	rejectGRQ(grq, reason, remoteAddr);
    	return;
    }

    // Let the derived class decide how to process it.
    //
    Proceed proceed = onUnicastDiscovery(rasPDU, remoteAddr, grq, reason);

    switch ( proceed )
    {
	case wait:
	{
	    // The gatekeeper is probably waiting for an external 
	    // event to drive the discovery forward. 
	    //
	    VDEBUG(log) << fn << ": waiting." << VDEBUG_END(log);
	    return;
	}

	case confirm:
	{
	    // Confirm
	    //
	    if ( validateGRQ(grq, reason) )
	    {
		confirmGRQ(grq, remoteAddr);

    		VDEBUG(log) << fn << ": Unicast discovery, address = " 
			   << *remoteAddr << VDEBUG_END(log);
    	    	break;
	    }

    	    // Reject falls through
	    //
	}
	default:
	{
    	    rejectGRQ(grq, reason, remoteAddr);

	    break;
	}
    }
}


ostream &           
DefaultDiscoveryPolicy::writeTo(ostream & out) const
{
    return ( out << "DefaultDiscoveryPolicy" );
}


Vocal::H323GK::RASPolicy::Proceed	    
DefaultDiscoveryPolicy::onMulticastDiscovery(
    Sptr<H323RasPDU>	    	    rasPDU, 
    Sptr<IPAddress> 	    	    remoteAddr,
    H225_GatekeeperRequest      &   grq,
    H225_GatekeeperRejectReason &   reason
)
{
    return ( confirm );
}


Vocal::H323GK::RASPolicy::Proceed	    
DefaultDiscoveryPolicy::onUnicastDiscovery(
    Sptr<H323RasPDU>	    	    rasPDU, 
    Sptr<IPAddress> 	    	    remoteAddr,
    H225_GatekeeperRequest      &   grq,
    H225_GatekeeperRejectReason &   reason
)
{
    return ( confirm );
}


void	    
DefaultDiscoveryPolicy::finalizeDiscoveryConfirm(
    H225_GatekeeperRequest      &   grq,
    H225_GatekeeperConfirm      &   gcf
)
{
    // By default do nothing.
}


void	    
DefaultDiscoveryPolicy::finalizeDiscoveryReject(
    H225_GatekeeperRequest  	&   grq,
    H225_GatekeeperReject       &   grj
)
{
    // By default do nothing.
}


Sptr<IPAddress>     
DefaultDiscoveryPolicy::responseAddress(
    H225_GatekeeperRequest  	&   grq, 
    Sptr<IPAddress>     	    remoteAddr
)
{
    // Get the reply to ras address.
    //
    if ( grq.m_rasAddress.GetTag() == H225_TransportAddress::e_ipAddress )
    {
    	H225_TransportAddress_ipAddress & rasIPAddr = grq.m_rasAddress;

    	remoteAddr = new IPAddress(rasIPAddr.m_ip.GetPointer(), 
	    	    	    	   rasIPAddr.m_port);
    }
    
    return ( remoteAddr );
}


bool  	    
DefaultDiscoveryPolicy::validateGRQ(
    H225_GatekeeperRequest  	&   grq,
    H225_GatekeeperRejectReason &   reason
)
{
    const string    fn("DefaultDiscoveryPolicy::validateGRQ");
    VLog log(fn);

    if  (   grq.HasOptionalField(H225_GatekeeperRequest::e_gatekeeperIdentifier) 
    	&&  grq.m_gatekeeperIdentifier.GetValue() 
    	        != myGatekeeperInfo.gatekeeperIdentifier().c_str()
    	)
    {
    	reason.SetTag(H225_GatekeeperRejectReason::e_undefinedReason);
	return ( false );
    }

    return ( true );
}


void	    
DefaultDiscoveryPolicy::confirmGRQ(
    H225_GatekeeperRequest  	&   grq, 
    Sptr<IPAddress>     	    remoteAddr
)
{
    const string    fn("DefaultDiscoveryPolicy::confirmGRQ");
    VLog    	    log(fn);
    
    // GatekeeperConfirm
    //  {
    //      requestSeqNum               := [from GRQ]
    //      protocolIdentifier          := [from openh323]
    //      gatekeeperIdentifier        := [from GatekeeperInfo]
    //      rasAddress                  := [from GatekeeperInfo]
    //  }

    H323RasPDU	    	    	    pdu;
    pdu.SetTag(H225_RasMessage::e_gatekeeperConfirm);

    H225_GatekeeperConfirm	&   gcf = pdu;
    
    gcf.m_requestSeqNum = grq.m_requestSeqNum;
    
    gcf.m_protocolIdentifier.SetValue(myGatekeeperInfo.h225ProtocolID().c_str());
						    
    gcf.IncludeOptionalField(H225_GatekeeperConfirm::e_gatekeeperIdentifier);
    gcf.m_gatekeeperIdentifier = myGatekeeperInfo.gatekeeperIdentifier().c_str();

    gcf.m_rasAddress.SetTag(H225_TransportAddress::e_ipAddress);
    H225_TransportAddress_ipAddress &	rasIPAddr = gcf.m_rasAddress;
    memcpy(&rasIPAddr.m_ip[0], &(myGatekeeperInfo.rasAddress().getIPAddress().sin_addr.s_addr), 4);
    rasIPAddr.m_port = myGatekeeperInfo.rasAddress().getPort();

    // Give the derived policy the last crack at the message.
    //
    finalizeDiscoveryConfirm(grq, gcf);

    // Ship it.
    //
    char    	buf[256];
    ostrstream	confirmLog(buf, 256);

    confirmLog << "Discovery, address = { " << *remoteAddr << " }" << ends;

    myRasGkProtocol.discoveryConfirm(pdu, remoteAddr, confirmLog);
}


void	    
DefaultDiscoveryPolicy::rejectGRQ(
    H225_GatekeeperRequest          &   grq, 
    H225_GatekeeperRejectReason     &   reason,
    Sptr<IPAddress>     	    	remoteAddr
)
{
    const string    fn("DefaultDiscoveryPolicy::rejectGRQ");
    VLog    	    log(fn);
    
    // GatekeeperReject
    // {
    //     requestSeqNum		:= [from GRQ]
    //     protocolIdentifier	    	:= [from openh323]
    //     gatekeeperIdentifier	    	:= [from GatekeeperInfo]
    //     rejectReason		    	:= [param]
    // }

    H323RasPDU	    	    	    pdu;
    pdu.SetTag(H225_RasMessage::e_gatekeeperReject);

    H225_GatekeeperReject	&   grj = pdu;
    
    grj.m_requestSeqNum = grq.m_requestSeqNum;
    
    grj.m_protocolIdentifier.SetValue(myGatekeeperInfo.h225ProtocolID().c_str());
						    
    grj.IncludeOptionalField(H225_GatekeeperReject::e_gatekeeperIdentifier);
    grj.m_gatekeeperIdentifier = myGatekeeperInfo.gatekeeperIdentifier().c_str();

    grj.m_rejectReason = reason;

    // Give the derived policy the last crack at the message.
    //
    finalizeDiscoveryReject(grq, grj);

    // Ship it.
    //
    myRasGkProtocol.discoveryReject(pdu, remoteAddr);
}
