#if !defined(VOCAL_DEFAULT_DISCOVERY_POLICY_HXX)
#define VOCAL_DEFAULT_DISCOVERY_POLICY_HXX

/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */


static const char* const DefaultDiscoveryPolicy_hxx_Version = 
"$Id: DefaultDiscoveryPolicy.hxx,v 1.11 2001/10/13 01:15:53 icahoon Exp $";


#include "ptlib.h"
#include "h323pdu.h"
#include <string>
#include "DiscoveryPolicy.hxx"
#include "IPAddress.hxx"


/** Vovida Open Communication Application Library<br><br>
 */
namespace Vocal
{


/** H.323 Gatekeeper<br><br>
 */
namespace H323GK
{



/** Default policy for determining how to handle RAS discovery messages.<br><br>
 *
 *  @see    Vocal::H323GK::DiscoveryPolicy
 *  @see    Vocal::H323GK::RASPolicy
 *  @see    Vocal::H323GK::RASGKProtocol
 *  @see    Vocal::SipH323Info::GatekeeperInfo
 *  @see    Vocal::SipH323Info::EndpointInfoMap
 *  @see    Vocal::Services::Service
 *  @see    Vocal::Transport::IPAddress
 */
class DefaultDiscoveryPolicy : public Vocal::H323GK::DiscoveryPolicy
{

    public:
    

    	/** Construct given the controling service, the associated ras gk
	 *  protocol, the configuration information for the gatekeeper and
	 *  the table of registered endpoints.
	 */
    	DefaultDiscoveryPolicy(
	    Service &,
	    RASGKProtocol &,
	    GatekeeperInfo &,
	    EndpointInfoMap &
	);


    	/** Virtual destructor
	 */
	virtual		    ~DefaultDiscoveryPolicy();
			

    	/** Called to process an incoming multicast discovery.
    	 *
	 *  This will first validate the incoming grq. It will then give
	 *  the user an opportunity to process the incoming message by calling
	 *  the callback onMulticastDiscovery. On confirmation, an GCF is sent.
	 *  On rejection, an GRJ is sent.
	 */
    	void 	    	    processMulticastDiscoveryRequest(
	    	    	    	Sptr<H323RasPDU>    rasPDU, 
			    	Sptr<IPAddress>     remoteAddr);


    	/** Called to process an incoming unicast discovery.
    	 *
	 *  This will first validate the incoming grq. It will then give
	 *  the user an opportunity to process the incoming message by calling
	 *  the callback onUnicastDiscovery. On confirmation, an GCF is sent.
	 *  On rejection, an GRJ is sent.
	 */
    	void 	    	    processUnicastDiscoveryRequest(
	    	    	    	Sptr<H323RasPDU>    rasPDU, 
			    	Sptr<IPAddress>     remoteAddr);


    	/** Write the policy to an ostream. 
	 */
    	ostream &           writeTo(ostream & out) const;

				
    protected:


	/** Called on receipt of a multicast discovery request. 
	 *  
	 *  On wait, the processing stops. It is the responsibility of
	 *  the derived class to retain message state.
	 *
	 *  On confirm, the processing continues processing the request.
	 *
	 *  On reject, a gatekeeper reject is sent, and the user needs
	 *  to supply a reason.
	 */
    	virtual Proceed	    onMulticastDiscovery(
				Sptr<H323RasPDU>	    	rasPDU, 
				Sptr<IPAddress> 	    	remoteAddr,
	    	    	    	H225_GatekeeperRequest      &	grq,
				H225_GatekeeperRejectReason &   reason);


	/** Called on receipt of a unicast discovery request. 
	 *  
	 *  On wait, the processing stops. It is the responsibility of
	 *  the derived class to retain message state.
	 *
	 *  On confirm, the processing continues processing the request.
	 *  
	 *  On reject, a gatekeeper reject is sent, and the user needs
	 *  to supply a reason.
	 */
    	virtual Proceed	    onUnicastDiscovery(
				Sptr<H323RasPDU>	    	rasPDU, 
				Sptr<IPAddress> 	    	remoteAddr,
	    	    	    	H225_GatekeeperRequest      &	grq,
				H225_GatekeeperRejectReason &   reason);


    	/** Called before the gcf is sent.
	 */
    	virtual void	    finalizeDiscoveryConfirm(
	    	    	    	H225_GatekeeperRequest      &	grq,
	    	    	    	H225_GatekeeperConfirm      &	gcf);


    	/** Called befire the rrj is sent.
	 */
    	virtual void	    finalizeDiscoveryReject(
	    	    	    	H225_GatekeeperRequest	    &	grq,
	    	    	    	H225_GatekeeperReject       &	grj);


    	/** Returns the address where the response will be sent.
	 *  If the grq contains a valid rasAddress, that will be used, 
	 *  otherwise the remote address will be used.
	 */
    	Sptr<IPAddress>     responseAddress(
	    	    	    	H225_GatekeeperRequest	    &   grq, 
			    	Sptr<IPAddress>     	    	remoteAddr);


    	/** Ensures the multicast gatekeeper request is sane. If the request
	 *  is sane it returns true, otherwise it returns false. 
	 */	
    	virtual bool  	    validateGRQ(
	    	    	    	H225_GatekeeperRequest	    &  grq,
				H225_GatekeeperRejectReason &   reason);


    	/** Builds and sends the rejection to the registration request.
	 *  Will finalize before sending.
	 */
    	virtual void	    confirmGRQ(
	    	    	    	H225_GatekeeperRequest  &   grq, 
			    	Sptr<IPAddress>     	    remoteAddr);


    	/** Builds and sends the rejection to the registration request.
	 *  Will finalize before sending.
	 */
    	virtual void	    rejectGRQ(
	    	    	    	H225_GatekeeperRequest      &   grq, 
				H225_GatekeeperRejectReason &   reason,
			    	Sptr<IPAddress>     	    	remoteAddr);
};


} // namespace H323GK
} // namespace Vocal


#endif // !defined(VOCAL_DEFAULT_DISCOVERY_POLICY_HXX)
