/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */


static const char* const DefaultRegistrationPolicy_cxx_Version = 
    "$Id: DefaultRegistrationPolicy.cxx,v 1.25 2001/10/15 19:08:20 icahoon Exp $";


#include "DefaultRegistrationPolicy.hxx"
#include "GatekeeperInfo.hxx"
#include "EndpointInfoMap.hxx"
#include "RASGKProtocol.hxx"
#include "KeepAliveTimerEvent.hxx"
#include "Service.hxx"
#include "VLog.hxx"
#include <strstream>


using Vocal::H323GK::DefaultRegistrationPolicy;
using Vocal::H323GK::AdmissionPolicy;
using Vocal::Behavioral::EventObserver;
using Vocal::H323GK::RASGKProtocol;
using Vocal::H323GK::KeepAliveTimerEvent;
using Vocal::SipH323Info::GatekeeperInfo;
using Vocal::SipH323Info::EndpointInfoMap;
using Vocal::SipH323Info::EndpointInfo;
using Vocal::Services::Service;
using Vocal::Transport::IPAddress;
using Vocal::Services::Event;
using Vocal::Logging::VLog;


DefaultRegistrationPolicy::DefaultRegistrationPolicy(
    Service  	    	&   service,
    RASGKProtocol   	&   rasGkProtocol,
    GatekeeperInfo  	&   gatekeeperInfo,
    EndpointInfoMap 	&   endpointInfoMap
)
    :	RegistrationPolicy(service, rasGkProtocol, gatekeeperInfo, endpointInfoMap),
    	EventObserver< Sptr<Event> >(service.getEventSubject())
{
    myService.subscribe(*this);
}


DefaultRegistrationPolicy::~DefaultRegistrationPolicy()
{
    myService.unsubscribe(*this);
}


bool	    
DefaultRegistrationPolicy::interestedIn(const Sptr<Event> & event)
{
    Sptr<KeepAliveTimerEvent>	kaTimerEvent;
    kaTimerEvent.dynamicCast(event);
    
    return ( kaTimerEvent != 0 );
}


void	    
DefaultRegistrationPolicy::onEvent(Sptr<Event> event)
{
    const string    fn("DefaultRegistrationPolicy::onEvent");
    VLog    	    log(fn);

    Sptr<KeepAliveTimerEvent>	kaTimerEvent;
    kaTimerEvent.dynamicCast(event);

    if ( kaTimerEvent == 0 )
    {
    	return;
    }
    
    // Find the endpoint associated with this timer.
    //
    const string  & eptId = kaTimerEvent->endpointIdentifier();
    
    Sptr<EndpointInfo>	endpointInfo = myEndpointInfoMap.findByEndpointId(eptId);

    // Defensive. This would be the situation where the endpoint was
    // unregistered between the start and expiry of the timer. The
    // unregistration should have cancelled the timer, but apparently didn't.
    //
    if ( endpointInfo == 0 )
    {
    	VWARN(log)  << fn << ": keep alive timer expired"
	    	    << ", no associated endpoint = " << eptId << VWARN_END(log);
    	return;
    }

    H225_UnregRequestReason reason(H225_UnregRequestReason::e_ttlExpired);

    Sptr<IPAddress> remoteAddr = new IPAddress(endpointInfo->rasAddress());
    
    unregister(reason, endpointInfo, remoteAddr);
}


void    	
DefaultRegistrationPolicy::processRegistrationRequest(
    Sptr<H323RasPDU>	    rasPDU, 
    Sptr<IPAddress> 	    remoteAddr
)
{
    const string    fn("DefaultRegistrationPolicy::processRegistrationRequest");
    VLog    	    log(fn);

    H225_RegistrationRequest	&   rrq = *rasPDU;

    H225_RegistrationRejectReason   reason(H225_RegistrationRejectReason::e_undefinedReason);
    Sptr<EndpointInfo>      	    endpointInfo = 0;

    // Handle the light weight registration request.
    //
    if ( isKeepAlive(rrq) )
    {
	// Let's first see if we can make some sense of it.
	//
	if ( !validateKeepAliveRRQ(rrq, endpointInfo, reason) )
	{
    	    rejectRRQ(rrq, reason, remoteAddr, true);

    	    removeEndpoint(endpointInfo);

	    return;
	}
	assert( endpointInfo != 0 );

    	remoteAddr = responseAddress(&rrq, endpointInfo, remoteAddr);

    	// Let the derived class decide how to process it.
	//
    	Proceed proceed = onKeepAliveRegistration(rasPDU, remoteAddr, 
	    	    	    	    	    	  endpointInfo, rrq, reason);
	switch ( proceed )
	{
	    case wait:
	    {
	    	// The gatekeeper is probably waiting for an external 
		// event to drive the registration forward. 
		//
	    	VDEBUG(log) << fn << ": waiting." << VDEBUG_END(log);
	    	return;
	    }
	    
	    case confirm:
	    {
		confirmRRQ(rrq, endpointInfo, remoteAddr, true);

    		VDEBUG(log) << fn << ": Keep alive register endpoint = { " 
			   << *endpointInfo << " }"<< VDEBUG_END(log);
		break;
	    }
	    default:
	    {
    	    	rejectRRQ(rrq, reason, remoteAddr, true);

    	    	removeEndpoint(endpointInfo);

		break;
	    }
	}
    	return;
    }

    // Let's first see if we can make some sense of it.
    //
    if ( !validateRRQ(rrq, endpointInfo, reason) )
    {
    	rejectRRQ(rrq, reason, remoteAddr, true);

	return;
    }

    // If we get here, the endpoint already existed, or validate
    // created it for us.
    //
    assert( endpointInfo != 0 );

    remoteAddr = responseAddress(&rrq, endpointInfo, remoteAddr);
    
    // Let the derived class decide how to process it.
    //
    Proceed proceed = onRegistration(rasPDU, remoteAddr, endpointInfo, 
    	    	    	    	     rrq, reason);

    switch ( proceed )
    {
	case wait:
	{
	    // The gatekeeper is probably waiting for an external 
	    // event to drive the registration forward. 
	    //
	    VDEBUG(log) << fn << ": waiting." << VDEBUG_END(log);
	    return;
	}

	case confirm:
	{
	    confirmRRQ(rrq, endpointInfo, remoteAddr);

    	    VDEBUG(log) << fn << ": Register endpoint = { " 
		       << *endpointInfo << " }"<< VDEBUG_END(log);
    	    break;
	}
	default:
	{
    	    rejectRRQ(rrq, reason, remoteAddr);

    	    removeEndpoint(endpointInfo);

    	    break;
	}
    }
}


void    	
DefaultRegistrationPolicy::processUnregistrationRequest(
    Sptr<H323RasPDU>	    rasPDU, 
    Sptr<IPAddress> 	    remoteAddr
)
{
    const string    fn("DefaultRegistrationPolicy::processUnregistrationRequest");
    VLog    	    log(fn);

    H225_UnregistrationRequest	&   urq = *rasPDU;

    H225_UnregRejectReason      reason(H225_UnregRejectReason::e_undefinedReason);
    Sptr<EndpointInfo>	    	endpointInfo = 0;

    if ( !validateURQ(urq, endpointInfo, reason) )
    {
    	rejectURQ(urq, reason, remoteAddr);
	return;
    }
    assert( endpointInfo != 0 );
		    
    remoteAddr = responseAddress(0, endpointInfo, remoteAddr);

    // Let the derived class decide how to process it.
    //
    Proceed proceed = onUnregistration(rasPDU, remoteAddr, endpointInfo, 
    	    	    	    	       urq, reason);
    switch ( proceed )
    {
	case wait:
	{
	    // The gatekeeper is probably waiting for an external 
	    // event to drive the unregistration forward. 
	    //
	    VDEBUG(log) << fn << ": waiting." << VDEBUG_END(log);
	    return;
	}

	case confirm:
	{
	    // Confirm
	    //
	    confirmURQ(urq, endpointInfo, remoteAddr);

    	    VDEBUG(log) << fn << ": Unregister endpoint = { " 
		       << *endpointInfo << " }"<< VDEBUG_END(log);

    	    removeEndpoint(endpointInfo);

	    break;
	}
	default:
	{
    	    rejectURQ(urq, reason, remoteAddr);
	    
	    break;
	}
    }
}

			    
void    	
DefaultRegistrationPolicy::processUnregistrationResponse(
    Sptr<H323RasPDU>	    rasPDU, 
    Sptr<IPAddress> 	    remoteAddr
)
{
    const string fn("DefaultRegistrationPolicy::processUnregistrationConfirm");
    VLog    	 log(fn);

    onUnregistrationResponse(rasPDU);
}


void	    	    
DefaultRegistrationPolicy::unregister(
    H225_UnregRequestReason         	&   reason,
    Sptr<EndpointInfo>  	    	    endpointInfo,
    Sptr<IPAddress>     	    	    remoteAddr
)
{
    const string    fn("DefaultRegistrationPolicy::unregister");
    VLog    	    log(fn);

    // Note: The right thing to do here would be to issue an unregistration
    // request, wait for the response, or for the retries to timeout,
    // then remove the endpointInfo record. For now, since it isn't
    // really required, just send out a urq and remove the record.
    //

    // UnregistrationRequest
    // {
    //     requestSeqNum		:= 1
    //     callSignalAddress	    	:= [Endpoint's CallSignalAddress]
    //     reason			:= ttlExpired OPTIONAL
    // }

    Sptr<H323RasPDU>	    	    pdu = new H323RasPDU;
    pdu->SetTag(H225_RasMessage::e_unregistrationRequest);

    H225_UnregistrationRequest	&   urq = *pdu;

    urq.m_requestSeqNum = 1;
    
    appendAddress(urq.m_callSignalAddress, endpointInfo->callSignalAddress());

    urq.m_reason = reason;

    H225_UnregRejectReason         urjReason;
    onUnregistration(pdu, remoteAddr, endpointInfo, urq, urjReason);

    // Give the derived policy the last crack at the message.
    //
    finalizeUnregistrationRequest(urq, endpointInfo);

    // Ship it.
    //
    myRasGkProtocol.unregistrationRequest(*pdu, remoteAddr);

    // Remove the endpointInfo.
    //
    myEndpointInfoMap.erase(endpointInfo);
}


ostream &           
DefaultRegistrationPolicy::writeTo(ostream & out) const
{
    return ( out << "DefaultRegistrationPolicy" );
}


Vocal::H323GK::RASPolicy::Proceed	    
DefaultRegistrationPolicy::onKeepAliveRegistration(
    Sptr<H323RasPDU>	    	    	rasPDU,
    Sptr<IPAddress> 	    	    	remoteAddr,
    Sptr<EndpointInfo>  	    	endpointInfo,
    H225_RegistrationRequest        &	karrq,
    H225_RegistrationRejectReason   &   reason
)
{
    return ( confirm );
}


void	    
DefaultRegistrationPolicy::finalizeKeepAliveRegistrationConfirm(
    H225_RegistrationRequest        &	karrq,
    H225_RegistrationConfirm        &	karcf,
    Sptr<EndpointInfo>  	    	endpointInfo
)
{
    // By default do nothing.
}


void	    
DefaultRegistrationPolicy::finalizeKeepAliveRegistrationReject(
    H225_RegistrationRequest        &	karrq,
    H225_RegistrationReject         &	karrj
)
{
    // By default do nothing.
}


Vocal::H323GK::RASPolicy::Proceed	    
DefaultRegistrationPolicy::onRegistration(
    Sptr<H323RasPDU>	    	    	rasPDU, 
    Sptr<IPAddress> 	    	    	remoteAddr,
    Sptr<EndpointInfo>  	    	endpointInfo,
    H225_RegistrationRequest        &   rrq,
    H225_RegistrationRejectReason   &   reason
)
{
    return ( confirm );
}


void	    
DefaultRegistrationPolicy::finalizeRegistrationConfirm(
    H225_RegistrationRequest        &	rrq,
    H225_RegistrationConfirm        &	rcf,
    Sptr<EndpointInfo>  	    	endpointInfo
)
{
    // By default do nothing.
}
    

void	    
DefaultRegistrationPolicy::finalizeRegistrationReject(
    H225_RegistrationRequest        &	rrq,
    H225_RegistrationReject         &	rrj
)
{
    // By default do nothing.
}


Vocal::H323GK::RASPolicy::Proceed     
DefaultRegistrationPolicy::onUnregistration(
    Sptr<H323RasPDU>	    	    	rasPDU, 
    Sptr<IPAddress> 	    	    	remoteAddr,
    Sptr<EndpointInfo>  	    	endpointInfo,
    H225_UnregistrationRequest	    & 	urq,
    H225_UnregRejectReason          &   reason
)
{
    return ( confirm );
}


void	    
DefaultRegistrationPolicy::finalizeUnregistrationRequest(
    H225_UnregistrationRequest      &	urq,
    Sptr<EndpointInfo>  	    	endpointInfo
)
{
    // By default do nothing.
}


void	    
DefaultRegistrationPolicy::finalizeUnregistrationConfirm(
    H225_UnregistrationRequest      &	urq,
    H225_UnregistrationConfirm      &	ucf,
    Sptr<EndpointInfo>  	    	endpointInfo
)
{
    // By default do nothing.
}


void	    
DefaultRegistrationPolicy::finalizeUnregistrationReject(
    H225_UnregistrationRequest      &	urq,
    H225_UnregistrationReject       &	urj
)
{
    // By default do nothing.
}


void	    
DefaultRegistrationPolicy::onUnregistrationResponse(
    Sptr<H323RasPDU>	    	    	rasPDU
)
{
    // By default do nothing.
}


bool	    	    
DefaultRegistrationPolicy::isKeepAlive(H225_RegistrationRequest & rrq) const
{
    if ( rrq.HasOptionalField(H225_RegistrationRequest::e_keepAlive) )
    {
    	return ( rrq.m_keepAlive == TRUE );
    }
    return ( false );
}


Sptr<IPAddress>     
DefaultRegistrationPolicy::responseAddress(
    H225_RegistrationRequest    *   rrq, 
    Sptr<EndpointInfo>  	    endpointInfo,
    Sptr<IPAddress>     	    remoteAddr
)
{
    // First try to get it from the endpointInfo
    //
    if ( endpointInfo != 0 )
    {
    	remoteAddr = new IPAddress(endpointInfo->rasAddress());
    }

    // Then try to get it from the rrq.
    //
    else if ( rrq )
    {
	// Get ras address. Look for the first IPAddress.
	//
	for ( size_t i = 0; i < rrq->m_rasAddress.GetSize(); i++ )
	{
            if (    rrq->m_rasAddress[i].GetTag() 
	    	==  H225_TransportAddress::e_ipAddress 
		)
            {
    	    	H225_TransportAddress_ipAddress & ipAddr = rrq->m_rasAddress[i];
        	remoteAddr->setIP(ipAddr.m_ip.GetPointer());
		remoteAddr->setPort(ipAddr.m_port);
		break;
            }
	}
    }

    // The last resort is to use the address obtained from the socket.
    //
    return ( remoteAddr );
}


bool
DefaultRegistrationPolicy::validateKeepAliveRRQ(
    H225_RegistrationRequest        &	rrq,
    Sptr<EndpointInfo>      	    &	endpointInfo,
    H225_RegistrationRejectReason   &   reason
)
{
    if ( !rrq.HasOptionalField(H225_RegistrationRequest::e_endpointIdentifier) )
    {
    	reason.SetTag(H225_RegistrationRejectReason::e_fullRegistrationRequired);
	return ( false );
    }

    string  endpointIdentifier;

    endpointIdentifier = rrq.m_endpointIdentifier.GetValue();
    endpointInfo = myEndpointInfoMap.findByEndpointId(endpointIdentifier);

    // If they don't have an endpointIdentifier, they aren't validated.
    //
    if ( endpointInfo == 0 )
    {
    	return ( false );
    }

    u_int32_t   ttl = 0;
    
    if ( rrq.HasOptionalField(H225_RegistrationRequest::e_timeToLive) )
    {
    	ttl = rrq.m_timeToLive;
    }

    endpointInfo->timeToLive(ttl);
    endpointInfo->updateLastRegistration();

    stopKeepAliveTimer(endpointInfo);
    startKeepAliveTimer(endpointInfo);

    return ( true );
}


bool
DefaultRegistrationPolicy::validateRRQ(
    H225_RegistrationRequest        &	rrq,
    Sptr<EndpointInfo>      	    &	endpointInfo,
    H225_RegistrationRejectReason   &   reason
)
{
    const string    fn("DefaultRegistrationPolicy::validateRRQ");
    VLog    	    log(fn);

    endpointInfo = 0;
    
    // Pull the endpointIdentifier out of the rrq.
    //
    string  endpointIdentifier;
    
    if ( rrq.HasOptionalField(H225_RegistrationRequest::e_endpointIdentifier) )
    {
    	endpointIdentifier = rrq.m_endpointIdentifier.GetValue();
    	endpointInfo = myEndpointInfoMap.findByEndpointId(endpointIdentifier);
    }

    // If don't have an endpointIdentifier, look for the alias.
    //
    if ( endpointInfo == 0  )
    {
    	endpointInfo = myEndpointInfoMap.findByAlias(rrq.m_terminalAlias);
    }

    // If it's a gateway, check to see if we can find it by call signaling 
    // address This is making an assumption that there is one logical 
    // gateway per ip address, since finding by ip address matches the ip 
    // address and not the port (thanks to that wonderful NetMeeting
    // bug). This is probably a safe assumption.
    //
    if  (   endpointInfo == 0
        &&  rrq.m_terminalType.HasOptionalField(H225_EndpointType::e_gateway)
        )
    {
        // Get the callSignalAddress.
        //
        IPAddress csAddr;

        size_t  size = rrq.m_callSignalAddress.GetSize();

        for ( size_t i = 0; i < size; i++ )
        {
            if ( rrq.m_callSignalAddress[i].GetTag() == H225_TransportAddress::e_ipAddress )
            {
    	        H225_TransportAddress_ipAddress & csIP = rrq.m_callSignalAddress[i];
                csAddr.setIP(csIP.m_ip.GetPointer());
	        csAddr.setPort(csIP.m_port);

                endpointInfo = myEndpointInfoMap.findByCallSignalAddress(csAddr);
	        break;
            }
        }
    }
    
    // Make a new endpointInfo record.
    //
    if ( endpointInfo == 0 )
    {
    	if ( (endpointInfo = createEndpoint(rrq)) == 0 )
	{
            reason.SetTag(H225_RegistrationRejectReason::e_invalidAlias);
	    return ( false );
	}
    }

    // We need to update the record using the same endpoint identifier.
    //
    // Only update if the rasAddress is the same as the current.
    //
    else
    {
        // We have a registration from a dynamically registered endpoint, 
        // which is fine, since it may have been statically registered just 
        // for the prefix.
        //
        //  OR ...
        //
    	// We have either a reregistration (when the entire ip/port of the
	// ras address is the same), or a registration from the same endpoint
	// but the port has changed (say after an endpoint crash), so we
	// need to do an update.
	//
        EndpointInfo    newEndpointInfo(rrq);
    	
        if  (	endpointInfo->staticRegistration()
            ||
                newEndpointInfo.rasAddress().getIPAsULong() 
	    	    ==  endpointInfo->rasAddress().getIPAsULong() 
	    )
	{

            newEndpointInfo.endpointIdentifier(endpointInfo->endpointIdentifier());
	
	    // The update failed. Reject the registration and remove
	    // it.
	    //
	    if ( !newEndpointInfo.valid() )
	    {
                removeEndpoint(endpointInfo);
	    	return ( false );
	    }
	
	    *endpointInfo = newEndpointInfo;

	    stopKeepAliveTimer(endpointInfo);
	}
	else
	{
	    VWARN(log)  << fn << ": attempt to usurp registration for endpoint = { " 
	    	    	<< endpointInfo->alias() << "\n}, " 
			<< endpointInfo->rasAddress()
			<< ", from " << newEndpointInfo.rasAddress()
			<< VWARN_END(log);
			
	    return ( false );
	}
    }

    startKeepAliveTimer(endpointInfo);
    
    return ( true );
}
					
				
void	    
DefaultRegistrationPolicy::confirmRRQ(
    H225_RegistrationRequest    &	rrq, 
    Sptr<EndpointInfo>  	    	endpointInfo,
    Sptr<IPAddress>     	    	remoteAddr,
    bool	    	    	    	keepAlive
)
{
    const string    fn("DefaultRegistrationPolicy::confirmRRQ");
    VLog    	    log(fn);

    // RegistrationConfirm
    // {
    //     requestSeqNum    	    := [from rrq]
    //     protocolIdentifier       := [from openh323]
    //     callSignalAddress        := [from GatekeeperInfo]
    //     gatekeeperIdentifier     := [from GatekeeperInfo]
    //     endpointIdentifier       := [from EndpointInfo]
    //     timeToLive 	    	    := [from EndpointInfo]
    //     willRespondToIRR         := false
    //     preGrantedARQ
    //     {
    //        makeCall                            := true
    //        useGKCallSignalAddressToMakeCall    := false
    //        answerCall                          := true
    //        useGKCallSignalAddressToAnswer      := false
    //     } OPTIONAL
    // }

    H323RasPDU	    	    	    pdu;
    pdu.SetTag(H225_RasMessage::e_registrationConfirm);

    H225_RegistrationConfirm	&   rcf = pdu;

    rcf.m_requestSeqNum = rrq.m_requestSeqNum;

    rcf.m_protocolIdentifier.SetValue(myGatekeeperInfo.h225ProtocolID().c_str());

    appendAddress(rcf.m_callSignalAddress, myGatekeeperInfo.callSignalAddress());

    rcf.IncludeOptionalField(H225_RegistrationConfirm::e_gatekeeperIdentifier);
    rcf.m_gatekeeperIdentifier = myGatekeeperInfo.gatekeeperIdentifier().c_str();

    rcf.m_endpointIdentifier = endpointInfo->endpointIdentifier().c_str();
   
    if ( endpointInfo->timeToLive() )
    {
    	rcf.IncludeOptionalField(H225_RegistrationConfirm::e_timeToLive);
    	rcf.m_timeToLive = endpointInfo->timeToLive();
    }

    rcf.m_willRespondToIRR = false;
    
    // Give the derived policy the last crack at the message.
    //
    if ( keepAlive )
    {
    	finalizeKeepAliveRegistrationConfirm(rrq, rcf, endpointInfo);
    }
    else
    {
        rcf.IncludeOptionalField(H225_RegistrationConfirm::e_preGrantedARQ);
        rcf.m_preGrantedARQ.m_makeCall = true;
        rcf.m_preGrantedARQ.m_useGKCallSignalAddressToMakeCall = true;
        rcf.m_preGrantedARQ.m_answerCall = true;
        rcf.m_preGrantedARQ.m_useGKCallSignalAddressToAnswer = true;

    	finalizeRegistrationConfirm(rrq, rcf, endpointInfo);
    }

    // Ship it.
    //
    char    	buf[256];
    ostrstream	confirmLog(buf, 256);

    confirmLog << "Register endpoint = { " << *endpointInfo << " }" << ends;

    myRasGkProtocol.registrationConfirm(pdu, remoteAddr, confirmLog);
}


void	    
DefaultRegistrationPolicy::rejectRRQ(
    H225_RegistrationRequest    	&   rrq,
    H225_RegistrationRejectReason	&   reason,
    Sptr<IPAddress>     	    	    remoteAddr,
    bool	    	    	    	    keepAlive
)
{
    const string    fn("DefaultRegistrationPolicy::rejectRRQ");
    VLog    	    log(fn);

    // RegistrationReject
    // {
    //     requestSeqNum	:= [from rrq]
    //     protocolIdentifier	:= [from openh323]
    //     rejectReason		:= [param]
    //     gatekeeperIdentifier	:= [from GatekeeperInfo] OPTIONAL,
    // }

    H323RasPDU	    	    	    pdu;
    pdu.SetTag(H225_RasMessage::e_registrationReject);

    H225_RegistrationReject	&   rrj = pdu;

    rrj.m_requestSeqNum = rrq.m_requestSeqNum;

    rrj.m_protocolIdentifier.SetValue(myGatekeeperInfo.h225ProtocolID().c_str());

    rrj.m_rejectReason = reason;
    
    rrj.IncludeOptionalField(H225_RegistrationReject::e_gatekeeperIdentifier);
    rrj.m_gatekeeperIdentifier = myGatekeeperInfo.gatekeeperIdentifier().c_str();

    // Give the derived policy the last crack at the message.
    //
    if ( keepAlive )
    {
    	finalizeKeepAliveRegistrationReject(rrq, rrj);
    }
    else
    {
    	finalizeRegistrationReject(rrq, rrj);
    }

    // Ship it.
    //
    myRasGkProtocol.registrationReject(pdu, remoteAddr);
}


bool
DefaultRegistrationPolicy::validateURQ(
    H225_UnregistrationRequest      &	urq,
    Sptr<EndpointInfo>  	    &	endpointInfo,
    H225_UnregRejectReason          &	reason
)
{
    endpointInfo = 0;

    // If there isn't a endpointIdentifier, it's not registered with us.
    //
    if  ( !urq.HasOptionalField(H225_UnregistrationRequest::e_endpointIdentifier) )
    {
    	reason.SetTag(H225_UnregRejectReason::e_notCurrentlyRegistered);
	return ( false );
    }
    // Pull the endpoint identifier out of the URQ.
    //
    string	endpointIdentifier 
	= urq.m_endpointIdentifier.GetValue().GetPointer();

    // See if this is an endpoint we know about.
    //
    endpointInfo = myEndpointInfoMap.findByEndpointId(endpointIdentifier);

    if ( endpointInfo == 0 )
    {
    	reason.SetTag(H225_UnregRejectReason::e_notCurrentlyRegistered);
	return ( false );
    }

    // This is an unregistration, so we need to set the endpoint ttl to zero
    // and cancel the ttl timer if we have one.
    //
    endpointInfo->timeToLive(0);
    endpointInfo->updateLastRegistration();

    stopKeepAliveTimer(endpointInfo);
    
    return ( true );    
}


void	    	    
DefaultRegistrationPolicy::confirmURQ(
    H225_UnregistrationRequest    &	urq, 
    Sptr<EndpointInfo>  	    	endpointInfo,
    Sptr<IPAddress>     	    	remoteAddr
)
{
    const string    fn("DefaultRegistrationPolicy::confirmURQ");
    VLog    	    log(fn);

    // UnregistrationConfirm
    // {
    //     requestSeqNum	:= [from urq]
    // }

    H323RasPDU	    	    	    pdu;
    pdu.SetTag(H225_RasMessage::e_unregistrationConfirm);

    H225_UnregistrationConfirm	&   ucf = pdu;
    
    ucf.m_requestSeqNum = urq.m_requestSeqNum;
    
    // Give the derived policy the last crack at the message.
    //
    finalizeUnregistrationConfirm(urq, ucf, endpointInfo);

    // Ship it.
    //
    char    	buf[256];
    ostrstream	confirmLog(buf, 256);

    confirmLog << "Unregister endpoint = { " << *endpointInfo << " }" << ends;

    myRasGkProtocol.unregistrationConfirm(pdu, remoteAddr, confirmLog);
}


void	    
DefaultRegistrationPolicy::rejectURQ(
    H225_UnregistrationRequest    	&   urq,
    H225_UnregRejectReason  	    	&   reason,
    Sptr<IPAddress>     	    	    remoteAddr
)
{
    const string    fn("DefaultRegistrationPolicy::rejectURQ");
    VLog    	    log(fn);

    // UnregistrationReject
    // {
    //     requestSeqNum	:= [from urq]
    //     rejectReason		:= [param]
    // }

    H323RasPDU	    	    	    pdu;
    pdu.SetTag(H225_RasMessage::e_unregistrationReject);

    H225_UnregistrationReject	&   urj = pdu;
    
    urj.m_requestSeqNum = urq.m_requestSeqNum;

    urj.m_rejectReason = reason;

    // Give the derived policy the last crack at the message.
    //
    finalizeUnregistrationReject(urq, urj);
    
    // Ship it.
    //
    myRasGkProtocol.unregistrationReject(pdu, remoteAddr);
}


void	    	    
DefaultRegistrationPolicy::appendAddress(
    H225_ArrayOf_TransportAddress   	&   transportAddresses,
    const IPAddress 	    	    	&   ipAddress
)
{
    PINDEX size = transportAddresses.GetSize();
    transportAddresses.SetSize(size + 1);
    
    H225_TransportAddress & transAddr = transportAddresses[size];

    transAddr.SetTag(H225_TransportAddress::e_ipAddress);

    H225_TransportAddress_ipAddress & ipAddr = transAddr;
    
    memcpy( &ipAddr.m_ip[0], 
    	    &(ipAddress.getIPAddress().sin_addr.s_addr), 
	    4);

    ipAddr.m_port = ipAddress.getPort();
}


void	    	    
DefaultRegistrationPolicy::startKeepAliveTimer(
    Sptr<EndpointInfo>  	endpointInfo
)
{
    u_int32_t ttl = endpointInfo->timeToLive();
    
    if ( ttl )
    {
    	static u_int32_t    min_ttl = 20;

    	if ( ttl < min_ttl )
	{
	    ttl = min_ttl;
	}

	milliseconds_t  delay = ttl * 1000;

	Sptr<KeepAliveTimerEvent>	timer 
	    = new KeepAliveTimerEvent(endpointInfo->endpointIdentifier());

	endpointInfo->timeToLiveTimerEventId(
	    myService.getFifo().addDelayMs(timer, delay));
    }
}


void	    	    
DefaultRegistrationPolicy::stopKeepAliveTimer(
    Sptr<EndpointInfo>  	endpointInfo
)
{
    myService.getFifo().cancel(
    	endpointInfo->timeToLiveTimerEventId());

    endpointInfo->timeToLiveTimerEventId(0);
}


Sptr<EndpointInfo>  
DefaultRegistrationPolicy::createEndpoint(
    H225_RegistrationRequest    	&   rrq
)
{
    Sptr<EndpointInfo> endpointInfo = new EndpointInfo(rrq);

    endpointInfo->endpointIdentifier(
	    	    myGatekeeperInfo.generateEndpointIdentifier());

    if ( !endpointInfo->valid() )
    {
	endpointInfo = 0;
    }
    else
    {
    	myEndpointInfoMap.insert(endpointInfo);
    }
    return ( endpointInfo );
}				


void	    	    
DefaultRegistrationPolicy::removeEndpoint(
    Sptr<EndpointInfo>  	endpointInfo
)
{
    if ( endpointInfo != 0 )
    {
    	myEndpointInfoMap.erase(endpointInfo);
    }
}
