#if !defined(VOCAL_DEFAULT_REGISTRATION_POLICY_HXX)
#define VOCAL_DEFAULT_REGISTRATION_POLICY_HXX

/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */


static const char* const DefaultRegistrationPolicy_hxx_Version = 
    "$Id: DefaultRegistrationPolicy.hxx,v 1.14 2001/05/22 07:53:09 icahoon Exp $";


#include "ptlib.h"
#include "h323pdu.h"
#include "RegistrationPolicy.hxx"
#include "EventObserver.hxx"
#include "Sptr.hxx"
#include "IPAddress.hxx"
#include "EndpointInfo.hxx"


/** Vovida Open Communication Application Library<br><br>
 */
namespace Vocal
{


/** H.323 Gatekeeper<br><br>
 */
namespace H323GK
{


using Vocal::Services::Event;


/** Default policy for determining how to handle RAS registration and 
 *  unregistration messages.<br><br>
 *
 *  @see    Vocal::H323GK::RegistrationPolicy
 *  @see    Vocal::Behavioral::EventObserver
 *  @see    Vocal::H323GK::RASPolicy
 *  @see    Vocal::H323GK::RASGKProtocol
 *  @see    Vocal::SipH323Info::GatekeeperInfo
 *  @see    Vocal::SipH323Info::EndpointInfoMap
 *  @see    Vocal::SipH323Info::EndpointInfo
 *  @see    Vocal::Services::Service
 *  @see    Vocal::Transport::IPAddress
 *  @see    Vocal::H323GK::KeepAliveTimerEvent
 *  @see    Vocal::Services::Event
 *  @see    Sptr
 */
class DefaultRegistrationPolicy 
    :   public Vocal::H323GK::RegistrationPolicy,
    	public Vocal::Behavioral::EventObserver< Sptr<Event> >
{

    public:


    	/** Construct given the controling service, the associated ras gk
	 *  protocol, the configuration information for the gatekeeper and
	 *  the table of registered endpoints.
	 */
    	DefaultRegistrationPolicy(
	    Service &,
	    RASGKProtocol &,
	    GatekeeperInfo &,
	    EndpointInfoMap &
	);


    	/** Virtual destructor
	 */
	virtual		    ~DefaultRegistrationPolicy();


    	/** EventObserver callback. Returns true if the event
	 *  is a keep alive timer event.
	 */
    	virtual bool	    interestedIn(const Sptr<Event> &);


    	/** EventObserver callback. This processes the keep alive time events.
	 */
    	virtual void	    onEvent(Sptr<Event>);


    	/** Called to process an incoming registration.
    	 *
	 *  This will first validate the incoming rrq. It will then give
	 *  the user an opportunity to process the incoming message by calling
	 *  the callback onRegistration. On confirmation, an RCF is sent.
	 *  On rejection, an RRJ is sent.
	 */
    	void        	    processRegistrationRequest(
	    	    		Sptr<H323RasPDU>    rasPDU, 
				Sptr<IPAddress>     remoteAddr);


    	/** Called to process an incoming unregistration.
	 *
	 *  This will first validate the incoming drq. It will then give
	 *  the user an opportunity to process the incoming message by calling
	 *  the callback onDisengage. On confirmation, an DCF is sent.
	 *  On rejection, an DRJ is sent.
	 */
    	void        	    processUnregistrationRequest(
	    	    		Sptr<H323RasPDU>    rasPDU, 
				Sptr<IPAddress>     remoteAddr);


    	/** Called to process an incoming unregistration response.
	 *
	 *  This will give the user an opportunity to process the incoming 
	 *  message by calling the callback onUnregistrationResponse.
	 */
    	void        	    processUnregistrationResponse(
	    	    	    	Sptr<H323RasPDU>    rasPDU, 
			    	Sptr<IPAddress>     remoteAddr);


    	/** Builds and sends an unregistration request.
	 *  Will finalize before sending.
	 */
    	void	    	    unregister(
				H225_UnregRequestReason     &	reason,
				Sptr<EndpointInfo>  	    	endpointInfo,
			    	Sptr<IPAddress>     	    	remoteAddr);


    	/** Write the policy to an ostream. 
	 */
    	ostream &           writeTo(ostream & out) const;


    protected:

	
	/** Called on receipt of a keep alive registration request. 
	 *  	 
	 *  On wait, the processing stops. It is the responsibility of
	 *  the derived class to retain message state.
	 *  	 
	 *  On confirm, the processing continues processing the request.
	 *  	 
	 *  On reject, a registration reject is sent, and the user needs
	 *  to supply a reason.
	 */
	virtual Proceed	    onKeepAliveRegistration(
				Sptr<H323RasPDU>	    	    rasPDU, 
				Sptr<IPAddress> 	    	    remoteAddr,
	    	    	    	Sptr<EndpointInfo>  	    	    endpointInfo,
	    	    	    	H225_RegistrationRequest    	&   karrq,
				H225_RegistrationRejectReason	&   reason);


    	/** Called before the rcf is sent.
	 */
    	virtual void	    finalizeKeepAliveRegistrationConfirm(
	    	    	    	H225_RegistrationRequest    &	karrq,
				H225_RegistrationConfirm    &	karcf,
				Sptr<EndpointInfo>  	    	endpointInfo);


    	/** Called befire the rrj is sent.
	 */
    	virtual void	    finalizeKeepAliveRegistrationReject(
	    	    	    	H225_RegistrationRequest    &	karrq,
				H225_RegistrationReject     &	karrj);


	/** Called on receipt of a registration request. 
	 *  	
	 *  On wait, the processing stops. It is the responsibility of
	 *  the derived class to retain message state.
	 *  	 
	 *  On confirm, the processing continues processing the request.
	 *  	 
	 *  On reject, a registration reject is sent, and the user needs
	 *  to supply a reason.
	 */
    	virtual Proceed	    onRegistration(
				Sptr<H323RasPDU>	    	    rasPDU, 
				Sptr<IPAddress> 	    	    remoteAddr,
	    	    	    	Sptr<EndpointInfo>  	    	    endpointInfo,
	    	    	    	H225_RegistrationRequest    	&   rrq,
				H225_RegistrationRejectReason	&   reason);


    	/** Called before the rcf is sent.
	 */
    	virtual void	    finalizeRegistrationConfirm(
	    	    	    	H225_RegistrationRequest    &	rrq,
	    	    	    	H225_RegistrationConfirm    &	rcf,
				Sptr<EndpointInfo>  	    	endpointInfo);


    	/** Called befire the rrj is sent.
	 */
    	virtual void	    finalizeRegistrationReject(
	    	    	    	H225_RegistrationRequest    &	rrq,
	    	    	    	H225_RegistrationReject     &	rrj);


	/** Called on receipt of a unregistration request. 
	 *  	
	 *  On wait, the processing stops. It is the responsibility of
	 *  the derived class to retain message state.
	 *  	 
	 *  On confirm, the processing continues processing the request.
	 *  	 
	 *  On reject, a unregistration reject is sent, and the user needs
	 *  to supply a reason.
	 */
    	virtual Proceed     onUnregistration(
				Sptr<H323RasPDU>	    	rasPDU, 
				Sptr<IPAddress> 	    	remoteAddr,
	    	    	    	Sptr<EndpointInfo>  	    	endpointInfo,
	    	    	    	H225_UnregistrationRequest  &	urq,
				H225_UnregRejectReason      &   reason);
	

    	/** Called before the urq is sent.
	 */
    	virtual void	    finalizeUnregistrationRequest(
	    	    	    	H225_UnregistrationRequest  &	urq,
				Sptr<EndpointInfo>  	    	endpointInfo);


    	/** Called before the ucf is sent.
	 */
    	virtual void	    finalizeUnregistrationConfirm(
	    	    	    	H225_UnregistrationRequest  &	urq,
	    	    	    	H225_UnregistrationConfirm  &	ucf,
				Sptr<EndpointInfo>  	    	endpointInfo);


    	/** Called before the urj is sent.
	 */
    	virtual void	    finalizeUnregistrationReject(
	    	    	    	H225_UnregistrationRequest  &	urq,
	    	    	    	H225_UnregistrationReject   &	urj);


	/** Called on receipt of a unregistration response. 
	 */
    	virtual void	    onUnregistrationResponse(
    	    	    	    	Sptr<H323RasPDU>	    	rasPDU);
	

    	/** Returns true if the registration request is of the keep alive
	 *   variety.
	 */
    	bool	    	    isKeepAlive(H225_RegistrationRequest & rrq) const;


    	/** Returns the address where the response will be sent.
	 *  If the endpoint info is set, the endpointInfo's rasAddress will 
	 *  be used, otherwise if the rrq contains a valid rasAddress, that 
	 *  will be used, otherwise the remote address will be used.
	 */
    	Sptr<IPAddress>     responseAddress(
	    	    	    	H225_RegistrationRequest    *	rrq, 
				Sptr<EndpointInfo>  	    	endpointInfo,
			    	Sptr<IPAddress>     	    	remoteAddr);


    	/** Ensures the keep alive registration request is sane. If the request
	 *  is sane it returns the endpointInfo associated with the 
	 *  request. On successIt should never return a 0 for the endpointInfo.
	 */	
    	virtual bool  	    validateKeepAliveRRQ(
	    	    	    	H225_RegistrationRequest    &	rrq,
				Sptr<EndpointInfo>  	    &	endpointInfo,
				H225_RegistrationRejectReason	&   reason);


    	/** Ensures the registration request is sane. If the request
	 *  is sane and the endpoint already is known to the gatekeeper,
	 *  it returns true. On success the endpointInfo associated with 
	 *  the request will be set, otherwise it returns 0. On failure, it
	 *  returns false and reason may be set.
	 */	
    	virtual bool  	    validateRRQ(
	    	    	    	H225_RegistrationRequest    	&   rrq,
				Sptr<EndpointInfo>  	    	&   endpointInfo,
				H225_RegistrationRejectReason	&   reason);
					

    	/** Builds and sends the confirmation to the registration request.
	 *  Will finalize before sending.
	 */
    	void	    	    confirmRRQ(
	    	    	    	H225_RegistrationRequest    &	rrq, 
				Sptr<EndpointInfo>  	    	endpointInfo,
			    	Sptr<IPAddress>     	    	remoteAddr,
				bool	    	    	    	keepAlive = false);


    	/** Builds and sends the rejection to the registration request.
	 *  Will finalize before sending.
	 */
    	void	    	    rejectRRQ(
	    	    	    	H225_RegistrationRequest    	&   rrq,
				H225_RegistrationRejectReason	&   reason,
			    	Sptr<IPAddress>     	    	    remoteAddr,
				bool	    	    	    	    keepAlive = false);

    	/** Ensures the unregistration request is sane. If the request
	 *  is sane it returns the endpointInfo associated with the 
	 *  request, otherwise it returns 0. It may create a new endpointInfo 
	 *  record, which it will add to the map.
	 */	
    	virtual bool	    validateURQ(
	    	    	    	H225_UnregistrationRequest  &	urq,
				Sptr<EndpointInfo>  	    &	endpointInfo,
				H225_UnregRejectReason      &	reason);
					

    	/** Builds and sends the confirmation to the unregistration request.
	 *  Will finalize before sending.
	 */
    	void	    	    confirmURQ(
	    	    	    	H225_UnregistrationRequest    &	urq, 
				Sptr<EndpointInfo>  	    	endpointInfo,
			    	Sptr<IPAddress>     	    	remoteAddr);


    	/** Builds and sends the rejection to the registration request.
	 *  Will finalize before sending.
	 */
    	void	    	    rejectURQ(
	    	    	    	H225_UnregistrationRequest  &   urq,
				H225_UnregRejectReason      &   reason,
			    	Sptr<IPAddress>     	    	remoteAddr);


    	/** Utility function to append the given ip address to the 
	 *  sequence of h.225 transport addresses.
	 */
    	void	    	    appendAddress(
	    	    	    	H225_ArrayOf_TransportAddress &,
				const IPAddress &);


    	/** Start the keep alive timer.
	 */
	void	    	    startKeepAliveTimer(
				Sptr<EndpointInfo>  	endpointInfo);


    	/** Stop the keep alive timer.
	 */
	void	    	    stopKeepAliveTimer(
				Sptr<EndpointInfo>  	endpointInfo);


    	/** Create endpoint record given the registration request.
	 */
    	Sptr<EndpointInfo>  createEndpoint(
	    	    	    	H225_RegistrationRequest    	&   rrq);
				

    	/** Remove endpoint from endpoint map.
	 */
	void	    	    removeEndpoint(
			    	Sptr<EndpointInfo>  	endpointInfo);

};


} // namespace H323GK
} // namespace Vocal


#endif // !defined(VOCAL_DEFAULT_REGISTRATION_POLICY_HXX)
