/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */


static const char* const RASMCastGKProtocol_cxx_Version = 
    "$Id: RASMCastGKProtocol.cxx,v 1.12 2001/11/07 08:57:15 icahoon Exp $";


#include "RASMCastGKProtocol.hxx"
#include "DiscoveryPolicy.hxx"
#include "LocationPolicy.hxx"
#include "SocketOptions.hxx"
#include "VLog.hxx"
#include "CallTrace.hxx"


using Vocal::H323GK::RASMCastGKProtocol;
using Vocal::Transport::Protocol;
using Vocal::H323GK::DiscoveryPolicy;
using Vocal::H323GK::LocationPolicy;
using Vocal::IO::FileDescriptor;
using Vocal::Transport::IPAddress;
using Vocal::Transport::UDPSocket;
using Vocal::Transport::SocketOptions;
using Vocal::Logging::VLog;


RASMCastGKProtocol::RASMCastGKProtocol(IPAddress rasMulticastAddress)
    : 	Protocol("RASMCastGK"),
    	myDiscoveryPolicy(0),
        myLocationPolicy(0),
	myRasMulticastSocket(0),
	myPerStreamSize(8192)
{
    IPAddress   ras_maddr;
    ras_maddr.setPort(rasMulticastAddress.getPort());

    // Create and bind the socket to the ras multicast address specified.
    //
    myRasMulticastSocket = new UDPSocket(ras_maddr);
    myRasMulticastSocket->setNonblocking();
        
    // Add the socket to the specified multicast group.
    //
    SocketOptions   	rasMulticastSocketOptions(*myRasMulticastSocket);
    rasMulticastSocketOptions.joinMulticastGroup(rasMulticastAddress);

    // Make it bsd compatible so that errors show up when they happen,
    // not on the system call after. This may change with future versions
    // of the kernel.
    //
    rasMulticastSocketOptions.bsdCompatible(true);

    awaitIncoming(true);
}

	
RASMCastGKProtocol::~RASMCastGKProtocol()
{
}


void	    	    	    
RASMCastGKProtocol::setDiscoveryPolicy(DiscoveryPolicy & discoveryPolicy)
{
    myDiscoveryPolicy = &discoveryPolicy;
}


void	    	    	    
RASMCastGKProtocol::setLocationPolicy(LocationPolicy & locationPolicy)
{
    myLocationPolicy = &locationPolicy;
}


const FileDescriptor &	    
RASMCastGKProtocol::getFileDescriptor() const
{
    assert(myRasMulticastSocket != 0);
    return ( *myRasMulticastSocket );
}


void	    	    	    
RASMCastGKProtocol::onIncomingAvailable()
{
    const string    fn("RASMCastGKProtocol::onIncomingAvailable");
    VLog	    log(fn);

    // Read the message from the socket.
    //
    Sptr<IPAddress> 	remoteAddr = new IPAddress;
    
    int bytesReceived = myRasMulticastSocket->receiveFrom
    	    	    	(
    	    	    	    myPerStream.GetPointer(myPerStreamSize),
			    myPerStreamSize,
			    *remoteAddr
			);

    // If we receive an empty message, return.
    //
    if ( bytesReceived == 0 )
    {
    	return;
    }
    
    // See if we need to grow the buffer.
    //
    if ( bytesReceived == (int)myPerStreamSize )
    {

	VDEBUG(log) << fn << ": resizing PER stream, old size = " 
	    	    << myPerStreamSize
		    << ", new size = " << (myPerStreamSize * 2)  
		    << VDEBUG_END(log);

    	// Double the size.
	//
	myPerStreamSize *= 2;
    }

    // Decode the RAS message.
    //    
    Sptr<H323RasPDU>	rasPDU = new H323RasPDU;

    if ( !rasPDU->Decode(myPerStream) )
    {
    	VWARN(log)  << fn << ": Incoming RAS PDU decode failed." 
	    	    << VWARN_END(log);
	return;
    }

    // Forward the rasPDU and remoteAddress to the DiscoveryPolicy.
    //
    switch ( rasPDU->GetTag() )
    {
    	case H225_RasMessage::e_gatekeeperRequest:
	{
	    if ( myDiscoveryPolicy )
	    {
		H225_GatekeeperRequest	&   grq = *rasPDU;

		CALLTRACE(log, CT_IN, *remoteAddr, CT_RAS, "GRQ MultiCast") 
		    << grq << CALLTRACE_END(log);

            	myDiscoveryPolicy->processMulticastDiscoveryRequest(rasPDU, remoteAddr); 
	    }
	    else
	    {
        	VDEBUG(log) << fn << ": Discarding GRQ. DiscoveryPolicy not set." 
		    	    << VDEBUG_END(log);
	    }
	    break;
    	}
        case H225_RasMessage::e_locationRequest:
        {
	    if ( myLocationPolicy )
	    {
		H225_LocationRequest	&   lrq = *rasPDU;

		CALLTRACE(log, CT_IN, *remoteAddr, CT_RAS, "LRQ MultiCast") 
		    << lrq << CALLTRACE_END(log);

            	myLocationPolicy->processMulticastLocationRequest(rasPDU, remoteAddr);
	    }
	    else
	    {
        	VDEBUG(log) << fn << ": Discarding LRQ. LocationPolicy not set."
		    	    << VDEBUG_END(log);
	    }
            break;
        }
	default:
	{
            VWARN(log)  << fn << ": Unhandled RAS PDU"
		        << ", message type: " << rasPDU->GetTagName()
	    	        << VWARN_END(log);
    	    break;
	}
    }
}
