/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */


static const char* const VocalGKConfig_cxx_Version = 
    "$Id: VocalGKConfig.cxx,v 1.1 2001/11/01 04:45:04 icahoon Exp $";


#include "VocalGKConfig.hxx"
#include "ParsePair.hxx"
#include <string>
#include <iostream>
#include <strstream>
#include <cstdlib>
#include <cerrno>
#include <sys/stat.h>
#include <unistd.h>


using namespace Vocal;
using Vocal::VocalGKConfig;
using Vocal::SipH323Info::EndpointInfo;
using Vocal::Transport::IPAddress;
using Vocal::Configuration::Config;
using Vocal::Configuration::NameValueMap;
using Vocal::Configuration::ParsePair;
using Vocal::Configuration::VALUE_PRESENT;
using Vocal::Configuration::NO_VALUE;
using std::string;
using std::list;


VocalGKConfig::VocalGKConfig()
    :   outputDirectoryStr  ("output"),
        gatekeeperIdStr     ("gkid"),
        endpointIdStr       ("eptid"),
        gatewaysStr         ("gateways"),
        outputDirectoryChar ('o'),
        gatekeeperIdChar    ('g'),
        endpointIdChar      ('e'),
        myOutputDirectory   ("./"),
        myGatekeeperId      (""),
        myEndpointId        ("endpoint_")
{
}


VocalGKConfig::~VocalGKConfig()
{
}


void        
VocalGKConfig::usage() const
{
    cerr << "usage: " << myApplicationName << " [ options ]" << endl
    	 << "  [ -o ] [ --output ]    directory   Output directory, " << endl
	 << "                                       Defaults to \"./\"" << endl
    	 << "  [ -l ] [ --loglvl ]    level       Log level, " << endl
	 << "                                       Defaults to \"LOG_ERR\"" << endl
    	 << "  [ -g ] [ --gkid ]      id          Use for gatekeeperIdentifier, " << endl
	 << "                                       Defaults to hostname" << endl
    	 << "  [ -e ] [ --eptid ]     id          Use for endpointIdentifier, " << endl
	 << "                                       Defaults to \"endpoint_\"" << endl;

}
        

void        
VocalGKConfig::loadCfgFileErrorMsg(ReturnCode) const
{
    cerr << "Error: config file \"" << myConfigFile
         << "\" could not be parsed." << endl;
}
        
    
ReturnCode  
VocalGKConfig::preParse()
{
    ReturnCode  rc = SUCCESS;
    
    if  (   ( rc = myOptions.add(outputDirectoryStr.c_str(), 
                                 outputDirectoryChar, 
                                 VALUE_PRESENT) ) 
                != SUCCESS
                
        ||  ( rc = myOptions.add(gatekeeperIdStr.c_str(), 
                                 gatekeeperIdChar, 
                                 VALUE_PRESENT) ) 
                != SUCCESS
                
        ||  ( rc = myOptions.add(endpointIdStr.c_str(), 
                                 endpointIdChar,
                                 VALUE_PRESENT) ) 
                != SUCCESS
        )
    {
        // Do nothing since rc is already set on failure.
    }

    return ( rc );
}
        
        
ReturnCode  
VocalGKConfig::postParse()
{
    ReturnCode  rc = SUCCESS;
    
    const NameValueMap &    optionMap = myOptions.options();     
    NameValueMap::const_iterator i;

    // Load outputDirectory value
    //
    if  (   (i = optionMap.find(outputDirectoryStr)) != optionMap.end()
        &&  i->second.type != NO_VALUE
        )
    {
        string  output = i->second.value.back();

        if ( output[output.size()-1] != '/' )
        {
	    output += "/";
        }
        if  (	output.find("../") == 0
	    ||	output.find("/../") != string::npos
	    )
        {
            usage();
	    cerr << "Error: directory cannot contain \"..\"" << endl;
	    return ( !SUCCESS );
        }

        struct stat    outputStat;

        if ( stat(output.c_str(), &outputStat) < SUCCESS )
        {
            usage();
    	    cerr << "Error: " << strerror(errno) << " for " << output << endl;
    	    return ( !SUCCESS );
        }
        if ( !S_ISDIR(outputStat.st_mode) )
        {
            usage();
	    cerr << "Error: file " << output << " isn't a directory" << endl;
	    return ( !SUCCESS );
        }

        myOutputDirectory = output;
    }

    // Load gatekeeperId value
    //
    if  (   (i = optionMap.find(gatekeeperIdStr)) != optionMap.end() 
        &&  i->second.type != NO_VALUE
        )
    {
        myGatekeeperId = i->second.value.back();
    }

    // Load endpointId value
    //
    if  (   (i = optionMap.find(endpointIdStr)) != optionMap.end() 
        &&  i->second.type != NO_VALUE
        )
    {
        myEndpointId = i->second.value.back();
    }

    rc = loadGateways();

    if ( rc != SUCCESS )
    {
        loadCfgFileErrorMsg(rc);
    }
            
    return ( rc );
}



string
VocalGKConfig::outputDirectory() const
{
    return ( myOutputDirectory );
}


string   
VocalGKConfig::gatekeeperId() const
{
    return ( myGatekeeperId );
}


string  
VocalGKConfig::endpointId() const
{
    return ( myEndpointId );
}

        
list< Sptr<EndpointInfo> >  
VocalGKConfig::gateways() const
{
    return ( myGateways );
}
        

ostream &   
VocalGKConfig::writeTo(ostream & out) const
{
    Config::writeTo(out);
    
    out << "\n  output:                " << myOutputDirectory
	<< "\n  gatekeeperId:          " << myGatekeeperId
    	<< "\n  endpointId:            " << myEndpointId;

    for ( list< Sptr<EndpointInfo> >::const_iterator i = myGateways.begin();
          i != myGateways.end();
          ++i
        )
    {
        if ( i == myGateways.begin() )
        {
            out << "\n\nStatic Registrations:";
        }
    
        out << "\n  " << *(*i);
    }
    
    return ( out );
}

                 
ReturnCode      
VocalGKConfig::loadGateways()
{
    const NameValueMap &    optionMap = myOptions.options();     
    NameValueMap::const_iterator i;

    // Load gateways value
    //
    if  (   (i = optionMap.find(gatewaysStr)) == optionMap.end() 
        ||  i->second.type == NO_VALUE
        )
    {
        return ( SUCCESS );
    }
    
    string  gateways = i->second.value.back();

    size_t position;

    size_t  count = 0;
    
    do
    {
        position = gateways.find_first_of(ParsePair::SEPARATORS);

        string current_gw = gateways.substr(0, position);

        if ( position != string::npos )
        {    
            gateways = gateways.substr(position);

            // Clear separators
            //
            position = gateways.find_first_not_of(ParsePair::SEPARATORS);
            gateways = gateways.substr(position);
        }
        
        // Load current gateway's call signaling address
        //
        IPAddress   callSignalAddress;
        
        string      gwCsAddrStr = current_gw + ".cs_address";

        if  (   (i = optionMap.find(gwCsAddrStr)) != optionMap.end() 
            &&  i->second.type != NO_VALUE
            )
        {
            callSignalAddress.setIP(i->second.value.back().c_str());
        }
        else
        {
            usage();
            cerr << "Error: " << gwCsAddrStr << " missing." << endl;
            return ( !SUCCESS );
        }

        // Load current gateway's prefixes
        //
        list<string>    prefixes;
        bool            stripPrefix = false;
        prefixes.clear();
        
        string          gwPrefixesStr = current_gw + ".prefix";
        string          gwStripPrefixStr = current_gw + ".strip_prefix";
        
        if  (   (i = optionMap.find(gwPrefixesStr)) != optionMap.end() 
            &&  i->second.type != NO_VALUE
            )
        {
            string  all_prefixes = i->second.value.back();

            size_t position;
            
            do 
            {
                position = all_prefixes.find_first_of(ParsePair::SEPARATORS);
                
                string current_prefix = all_prefixes.substr(0, position);

                if ( position != string::npos )
                {    
                    all_prefixes = all_prefixes.substr(position);

                    // Clear separators
                    //
                    position = all_prefixes.find_first_not_of(ParsePair::SEPARATORS);
                    all_prefixes = all_prefixes.substr(position);
                }

                prefixes.push_back(current_prefix);

            } while ( position != string::npos );

            if ( (i = optionMap.find(gwStripPrefixStr)) != optionMap.end() )
            {
                stripPrefix = true;
            }

        }
        else
        {
            // Accepts all prefixes
            //
            prefixes.push_back(string(""));
        }
        
        // Build the endpoint info.
        //
        Sptr<EndpointInfo>  endpointInfo = new EndpointInfo;
        
        char    countBuf[8];
        sprintf(countBuf, "%u", ++count);
        
        string  endpoint_id = current_gw;
        
        endpointInfo->endpointIdentifier(endpoint_id);
        endpointInfo->endpointType(EndpointInfo::GATEWAY_TYPE);
        endpointInfo->staticRegistration(true);
        endpointInfo->callSignalAddress(callSignalAddress);
        endpointInfo->prefix(prefixes);
        endpointInfo->stripPrefix(stripPrefix);
        endpointInfo->updateLastRegistration();
        
        // Validate the endpoint info
        //
        if ( !endpointInfo->valid() )
        {
            usage();
            cerr << "Error: Invalid endpoint information for gateway \""
                 << current_gw << '"' << endl;
            return ( !SUCCESS );
        }
        
        // Add the endpoint info into the endpoint list
        //
        myGateways.push_back(endpointInfo);        
        
    } while ( position != string::npos );

    return ( SUCCESS );
}
