#if !defined(VOCAL_ENDPOINT_INFO_HXX)
#define VOCAL_ENDPOINT_INFO_HXX

/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */


static const char* const EndpointInfo_hxx_Version = 
    "$Id: EndpointInfo.hxx,v 1.16 2001/11/08 02:47:27 icahoon Exp $";


#include "ptlib.h"
#include "h323pdu.h"

#include "Writer.hxx"
#include "IPAddress.hxx"
#include "FifoBase.h"
#include "Sptr.hxx"
#include "Event.hxx"
#include "Alias.hxx"

#include <string>
#include <list>
#include <sys/time.h>
#include <unistd.h>


/** Vovida Open Communication Application Library<br><br>
 */
namespace Vocal
{


/** SIP to H.323 Object Information<br><br>
 */
namespace SipH323Info
{


using Vocal::Transport::IPAddress;
using Vocal::Services::Event;
using Vocal::FifoBase;


/** Registration information for each endpoint.<br><br>
 *
 *  @see    Vocal::Transport::IPAddress
 */
class EndpointInfo : public Vocal::IO::Writer
{

    public:


    	/** Create an empty endpoint record 
    	 */
    	EndpointInfo();


    	/** Create endpoint record given an RRQ
    	 */
    	EndpointInfo(H225_RegistrationRequest & rrq);


    	/** Copy constructor
	 */
	EndpointInfo(const EndpointInfo &);


    	/** Virtual destructor
	 */
	virtual     	~EndpointInfo();


    	/** Assignment operator
	 */
    	EndpointInfo &  operator=(const EndpointInfo &);


    	/** Update the endpoint record given a RRQ. Returns true
	 *  if the resulting endpoint record is valid.
	 */
    	virtual bool	update(H225_RegistrationRequest & rrq);


    	/** Returns true is the endpoint record is valid.
	 *  An endpoint record is valid if it has a phone number in
	 *  the alias address, the endpoint identifier is not a null
	 *  string, it has a RAS address, and it has a call signal address.
	 */
    	virtual bool	valid() const;


    	/** Set the endpoint identifier.
	 */
    	void	    	endpointIdentifier(const string &);


    	/** Access the endpoint identifier.
	 */
    	string     	endpointIdentifier() const;


        /** The type of endpoint.
         */
        enum Type
        {
            UNDEFINED_TYPE = 0,
            TERMINAL_TYPE,
            GATEWAY_TYPE,
            MCU_TYPE,
            TYPE_SIZE
        };
        
        
        /** The type as a string.
         */
        static  const char * TypeName(Type);
        
        
        /** Set the endpoint type.
         */
        void            endpointType(Type);
        

        /** Access the endpoint type.
         */
        Type            endpointType() const;
        
        
    	/** Set the alias address.
	 */
    	void	    	alias(const H225_ArrayOf_AliasAddress &);


    	/** Access the alias address.
	 */
	H225_ArrayOf_AliasAddress   alias() const;


    	/** Set the gateway prefix.
	 */
    	void	    	prefix(const H225_GatewayInfo &);


    	/** Set the gateway prefix.
	 */
    	void	    	prefix(const list<string> &);


    	/** Access the gateway prefix.
	 */
	H225_ArrayOf_AliasAddress   prefix() const;


        /** If set, the gateway prefix will be stripped from the
         *  destination phone number when calling out to a gateway.
         */
        void            stripPrefix(bool);
        
        
        /** Access the stripPrefix flag.
         */
        bool            stripPrefix() const;
        

        /** Set to true if the registration is done statically.
         */
        void            staticRegistration(bool);
        

        /** Determine whether the prefix was set statically.
         */
        bool            staticRegistration() const;


    	/** Set the ras address.
	 */
    	void	    	rasAddress(const IPAddress &);


    	/** Access the ras address.
	 */
	IPAddress  	rasAddress() const;
	

    	/** Set the call signal address.
	 */
	void	    	callSignalAddress(const IPAddress &);


    	/** Access the call signal address.
	 */
	IPAddress  	callSignalAddress() const;


    	/** Set the time to live.
	 */
    	void	    	timeToLive(u_int32_t);


    	/** Access the time to live.
	 */
	u_int32_t  	timeToLive() const;
		

    	/** Set the event id for the keep alive timer.
	 */
	void	    	timeToLiveTimerEventId(FifoBase< Sptr<Event> >::EventId);


    	/** Access the event id for the keep alive timer.
	 */
	FifoBase< Sptr<Event> >::EventId timeToLiveTimerEventId() const;


    	/** Reset the registration time stamp.
	 */
    	void	    	updateLastRegistration();


    	/** Access the timestamp of the last registration.
	 */
	timeval    	lastRegistration() const;
		   

    	/** Reset the timestamp of the last activity.
	 */
	void	    	updateLastActivity();


    	/** Access the timestamp of the last activity.
	 */
	timeval    	lastActivity() const;


        /** This number of calls this endpoint has been admitted into.
         */
        u_int32_t        admissionCount() const;
        

        /** Admit this endpoint into a call.
         */
        void            admit();
                 

        /** Remove this endpoint from a call.
         */
        void            disengage();

    
        /** When set, the endpoint is almost out of resources.
         */         
        void            outOfResources(bool);


        /** When set, the endpoint is almost out of resources.
         */         
        bool            outOfResources() const;
        

    	/** Write the endpoint information to an ostream.
	 */
	virtual ostream &       writeTo(ostream &) const;


    protected:	

    	// Non Thread Safe methods: NTS
	//
		
    	virtual bool	    	NTS_update(
    	    	    	    	    H225_RegistrationRequest    &   rrq
				);

    	virtual bool	    	NTS_valid() const;

    	    	void	    	NTS_endpointIdentifier(const string &);
    	const	string &    	NTS_endpointIdentifier() const;

    	    	void	    	NTS_endpointType(Type);
    	        Type    	NTS_endpointType() const;

    	    	void	    	NTS_alias(const H225_ArrayOf_AliasAddress &);
	H225_ArrayOf_AliasAddress NTS_alias() const;

    	    	void	    	NTS_prefix(const H225_GatewayInfo &);
    	    	void	    	NTS_prefix(const H225_ArrayOf_AliasAddress &);
	H225_ArrayOf_AliasAddress NTS_prefix() const;

                void            NTS_stripPrefix(bool);
                bool            NTS_stripPrefix() const;

                void            NTS_staticRegistration(bool);
                bool            NTS_staticRegistration() const;

    	    	void	    	NTS_rasAddress(const IPAddress &);
	const	IPAddress & 	NTS_rasAddress() const;
	
	    	void	    	NTS_callSignalAddress(const IPAddress &);
	const	IPAddress &  	NTS_callSignalAddress() const;

    	    	void	    	NTS_timeToLive(u_int32_t);
	    	u_int32_t  	NTS_timeToLive() const;
		
		void	    	
		NTS_timeToLiveTimerEventId(FifoBase< Sptr<Event> >::EventId);
		    
		FifoBase< Sptr<Event> >::EventId NTS_timeToLiveTimerEventId() const;

    	    	void	    	NTS_updateLastRegistration();
    	const	timeval &   	NTS_lastRegistration() const;
		   
		void	    	NTS_updateLastActivity();
	const	timeval &    	NTS_lastActivity() const;
	

	mutable VMutex	    	    	    	    myMutex;

    private:
    	    	string	    	    	    	    myEndpointIdentifier;
                Type                                myEndpointType;
    	    	Alias 	    	    	    	    myAlias;
    	    	H225_ArrayOf_AliasAddress           myPrefix;
                bool                                myStripPrefix;
                bool                                myStaticRegistration;
		IPAddress   	    	    	    myRasAddress;
		IPAddress   	    	    	    myCallSignalAddress;
		u_int32_t   	    	    	    myTimeToLive;
		FifoBase< Sptr<Event> >::EventId    myTimeToLiveTimerEventId;
		timeval   	    	    	    myLastRegistration;
		timeval   	    	    	    myLastActivity;
                u_int32_t                           myAdmissions;
                bool                                myOutOfResources;
};


} // namespace SipH323Info
} // namespace Vocal


#endif // !defined(VOCAL_ENDPOINT_INFO_HXX)
