/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */


static const char* const EndpointInfoMap_cxx_Version = 
    "$Id: EndpointInfoMap.cxx,v 1.17 2001/05/29 09:00:12 icahoon Exp $";


#include "EndpointInfoMap.hxx"
#include "VLog.hxx"
#include <algorithm>


using Vocal::SipH323Info::EndpointInfoMap;
using Vocal::SipH323Info::EndpointInfo;
using Vocal::SipH323Info::EndpointIdMap;
using Vocal::SipH323Info::EndpointInfoSet;
using Vocal::SipH323Info::AliasMap;
using Vocal::SipH323Info::CallSignalAddressMap;
using Vocal::SipH323Info::PrefixMap;
using Vocal::SipH323Info::IPCompare;
using Vocal::SipH323Info::SptrEptInfoCompare;
using Vocal::SipH323Info::Alias;
using Vocal::Transport::IPAddress;
using Vocal::Logging::VLog;


bool 
IPCompare::operator()(
    const IPAddress 	&   left, 
    const IPAddress 	&   right
) const
{
    return 
    (	left.getIPAddress().sin_addr.s_addr 
	    <   right.getIPAddress().sin_addr.s_addr
    );
}


bool 
SptrEptInfoCompare::operator()(
    Sptr<EndpointInfo> 	left, 
    Sptr<EndpointInfo>  right
) const
{
    return ( left.operator->() < right.operator->() );
}


EndpointInfoMap::EndpointInfoMap()
{
}


EndpointInfoMap::~EndpointInfoMap()
{
}


void	    	    
EndpointInfoMap::insert(Sptr<EndpointInfo> endpointInfo)
{
    const string    fn("EndpointInfoMap::insert");
    VLog    	    log(fn);
    
    if ( endpointInfo == 0 || !endpointInfo->valid() )
    {
    	assert( "endpointInfo == 0 || !endpointInfo->valid()" == 0 );
	
    	VWARN(log)  << fn << ": attempted to insert invalid endpointInfo."
	    	    << VWARN_END(log);
    	return;
    }
    
    // Endpoint id is mandatory
    //
    myEndpointIdMap[endpointInfo->endpointIdentifier()] = endpointInfo;
    
    // Alias is not mandatory for gateways
    //
    if ( endpointInfo->alias().GetSize() > 0 )
    {
        // **** TODO ****
        // This is wasteful going from an Alias to an 
        // H225_ArrayOf_AliasAddress back to an Alias. 
        // Need to make an accessor for the alias in EndpointInfo.
        //
        Alias   alias = endpointInfo->alias();
        myAliasMap[alias] = endpointInfo;
    }

    // Call signal address is mandatory
    //
    myCallSignalAddressMap[endpointInfo->callSignalAddress()] = endpointInfo;


    // Prefix is optional and restricted to gateways only
    //

    if ( endpointInfo->endpointType() == EndpointInfo::GATEWAY_TYPE  )
    {
        H225_ArrayOf_AliasAddress   prefixes  = endpointInfo->prefix();
        size_t size = prefixes.GetSize();
        
        if ( size > 0 )
        {
            for ( size_t i = 0; i < size; i++ )
            {
                assert( prefixes[i].GetTag() == H225_AliasAddress::e_e164 );

                PASN_IA5String  &   e164Prefix = prefixes[i];

                string  e164(e164Prefix.GetValue());

                PrefixMap::iterator it = myPrefixMap.find(e164);

                if ( it == myPrefixMap.end() )
                {
                    EndpointInfoSet     eptInfoSet;
                    eptInfoSet.insert(endpointInfo);
                    myPrefixMap.insert(PrefixMap::value_type(e164, eptInfoSet));
                }
                else
                {
                    it->second.insert(endpointInfo);
                }

                VDEBUG(log) << fn << ": inserting prefix = \"" << e164 
                            << "\" => endpointInfo = { " 
                            << *endpointInfo << " }" << VDEBUG_END(log);
            }
        }
        else
        {
            string      nullPrefix("");
            
            PrefixMap::iterator it = myPrefixMap.find(nullPrefix);

            if ( it == myPrefixMap.end() )
            {
                EndpointInfoSet     eptInfoSet;
                eptInfoSet.insert(endpointInfo);
                myPrefixMap.insert(PrefixMap::value_type(nullPrefix, eptInfoSet));
            }
            else
            {
                it->second.insert(endpointInfo);
            }
        }
    }
    
    VDEBUG(log) << fn << ": inserting endpointInfo = { " << *endpointInfo
    	    	<< " }" << VDEBUG_END(log);

    VVERBOSE(log) << fn << *this << VVERBOSE_END(log);
}


void	    	    
EndpointInfoMap::erase(Sptr<EndpointInfo> endpointInfo)
{   
    const string    fn("EndpointInfoMap::erase");
    VLog    	    log(fn);

    if ( endpointInfo == 0 )
    {
    	VWARN(log)  << fn << ": attempted to erase null endpointInfo."
	    	    << VWARN_END(log);
    	return;
    }

    VDEBUG(log) << fn << ": erasing endpointInfo = { " << *endpointInfo
    	    	<< " }" << VDEBUG_END(log);

    EndpointIdMap::iterator   eptIdMapIter 
    	= myEndpointIdMap.find(endpointInfo->endpointIdentifier());

    if ( eptIdMapIter != myEndpointIdMap.end() )
    {
    	myEndpointIdMap.erase(eptIdMapIter);
    }
    else
    {
    	VDEBUG(log) << fn << ": endpointInfo = { " << *endpointInfo 
	    	    << " }, entry by endpointIdentifier not found."
		    << VDEBUG_END(log);
    }

    AliasMap::iterator	aliasMapIter 
    	= myAliasMap.find(endpointInfo->alias());

    if ( aliasMapIter != myAliasMap.end() )
    {
    	myAliasMap.erase(aliasMapIter);
    }
    else
    {
    	VDEBUG(log) << fn << ": endpointInfo = { " << *endpointInfo 
	    	    << " }, entry by alias not found."
		    << VDEBUG_END(log);
    }

    CallSignalAddressMap::iterator	callSignalAddressMapIter 
    	= myCallSignalAddressMap.find(endpointInfo->callSignalAddress());

    if ( callSignalAddressMapIter != myCallSignalAddressMap.end() )
    {
    	myCallSignalAddressMap.erase(callSignalAddressMapIter);
    }
    else
    {
    	VDEBUG(log) << fn << ": endpointInfo = { " << *endpointInfo 
	    	    << " }, entry by callSignalAddress not found."
		    << VDEBUG_END(log);
    }


    if ( endpointInfo->endpointType() == EndpointInfo::GATEWAY_TYPE )
    {
        H225_ArrayOf_AliasAddress   prefixes  = endpointInfo->prefix();
        size_t size = prefixes.GetSize();

        if ( size > 0 )
        {
            for ( size_t i = 0; i < size; i++ )
            {
                assert( prefixes[i].GetTag() == H225_AliasAddress::e_e164 );

                PASN_IA5String  &   e164Prefix = prefixes[i];

                string  e164(e164Prefix.GetValue());

                PrefixMap::iterator it = myPrefixMap.find(e164);

                if ( it != myPrefixMap.end() )
                {
                    EndpointInfoSet::iterator j = it->second.find(endpointInfo);

                    if ( j != it->second.end() )
                    {
                        VDEBUG(log) << fn << ": erasing prefix = \"" << e164
                            << "\" => endpointInfo = { " << *endpointInfo << " }"
                            << VDEBUG_END(log);

                        it->second.erase(j);
                    }

                    if ( it->second.empty() )
                    {
                        VDEBUG(log) << fn << ": erasing prefix = \"" << e164
                            << '"' << VDEBUG_END(log);

                        myPrefixMap.erase(it);
                    }
                }
            }
        }
        else
        {
            string  nullPrefix("");

            PrefixMap::iterator it = myPrefixMap.find(nullPrefix);

            if ( it != myPrefixMap.end() )
            {
                EndpointInfoSet::iterator j = it->second.find(endpointInfo);

                if ( j != it->second.end() )
                {
                    VDEBUG(log) << fn << ": erasing prefix = \"" << nullPrefix
                        << "\" => endpointInfo = { " << *endpointInfo << " }"
                        << VDEBUG_END(log);

                    it->second.erase(j);
                }

                if ( it->second.empty() )
                {
                    VDEBUG(log) << fn << ": erasing prefix = \"" << nullPrefix
                        << '"' << VDEBUG_END(log);

                    myPrefixMap.erase(it);
                }
            }
        }
    }

    VVERBOSE(log) << fn << *this << VVERBOSE_END(log);
}


Sptr<EndpointInfo>  
EndpointInfoMap::findByEndpointId(const string & eptId)
{
    const string    fn("EndpointInfoMap::findByEndpointId");
    VLog    	    log(fn);

    VVERBOSE(log) << fn << ": Searching for endpointId = " 
    	    	  << eptId << VVERBOSE_END(log);
    VVERBOSE(log) << fn << *this << VVERBOSE_END(log);

    EndpointIdMap::iterator it = myEndpointIdMap.find(eptId);
    return  (	it == myEndpointIdMap.end() 
    	    	?   Sptr<EndpointInfo>(0) 
		: (*it).second 
	    );
}


Sptr<EndpointInfo>  
EndpointInfoMap::findByAlias(const H225_ArrayOf_AliasAddress & alias__)
{
    Alias   alias(alias__);

    const string    fn("EndpointInfoMap::findByAlias");
    VLog    	    log(fn);

    VVERBOSE(log) << fn << ": Searching for alias = " 
    	    	  << alias << VVERBOSE_END(log);
    VVERBOSE(log) << fn << *this << VVERBOSE_END(log);

    AliasMap::iterator it = myAliasMap.find(alias);
    return  ( 	it == myAliasMap.end() 
    	    	?   Sptr<EndpointInfo>(0) 
		:   it->second 
	    );
}


Sptr<EndpointInfo>  
EndpointInfoMap::findByCallSignalAddress(const IPAddress & callSignalAddress)
{
    const string    fn("EndpointInfoMap::findByCallSignalAddress");
    VLog    	    log(fn);

    VVERBOSE(log) << fn << "Searching for callSignalAddress = " 
    	    	  << callSignalAddress << VVERBOSE_END(log);
    VVERBOSE(log) << fn << *this << VVERBOSE_END(log);

    CallSignalAddressMap::iterator it = myCallSignalAddressMap.find(callSignalAddress);
    return  ( 	it == myCallSignalAddressMap.end() 
    	    	?   Sptr<EndpointInfo>(0) 
		:   it->second 
	    );
}


class PrefixFinder
{
    public:
        PrefixFinder(const string & alias);

        bool operator()(const PrefixMap::value_type &) const;
        
        const string &  myAlias;
};


PrefixFinder::PrefixFinder(const string & alias)
    :   myAlias(alias)
{
}


bool
PrefixFinder::operator()(
    const PrefixMap::value_type &   prefix
) const
{
    return  (   prefix.first.size() > 0 
                ?   !myAlias.compare(prefix.first, 0, prefix.first.size())
                :   true        // Empty prefix matches any
            );
}


EndpointInfoSet
EndpointInfoMap::findByPrefix(const string & prefix, string & matchedPrefix)
{
    const string    fn("EndpointInfoMap::findByPrefix");
    VLog            log(fn);

    VVERBOSE(log) << fn << ": Searching for prefix = "
                  << prefix << VVERBOSE_END(log);
    VVERBOSE(log) << fn << *this << VVERBOSE_END(log);
    
    PrefixFinder    prefixFinder(prefix);

    PrefixMap::iterator it = find_if(myPrefixMap.begin(), 
				     myPrefixMap.end(), 
				     prefixFinder);
    
    if ( it != myPrefixMap.end() )
    {
        matchedPrefix = it->first;
    }
    else
    {
        matchedPrefix = "";
    }
    
    EndpointInfoSet     nullSet;
    
    return  (   it == myPrefixMap.end()
                ?   nullSet
                :   it->second
            );
}


void	    	    
EndpointInfoMap::clear()
{
    const string    fn("EndpointInfoMap::erase");
    VLog    	    log(fn);

    myEndpointIdMap.clear();
    myAliasMap.clear();
    myCallSignalAddressMap.clear();

    VDEBUG(log) << fn << VDEBUG_END(log);
    VVERBOSE(log) << fn << *this << VVERBOSE_END(log);
}


ostream &           
EndpointInfoMap::writeTo(ostream & out) const
{
    out << "\nendpointIdMap (size: " << myEndpointIdMap.size() << ") =\n{\n";    

    for (   EndpointIdMap::const_iterator it = myEndpointIdMap.begin();
    	    it != myEndpointIdMap.end();
	    it++
	)
    {
    	assert( (*it).second != 0 );
    	out << "  ( " << (*it).first << ", " << *(*it).second << " )\n";
    }
    out << '}';
    
    out << "\naliasMap (size: " << myAliasMap.size() << ") =\n{\n";
    
    for (   AliasMap::const_iterator it = myAliasMap.begin();
    	    it != myAliasMap.end();
	    it++
	)
    {
    	assert( (*it).second != 0 );
    	out << "  ( " << (*it).first << ", " << *(*it).second << ", )\n";
    }
    out << '}';

    out << "\ncallSignalAddressMap (size: " << myCallSignalAddressMap.size() << ") =\n{\n";
    
    for (   CallSignalAddressMap::const_iterator it = myCallSignalAddressMap.begin();
    	    it != myCallSignalAddressMap.end();
	    it++
	)
    {
    	assert( (*it).second != 0 );
    	out << "  ( " << (*it).first << ", " << *(*it).second << ", )\n";
    }
    out << '}';

    return ( out );
}

EndpointIdMap &
EndpointInfoMap::endpointIdMap()
{
    return ( myEndpointIdMap );
}
