/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */


static const char* const GatekeeperInfo_cxx_Version = 
    "$Id: GatekeeperInfo.cxx,v 1.13 2001/11/07 08:57:15 icahoon Exp $";


#include "GatekeeperInfo.hxx"
#include "SystemException.hxx"
#include "Lock.hxx"
#include "VLog.hxx"
#include <strstream.h>
#include <unistd.h>
#include <netdb.h>
#include <cerrno>


using Vocal::SipH323Info::GatekeeperInfo;
using Vocal::SipH323Info::EndpointInfo;
using Vocal::Transport::IPAddress;
using Vocal::Threads::Lock;
using Vocal::Logging::VLog;


u_int32_t   GatekeeperInfo::myEndpointNumber = 1;


GatekeeperInfo::GatekeeperInfo(
    list< Sptr<EndpointInfo> >      staticGWs,
    const string                &   gatekeeperId,
    const string                &   endpointId,
    bool    	    	            stndAlone,
    bool    	    	            gkRouted,
    bool                            pDiscoveryEnabled,
    bool                            pRegistrationEnabled,
    bool                            pAdmissionEnabled,
    bool                            pLocationEnabled,
    bool                            pResourcesAvailableEnabled
)
throw ( Vocal::SystemException )
    :   myDiscoveryEnabled(pDiscoveryEnabled),
        myRegistrationEnabled(pRegistrationEnabled),
        myAdmissionEnabled(pAdmissionEnabled),
        myLocationEnabled(pLocationEnabled),
        myResourcesAvailableEnabled(myResourcesAvailableEnabled)
{
    const string    fn("GatekeepeperInfo::GatekeepeperInfo");
    VLog    	    log(fn);

    Lock    	lock(myMutex); (void)lock;

    // Copy the statically registered gateways
    //
    myStaticGateways = staticGWs;

    // Setup hostName and hostAddress
    //
    char    localHostName[128];
    if ( gethostname(localHostName, 128) < 0 )
    {
    	throw Vocal::SystemException(
	    	fn + ": gethostname failed: error = " + strerror(errno), 
	    	__FILE__, __LINE__, errno);
    }
    
    myHostName = localHostName;

    VDEBUG(log) << fn << ": host name = " << myHostName << VDEBUG_END(log);

    // Setup the hostAddress
    //    
    hostent *	hostEntry = gethostbyname(localHostName);
    
    if ( hostEntry == 0 )
    {
    	throw Vocal::SystemException(fn + ": gethostbyname failed.", 
	    	    	    	__FILE__, __LINE__, errno);
    }

    sockaddr_in inAddr;
    inAddr.sin_port = 0;
    inAddr.sin_addr.s_addr = *((u_int32_t *)(hostEntry->h_addr_list[0]));

    myHostAddress.setIPAddress(inAddr);

    VDEBUG(log) << fn << ": host address = " << myHostAddress << VDEBUG_END(log);

    // Setup the gatekeeperIdentifier
    //
    if ( gatekeeperId.size() == 0 )
    {
        myGatekeeperIdentifier = myHostName;
        /*
        string::size_type	pos = myGatekeeperIdentifier.find(".");
    
	if ( pos != string::npos )
	{
    	    myGatekeeperIdentifier.resize(pos);
	}
        */
    }
    else
    {
    	myGatekeeperIdentifier = gatekeeperId;
    }

    VDEBUG(log) << fn << ": gatekeeper Identifier = " 
    	    	<< myGatekeeperIdentifier << VDEBUG_END(log);

    // Setup rasAddress from hostAddress
    //
    myRasAddress = myHostAddress;
    myRasAddress.setPort(1719);   

    VDEBUG(log) << fn << ": ras address = " << myRasAddress << VDEBUG_END(log);

    // Setup rasMulticastAddress, using well defined value
    // See section IV.1.1.1 of h.225.0 version 2 (2/98).
    //
    myRasMulticastAddress.setIP("224.0.1.41");
    myRasMulticastAddress.setPort(1718);
    
    VDEBUG(log) << fn << ": multicast ras address = " << myRasMulticastAddress
                << VDEBUG_END(log);
                
    // Setup callSignalAddress from hostAddress
    //
    myCallSignalAddress = myHostAddress;
    myCallSignalAddress.setPort(1720);   

    VDEBUG(log) << fn << ": call signal address = " 
    	    	<< myCallSignalAddress << VDEBUG_END(log);

    // Setup endpointIdentifier.
    //
    if ( endpointId.size() == 0 )
    {
        myEndpointIdentifier = "endpoint_";
    }
    else
    {
        myEndpointIdentifier = endpointId;
    }

    VDEBUG(log) << fn << ": endpointIdentifier = " 
    	    	<< myEndpointIdentifier << VDEBUG_END(log);

    myGatekeeperRouted = gkRouted;

    VDEBUG(log) << fn << ": call model = " 
    	    	<< ( myGatekeeperRouted ? "gatekeeper routed" : "point to point" )
		<< VDEBUG_END(log);

    myStandAlone = stndAlone;

    VDEBUG(log) << fn << ": stand alone = " 
    	    	<< ( myStandAlone ? "true" : "false" )
		<< VDEBUG_END(log);
    
    // Setup h225ProtocolID
    //
    myH225ProtocolID = "0.0.8.2250.0.2";

    VDEBUG(log) << fn << ": h225ProtocolID = " 
    	    	<< myH225ProtocolID << VDEBUG_END(log);
}


GatekeeperInfo::~GatekeeperInfo()
{
}


bool                
GatekeeperInfo::discoveryEnabled() const
{
    Lock    lock(myMutex); (void)lock;
    return ( myDiscoveryEnabled );
}


void                
GatekeeperInfo::discoveryEnabled(bool pDiscoveryEnabled)
{
    Lock    lock(myMutex); (void)lock;
    myDiscoveryEnabled = pDiscoveryEnabled;
}


bool                
GatekeeperInfo::registrationEnabled() const
{
    Lock    lock(myMutex); (void)lock;
    return ( myRegistrationEnabled );
}


void                
GatekeeperInfo::registrationEnabled(bool pRegistrationEnabled)
{
    Lock    lock(myMutex); (void)lock;
    myRegistrationEnabled = pRegistrationEnabled;
}


bool                
GatekeeperInfo::admissionEnabled() const
{
    Lock    lock(myMutex); (void)lock;
    return ( myAdmissionEnabled );
}


void                
GatekeeperInfo::admissionEnabled(bool pAdmissionEnabled)
{
    Lock    lock(myMutex); (void)lock;
    myAdmissionEnabled = pAdmissionEnabled;
}


bool                
GatekeeperInfo::locationEnabled() const
{
    Lock    lock(myMutex); (void)lock;
    return ( myLocationEnabled );
}


void                
GatekeeperInfo::locationEnabled(bool pLocationEnabled)
{
    Lock    lock(myMutex); (void)lock;
    myLocationEnabled = pLocationEnabled;
}


bool                
GatekeeperInfo::resourcesAvailableEnabled() const
{
    Lock    lock(myMutex); (void)lock;
    return ( myResourcesAvailableEnabled );
}


void                
GatekeeperInfo::resourcesAvailableEnabled(bool pResourcesAvailableEnabled)
{
    Lock    lock(myMutex); (void)lock;
    myResourcesAvailableEnabled = pResourcesAvailableEnabled;
}


string	    	    	
GatekeeperInfo::generateEndpointIdentifier()
{
    Lock    lock(myMutex); (void)lock;
    
    ostrstream	eptId;
    eptId << myEndpointIdentifier << myEndpointNumber++ << ends;

    string endpointId(eptId.str());

    eptId.freeze(false);

    return ( endpointId );
}


void	    	    
GatekeeperInfo::hostName(const string & host_name)
{
    Lock    lock(myMutex); (void)lock;
    myHostName = host_name;
}


string      	    
GatekeeperInfo::hostName() const
{
    Lock    lock(myMutex); (void)lock;
    return ( myHostName );
}


void        	    
GatekeeperInfo::hostAddress(const IPAddress & hostAddr)
{
    Lock    lock(myMutex); (void)lock;
    myHostAddress = hostAddr;
}


IPAddress   	    
GatekeeperInfo::hostAddress() const
{
    Lock    lock(myMutex); (void)lock;
    return ( myHostAddress );
}


list< Sptr<EndpointInfo> >
GatekeeperInfo::staticGateways() const
{
    Lock    lock(myMutex); (void)lock;
    return ( myStaticGateways );
}


void	    	    
GatekeeperInfo::gatekeeperIdentifier(const string & gkid)
{
    Lock    lock(myMutex); (void)lock;
    myGatekeeperIdentifier = gkid;
}


string	    	    
GatekeeperInfo::gatekeeperIdentifier() const
{
    Lock    lock(myMutex); (void)lock;
    return ( myGatekeeperIdentifier );
}


void	    	    
GatekeeperInfo::rasAddress(const IPAddress & rasAddr)
{
    Lock    lock(myMutex); (void)lock;
    myRasAddress = rasAddr;
}


IPAddress   	    
GatekeeperInfo::rasAddress() const
{
    Lock    lock(myMutex); (void)lock;
    return ( myRasAddress );
}


void	    	    
GatekeeperInfo::rasMulticastAddress(const IPAddress & rasMAddr)
{
    Lock    lock(myMutex); (void)lock;
    myRasMulticastAddress = rasMAddr;
}


IPAddress   	    
GatekeeperInfo::rasMulticastAddress() const
{
    Lock    lock(myMutex); (void)lock;
    return ( myRasMulticastAddress );
}


void	    	    
GatekeeperInfo::callSignalAddress(const IPAddress & csAddr)
{
    Lock    lock(myMutex); (void)lock;
    myCallSignalAddress = csAddr;
}


IPAddress   	    
GatekeeperInfo::callSignalAddress() const
{
    Lock    lock(myMutex); (void)lock;
    return ( myCallSignalAddress );
}


void	    	    
GatekeeperInfo::endpointIdentifier(const string & eptId)
{
    Lock    lock(myMutex); (void)lock;
    myEndpointIdentifier = eptId;
}


string	    	    
GatekeeperInfo::endpointIdentifier() const
{
    Lock    lock(myMutex); (void)lock;
    return ( myEndpointIdentifier );
}


void	    	    
GatekeeperInfo::gatekeeperRouted(bool gkRouted)
{
    Lock    lock(myMutex); (void)lock;
    myGatekeeperRouted = gkRouted;
}


bool	    	    
GatekeeperInfo::gatekeeperRouted() const
{
    Lock    lock(myMutex); (void)lock;
    return ( myGatekeeperRouted );
}


void	    	    
GatekeeperInfo::standAlone(bool stndAlone)
{
    Lock    lock(myMutex); (void)lock;
    myStandAlone = stndAlone;
}


bool	    	    
GatekeeperInfo::standAlone() const
{
    Lock    lock(myMutex); (void)lock;
    return ( myStandAlone );
}


string	    	    
GatekeeperInfo::h225ProtocolID() const
{
    Lock    lock(myMutex); (void)lock;
    return ( myH225ProtocolID );
}

