/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */


static const char* const Call_cxx_Version = 
    "$Id: Call.cxx,v 1.20 2001/05/09 22:57:14 vzubarev Exp $";


#include "VLog.hxx"

#include "TPKTClientSocket.hxx"
#include "PollFifoService.hxx"

#include "RouterInfo.hxx"

#include "Call.hxx"
#include "H245Protocol.hxx"
#include "H245ServerProtocol.hxx"
#include "SesH245.hxx"
#include "SesH245.hxx"
#include "RouterCallSig.hxx"
#include "RouterCallCtl.hxx"
#include "Q931ProtocolBase.hxx"

using namespace router;
using Vocal::Transport::TransportAddress;
using Vocal::Logging::VLog;

Call::
Call(mte::Mediator& master)
    : SMCall(),
      master_(master),
      sigConnection_(this),
      ctlConnection_(this),
      callsig_(this),
      msd_(this),
      rtd_(this),
      ceo_(this),
      cei_(this),
      callCtlSession_(this),
      openCC_(this, &h245ActiveProtocol_),
      acceptCC_(this),
      h245_(this),
      rtLco_(*this),
      rtLci_(*this),
      fifo_(0),
      h245PassiveProtocol_(0),
      h245ActiveProtocol_(0),
      h245Protocol_(0),
      ccServerProtocol_(0),
      sesH245_(0),
      routerCallSig_(0),
      routerCallCtl_(0),
      q931Protocol_(0),
      routerInfo_(0),
      evtH245TCPOpened_(this),
      q931EvtSource_(0),
      terminateReason_(H225_ReleaseCompleteReason::e_undefinedReason),
      evtTCSRejected_(this),
      evtTCSConfirmed_(this),
      evtMSDStarted_(this),
      evtMSDFailed_(this),
      evtMSDConfirmed_(this)
{
    openCC_.ccConnected_ = mte::EventPtr<CallbackEvent>(&evtH245TCPOpened_);
    acceptCC_.ccConnected_ = mte::EventPtr<CallbackEvent>(&evtH245TCPOpened_);
    ceo_.callbackReject_ = mte::EventPtr<CallbackEvent>(&evtTCSRejected_);
    ceo_.callbackConfirm_ = mte::EventPtr<CallbackEvent>(&evtTCSConfirmed_);
    msd_.callbackIndication_ = mte::EventPtr<CallbackEvent>(&evtMSDStarted_);
    msd_.callbackReject_ = mte::EventPtr<CallbackEvent>(&evtMSDFailed_);
    msd_.callbackConfirm_ = mte::EventPtr<EvtMSDConfirmed>(&evtMSDConfirmed_);
}


Call::
~Call()
{
    const string    	fn("Call::~Call");
    VLog    	    	log(fn);

    routerCallSig_->remove(sigConnection_);
    delete ccServerProtocol_;
    delete h245ActiveProtocol_;
    delete h245PassiveProtocol_;
    delete q931Protocol_;
}


void Call::
init()
{
    assert(this);

    if (!q931Protocol_)
    {
        q931Protocol_ = new Q931ProtocolBase(
            *fifo_,
		    new TPKTClientSocket,
            *q931EvtSource_,
            *routerInfo_
            );
    }
    assert(q931Protocol_);

    sigConnection_.setSocket(&(q931Protocol_->getTPKTClientSocket()));
    q931Protocol_->setConnection(&sigConnection_);
    routerCallSig_->add(sigConnection_); // To be done in router?

    delete ccServerProtocol_;
    ccServerProtocol_ = new H245ServerProtocol(
        *this,
        *fifo_,
        *sesH245_,
        *routerCallCtl_,
        ctlConnection_,
        &h245PassiveProtocol_
    );
    assert(ccServerProtocol_);
}


mte::Mediator& Call::
getMaster() throw()
{
    assert(this);

    return master_;
}


FeatureCallSigBase* Call::
getCallSig() throw()
{
    assert(this);

    return &callsig_;
}


FeatureCallSig& Call::
getCallSigImp() throw()
{
    assert(this);

    return callsig_;
}


FeatureMsdBase* Call::
getMsd() throw()
{
    assert(this);

    return &msd_;
}


FeatureMsd& Call::
getMsdImp() throw()
{
    assert(this);

    return msd_;
}


FeatureRTDBase* Call::
getRTD() throw()
{
    return &rtd_;
}


FeatureRTD& Call::
getRTDImp() throw()
{
    return rtd_;
}


FeatureCEOBase* Call::
getCEO() throw()
{
    assert(this);

    return &ceo_;
}


FeatureCEO& Call::
getCEOImp() throw()
{
    assert(this);

    return ceo_;
}


FeatureCEIBase* Call::
getCEI() throw()
{
    assert(this);

    return &cei_;
}


FeatureCEI& Call::
getCEIImp() throw()
{
    assert(this);

    return cei_;
}


FeatureCallCtlSessionBase* Call::
getCallCtlSession() throw()
{
    assert(this);

    return &callCtlSession_;
}


FeatureCallCtlSession& Call::
getCallCtlSessionImp() throw()
{
    assert(this);

    return callCtlSession_;
}


FeatureOpenCCBase* Call::
getOpenCC() throw()
{
    assert(this);

    return &openCC_;
}


FeatureOpenCC& Call::
getOpenCCImp() throw()
{
    assert(this);

    return openCC_;
}


FeatureAcceptCCBase* Call::
getAcceptCC() throw()
{
    assert(this);

    return &acceptCC_;
}


FeatureAcceptCC& Call::
getAcceptCCImp() throw()
{
    assert(this);

    return acceptCC_;
}


FeatureH245Base* Call::
getH245() throw()
{
    assert(this);

    return &h245_;
}


FeatureH245& Call::
getH245Imp() throw()
{
    assert(this);

    return h245_;
}


ConnectionCallSig* Call::
getCallSigConnection() throw()
{
    assert(this);

    return &sigConnection_;
}


ConnectionCallCtl* Call::
getCallCtlConnection() throw()
{
    assert(this);

    return &ctlConnection_;
}


RouterLCO* Call::
getRtLCO() throw()
{
    assert(this);

    return &rtLco_;
}


RouterLCI* Call::
getRtLCI() throw()
{
    assert(this);

    return &rtLci_;
}


void Call::
makeH245Protocol()
{
    delete h245ActiveProtocol_;
    h245ActiveProtocol_ = 0;

    Sptr<TPKTClientSocket> socket = new TPKTClientSocket;

    h245ActiveProtocol_ = new H245Protocol(
        *fifo_,
        socket,
        *sesH245_,
        *routerCallCtl_,
        ctlConnection_
        );
}


PollFifoService* Call::
getFifo() throw()
{
    assert(this);

    return fifo_;
}


void Call::
setFifo(PollFifoService* fifo) throw()
{
    assert(this);
    assert(fifo);
    
    fifo_ = fifo;
}


void Call::
setSesH245(SesH245* ses) throw()
{
    assert(this);
    assert(ses);
    
    sesH245_ = ses;
}


void Call::
setCallSigRouter(RouterCallSig* q931Router) throw()
{
    assert(this);
    assert(q931Router);

    routerCallSig_ = q931Router;
}


void Call::
setCallCtlRouter(RouterCallCtl* h245Router) throw()
{
    assert(this);
    assert(h245Router);

    routerCallCtl_ = h245Router;
}


void Call::
setRouterInfo(RouterInfo* routerInfo) throw()
{
    assert(this);
    assert(routerInfo);

    routerInfo_ = routerInfo;
}


RouterInfo* Call::
getRouterInfo() throw()
{
    return routerInfo_;
}


void Call::
selectActiveProtocol() throw()
{
    assert(this);
    assert(h245ActiveProtocol_);

    h245Protocol_ = h245ActiveProtocol_;

    if (h245PassiveProtocol_)
    {
        h245PassiveProtocol_->ignore();
    }
}


void Call::
selectPassiveProtocol() throw()
{
    assert(this);
    assert(h245PassiveProtocol_);

    h245Protocol_ = h245PassiveProtocol_;
    delete h245ActiveProtocol_;
    h245ActiveProtocol_ = 0;
}


const IPAddress& Call::
getH245Address() const
{
    assert(this);

    const TransportAddress& address =
        ccServerProtocol_->getTPKTServerSocket().getLocalAddress();

    // TODO: transport library should allow getting address without casting
    return dynamic_cast<const IPAddress&>(address);
}


TPKTClientSocket* Call::
getH245Socket()
{
    assert(this);

    if (h245Protocol_)
    {
        return &(h245Protocol_->getTPKTClientSocket());
    }
    else
    {
        return 0;
    }
}


IPAddress Call::getCallSignalAddress() const
{
    assert(this);
    assert(routerInfo_);
    
    return routerInfo_->callSignalAddress();
}


void Call::
setProtocol(Q931ProtocolBase* q931Protocol) throw()
{
    assert(this);

    q931Protocol_ = q931Protocol;
}


Q931ProtocolBase* Call::
getProtocol() throw ()
{
    return q931Protocol_;
}

H245Protocol* Call::
getH245Protocol() throw ()
{
    return h245Protocol_;
}

void Call::
setQ931Src(SesQ931* q931EvtSource) throw ()
{
    q931EvtSource_ = q931EvtSource;
}


int Call::
getTerminateReason() const throw()
{
    assert(this);

    return terminateReason_;
}


void Call::
setTerminateReason(int reason) throw()
{
    assert(this);

    terminateReason_ = reason;
}


bool Call::
isActiveH245AddressGreater()
{
    assert(this);
    assert(h245PassiveProtocol_ || h245ActiveProtocol_);

    if (!h245PassiveProtocol_)
    {
        return true;
    }
    else if (!h245ActiveProtocol_)
    {
        return false;
    }
    else
    {
        const TransportAddress& addressA =
            h245ActiveProtocol_->getTPKTClientSocket().getLocalAddress();
        const TransportAddress& addressP =
            h245PassiveProtocol_->getTPKTClientSocket().getLocalAddress();
        const IPAddress& addressAI =
            dynamic_cast<const IPAddress&>(addressA);
        const IPAddress& addressPI =
            dynamic_cast<const IPAddress&>(addressP);

        return addressAI > addressPI;
    }
}


void Call::
disable(H245Protocol* protocol)
{
    if (protocol == h245PassiveProtocol_)
    {
        delete h245PassiveProtocol_;
        h245PassiveProtocol_ = 0;
    }
    else if (protocol == h245ActiveProtocol_)
    {
        delete h245ActiveProtocol_;
        h245ActiveProtocol_ = 0;
    }
    else
    {
        assert(false);
    }
}


void Call::
complete()
{
    assert(this);
    const string    	fn("Call::complete");
    VLog    	    	log(fn);

    callbackComplete_.start();
}


void Call::
terminate(int reason = H225_ReleaseCompleteReason::e_undefinedReason)
{
    assert(this);
    const string    	fn("Call::terminate");
    VLog    	    	log(fn);

    if (reason != H225_ReleaseCompleteReason::e_undefinedReason)
    {
	setTerminateReason(reason);
    }

    callbackTerminate_.start();
}


void Call::
close()
{
    assert(this);
    const string    	fn("Call::close");
    VLog    	    	log(fn);

    callbackClose_.start();
}


void Call::
startTimer(Sptr<mte::EvtTimer<TimerEvent> > timer)
{
    const string    	fn("Call::startTimer");
    VLog    	    	log(fn);

    if (fifo_ && !timer->id_)
    {
        timer->id_ = fifo_->getFifo().addDelayMs(timer, timer->getTimerValue());
    }
}


void Call::
stopTimer(Sptr<mte::EvtTimer<TimerEvent> > timer)
{
    if (fifo_ && timer->id_)
    {
    const string    	fn("canceling timer");
    VLog    	    	log(fn);
        fifo_->getFifo().cancel(timer->id_);
    const string    	fn2("done canceling timer");
    VLog    	    	log2(fn2);
	timer->id_ = 0;
    }
}


SesH245* Call::
getSesH245() throw ()
{
    return sesH245_;
}


#ifndef DISABLE_WRITER

ostream& Call::
writeTo(ostream& out) const
{
    out << "Call: " << this << "\n"
        << "{\n"
        << "    " << sigConnection_ << "\n"
        << "    " << ctlConnection_ << "\n"
        << "    " << callsig_ << "\n"
        << "    " << msd_ << "\n"
        << "    " << rtd_ << "\n"
        << "    " << ceo_ << "\n"
        << "    " << cei_ << "\n"
        << "    " << callCtlSession_ << "\n"
        << "    " << openCC_ << "\n"
        << "    " << acceptCC_ << "\n"
        << "    Termination reason = " << terminateReason_ << "\n"
        << "}" << endl;
    return out;
}

#endif
