/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */


static const char* const ConnectionCallSig_cxx_Version = 
    "$Id: ConnectionCallSig.cxx,v 1.23 2001/06/01 06:07:08 icahoon Exp $";


#include "VLog.hxx"
#include "PollFifoService.hxx"

#include "RouterInfo.hxx"

#include "ConnectionCallSig.hxx"
#include "ConnectionCallCtl.hxx"
#include "H323CodecException.hxx"
#include "H323Consts.hxx"
#include "EvtCCSentAddress.hxx"
#include "Q931ProtocolBase.hxx"
#include "CallTrace.hxx"
#include "H245Defs.hxx"
#include "SesH245.hxx"
#include "EvtReceivedCCAddress.hxx"


using namespace router;
using Vocal::Logging::VLog;


ConnectionCallSig::
ConnectionCallSig(SMCall* sm)
    : ConnCallSig(sm),
      outgoingCall_(false),
      socket_(0),
      connected_(false),
      haveRemoteH245_(false),
      overrideH245_(false),
      h245tunneling_(true),
      fastConnect_(false)
{
    conferenceID_ =
        H323GloballyUniqueID();
    callID_ =
        H323GloballyUniqueID();
}


void ConnectionCallSig::
setSocket(TPKTClientSocket* socket)
{
    socket_ = socket;
}


void ConnectionCallSig::
close()
{
    socket_ = 0;
    connected_ = false;
}


void ConnectionCallSig::
disable()
{
    connected_ = false;
}


void ConnectionCallSig::
connect()
{
    VLog log;
    VDEBUG(log) << "ConnectionCallSig::connect "
                << socket_
                << VDEBUG_END(log);
    VDEBUG(log) << "... containing "
                << *socket_
                << VDEBUG_END(log);
    VDEBUG(log) << "... to "
                << data_.toIP_
                << VDEBUG_END(log);

    try
    {
        socket_->connect(data_.toIP_);
        VDEBUG(log) << "Connect command sent" << VDEBUG_END(log);
    }
    catch(Vocal::SystemStatus& status)
    {
        // Ignore EINPROGRESS: we'll get 'outgoing available' anyway
    }

    getSM()->getProtocol()->awaitOutgoing(true);
    getSM()->getProtocol()->awaitIncoming(false);
    getSM()->getFifo()->registerProtocol(*(getSM()->getProtocol()));
    VDEBUG(log) << "Q931 client protocol registered" << VDEBUG_END(log);
}


void ConnectionCallSig::
connected() throw()
{
    connected_ = true;
}


void ConnectionCallSig::
attachTunneling(Q931Base& message)
{
    VLog log;
    VDEBUG(log) << "ConnectionCallSig::attachTunneling" << VDEBUG_END(log);

    while (!getSM()->getCallCtlConnection()->queue_.empty())
    {
        H245Message ccMessage =
            getSM()->getCallCtlConnection()->queue_.front();

        PPER_Stream strm;
        ccMessage.Encode(strm);
        strm.CompleteEncoding();

        PASN_OctetString pos; pos = strm;

        if( !message.m_h323_uu_pdu.HasOptionalField(
            H225_H323_UU_PDU::e_h245Control) )
        {
            message.m_h323_uu_pdu.IncludeOptionalField(
                H225_H323_UU_PDU::e_h245Control);
        }

        unsigned size = message.m_h323_uu_pdu.m_h245Control.GetSize();
        message.m_h323_uu_pdu.m_h245Control.SetSize(size + 1);
        message.m_h323_uu_pdu.m_h245Control[size] = pos;

        getSM()->getCallCtlConnection()->queue_.pop();
    }
}


void ConnectionCallSig::
send(Q931Base& message)
{
    VLog log;
    VDEBUG(log) << "ConnectionCallSig::send" << VDEBUG_END(log);

    // Send tunneling flag with each message
    message.m_h323_uu_pdu.m_h245Tunneling = h245tunneling_;

    // Attach tunneled data
    attachTunneling(message);

    // Encode the H.225 PDU into the Q931 PDU as User-User data
    PPER_Stream strm;
    message.Encode(strm);
    strm.CompleteEncoding();
    message.getQ931().SetIE(Q931::UserUserIE, strm);

#ifndef DISABLE_WRITER
    CALLTRACE(log, CT_OUT, 
    	      ( socket_ ? socket_->getRemoteAddress() : 0 ), 
	      CT_Q931, string(message.m().GetTagName())) 
        << message << setprecision(0) << "\n(Q.931)= " << message.getQ931()
	<< CALLTRACE_END(log);
#endif

    // Encode Q.931 PDU
    PBYTEArray rawData;
    if (!message.getQ931().Encode(rawData))
    {
        throw H323CodecException(
            "Problem encoding Q.931 pdu",
            __FILE__, __LINE__,
            H323CodecException::Q931_ENCODE_ERROR);
    }

    // Send it to TCP socket
    if (socket_ != 0 && connected_)
    {
        // TODO: is there a better way to get uint8*?
        PASN_OctetString pos; pos = rawData;

        VDEBUG(log) << "Sending Q931 PDU"
                    << VDEBUG_END(log);

        socket_->send(pos.GetPointer(), rawData.GetSize());
    }
    else
    {
        VDEBUG(log) << "No socket to send Q.931 message, that's OK if we just got"
                    << " ReleaseComplete or Q931 disconnect" << VDEBUG_END(log);
    }
}

void ConnectionCallSig::
processAttachments(Q931Base& message)
{
    VLog log;
    VDEBUG(log) << "ConnectionCallSig::processAttachments" << VDEBUG_END(log);

    // For each attached H.245 control element:
    for (unsigned int i = 0;
	 i < message.m_h323_uu_pdu.m_h245Control.GetSize();
	 i ++)
    {
	// Extract control element from H.225 message
	PASN_OctetString &rControlPDUStream =
	    message.m_h323_uu_pdu.m_h245Control[i];

	// Prepare stream for decoding
	PPER_Stream strm(
	    (const unsigned char*)rControlPDUStream.GetPointer(),
	    rControlPDUStream.GetSize());

	// Make decoded H.245 message
	H245Message msg;
	msg.DecodePER(strm);

#ifndef DISABLE_WRITER
	VINFO(log) << "Got tunneled PDU\n"
		   << "{\n  " << setprecision(2) << msg
		   << "\n}" << VINFO_END(log);
#endif
	// Pass H.245 message to the Call Control Connection
	getSM()->getSesH245()->generate(
	    msg, getSM()->getCallCtlConnection());
    }
}


void ConnectionCallSig::
onIncoming(Q931Base& message)
{
    VLog log;
    VDEBUG(log) << "ConnectionCallSig::onIncoming" << VDEBUG_END(log);

    int msgCallRef = static_cast<int>(message.getQ931().GetCallReference());
    
    // Setup specific info
    if (callRef_ == -1)
    {
        callRef_ = msgCallRef;
    }
    else if (callRef_ != msgCallRef)
    {
        VWARN(log) << "CallRef changed during the call" << VWARN_END(log);
    }

    if (message.getQ931().GetMessageType() == Q931::SetupMsg)
    {
        data_.fromDisplay_ = message.getQ931().GetDisplayName();
    }
    else if (message.getQ931().GetMessageType() == Q931::ConnectMsg)
    {
        data_.toDisplay_ = message.getQ931().GetDisplayName();
    }
    else
    {
        // Ignore "display" field
    }

    // If tunneling is disabled from remote
    if (!message.m_h323_uu_pdu.m_h245Tunneling)
    {
        // No tunneling anymore
        clearH245Tunneling();
    }

    if (message.getQ931().GetMessageType() == Q931::SetupMsg &&
        message.m_h323_uu_pdu.m_h323_message_body.GetTag() ==
        H225Tag::e_setup)
    {
        VDEBUG(log) << "Parsing Setup message" << VDEBUG_END(log);

        H225Setup& setup = message.m_h323_uu_pdu.m_h323_message_body;

        conferenceID_ = setup.m_conferenceID;

        if (setup.HasOptionalField(H225_Setup_UUIE::e_callIdentifier))
        {
            callID_ = setup.m_callIdentifier.m_guid;
        }

        if (setup.HasOptionalField(H225_Setup_UUIE::e_sourceAddress))
        {
            data_.fromAliases_ = setup.m_sourceAddress;
        }

        if (setup.HasOptionalField(H225_Setup_UUIE::e_destinationAddress))
        {
            data_.toAliases_ = setup.m_destinationAddress;
        }

        if (setup.HasOptionalField(
            H225_Setup_UUIE::e_sourceCallSignalAddress))
        {
            if(setup.m_sourceCallSignalAddress.GetTag() ==
               H225_TransportAddress::e_ipAddress)
            {
                H225_TransportAddress_ipAddress& ipAddress =
                    setup.m_sourceCallSignalAddress;
                data_.fromIP_.setIP(ipAddress.m_ip);
                data_.fromIP_.setPort(ipAddress.m_port);
            }
        }

        if (setup.HasOptionalField(
            H225_Setup_UUIE::e_destCallSignalAddress))
        {
            if(setup.m_destCallSignalAddress.GetTag() ==
               H225_TransportAddress::e_ipAddress)
            {
                H225_TransportAddress_ipAddress& ipAddress =
                    setup.m_destCallSignalAddress;
                data_.toIP_.setIP(ipAddress.m_ip);
                data_.toIP_.setPort(ipAddress.m_port);
            }
        }

        if (message.m_h323_uu_pdu.m_h245Control.GetSize() > 0 &&
            !getSM()->getRouterInfo()->routerMediaControl())
        {
            /* Remote side wants to use tunneling. If our application
               will use direct connection of Control Channels, tunneling
               should be disabled. Since there is no way to know if direct
               connection will be used or not, tunneling should be disabled
               anyway.
            */ 
            clearH245Tunneling();
        }
    }
}


void ConnectionCallSig::
setToDisplay(const std::string& display)
{
    data_.toDisplay_ = display;
}


void ConnectionCallSig::
prepareSetup(const CallData& data)
{
    VLog log;
    VDEBUG(log) << "ConnectionCallSig::prepareSetup" << VDEBUG_END(log);

    outgoingCall_ = true;
    data_ = data;

    if (data_.hasConfid_)
    {
        VDEBUG(log) << "Got tunnelled conference ID" << VDEBUG_END(log);
        conferenceID_ = data_.confid_;
    }

    if (data_.hasCid_)
    {
        VDEBUG(log) << "Got tunnelled call ID" << VDEBUG_END(log);
        callID_ = data_.cid_;
    }
}


void ConnectionCallSig::
sendSetup()
{
    VLog log;
    VDEBUG(log) << "ConnectionCallSig::sendSetup" << VDEBUG_END(log);

    outgoingCall_ = true;

    BH225Setup setup;

    // Setup specific info

    // Invent callRef if needed
    // TODO*: there may be some better way than using static var
    if (callRef_ <= 0)
    {
        static int s_callRef = 0;
        ++ s_callRef;

        if (s_callRef <= 0)
        {
            s_callRef = 1;
        }

        callRef_ = s_callRef;
    }

    setup.getQ931().BuildSetup(callRef_);
    setup.getQ931().SetDisplayName(data_.fromDisplay_.c_str());
    //setup.getQ931().SetCalledPartyNumber(data_.fromDisplay_.c_str());
    callRef_ = setup.getQ931().GetCallReference();
    setup.m().m_conferenceID = conferenceID_;
    setup.m().m_callIdentifier.m_guid = callID_;
    setup.m().m_conferenceGoal.SetTag(
        H225_Setup_UUIE_conferenceGoal::e_create);
    setup.m().m_callType.SetTag(H225_CallType::e_pointToPoint);
    setup.m().m_mediaWaitForConnect = FALSE;
    setup.m().m_canOverlapSend = FALSE;

    if (data_.fromAliases_.GetSize() > 0)
    {
        setup.m().IncludeOptionalField(H225_Setup_UUIE::e_sourceAddress);
        setup.m().m_sourceAddress = data_.fromAliases_;
        
        H225_ArrayOf_AliasAddress   &   aliases = data_.fromAliases_;
        size_t size = aliases.GetSize();
        for ( size_t i = 0; i < size; i++ )
        {
            if ( aliases[i].GetTag() == H225_AliasAddress::e_e164 )
            {
                PASN_IA5String  &   e164 = aliases[i];
                setup.getQ931().SetCallingPartyNumber(e164);
                break;
            }
        }
    }

    if (data_.hasFromIP_)
    {
        setup.m().IncludeOptionalField(
            H225_Setup_UUIE::e_sourceCallSignalAddress);
        setup.m().m_sourceCallSignalAddress.SetTag(
            H225_TransportAddress::e_ipAddress);
        H225_TransportAddress_ipAddress& ipAddress =
            setup.m().m_sourceCallSignalAddress;
        data_.fromIP_.getIPAsOctetString(ipAddress.m_ip.GetPointer());
        ipAddress.m_port = data_.fromIP_.getPort();
    }

    if (data_.toAliases_.GetSize() > 0)
    {
        setup.m().IncludeOptionalField(H225_Setup_UUIE::e_destinationAddress);
        setup.m().m_destinationAddress = data_.toAliases_;
        
        int i = 0, a_size = data_.toAliases_.GetSize();
        bool a_found = false;
        std::string ret_str;

        i = 0;
        while((i < a_size)&&(!a_found))
          a_found = (data_.toAliases_[i++].GetTag() == H225_AliasAddress::e_e164);

        if(a_found)
        {
          --i;
	  PASN_IA5String &ia5phone = (PASN_IA5String &)data_.toAliases_[i];
          ret_str = std::string((const char*)((PString)ia5phone));
	
	  setup.getQ931().SetCalledPartyNumber(ret_str.c_str());
	}
    }

    if (data_.hasToIP_)
    {
        setup.m().IncludeOptionalField(
            H225_Setup_UUIE::e_destCallSignalAddress);
        setup.m().m_destCallSignalAddress.SetTag(
            H225_TransportAddress::e_ipAddress);
        H225_TransportAddress_ipAddress& ipAddress =
            setup.m().m_destCallSignalAddress;
        data_.toIP_.getIPAsOctetString(ipAddress.m_ip.GetPointer());
        ipAddress.m_port = data_.toIP_.getPort();
    }
    
    // H.245 address
    if (getSM()->getRouterInfo()->routerMediaControl())
    {
        IPAddress h245addressPort = getSM()->getH245Address();
        IPAddress h245addressAddr = getSM()->getCallSignalAddress();

        if (h245addressPort.getPort() && h245addressAddr.getPort())
        {
            setup.m().IncludeOptionalField(H225_Setup_UUIE::e_h245Address);
            setup.m().m_h245Address.SetTag(
                H225_TransportAddress::e_ipAddress);
            H225_TransportAddress_ipAddress& ipAddress =
                setup.m().m_h245Address;
            h245addressAddr.getIPAsOctetString(ipAddress.m_ip.GetPointer());
            ipAddress.m_port = h245addressPort.getPort();
        }
    }

    // Change status of the Call Control (passive) connection state machine
    if (setup.m().HasOptionalField(H225_Setup_UUIE::e_h245Address))
    {
        EvtCCSentAddress evt(getSM());
        evt.start();
    }

    // Send fast start data
    if (!queue_.empty())
    {
        setup.m().IncludeOptionalField(H225_Setup_UUIE::e_fastStart);
        sendFastStart(&setup.m().m_fastStart);
    }

    // General info
    setup.m().m_protocolIdentifier.SetValue(H225ProtocolID);
    getSM()->setSiteInfo(setup.m().m_sourceInfo);

    send(setup);
}


void ConnectionCallSig::
sendCallProceeding()
{
    VLog log;
    VDEBUG(log) << "ConnectionCallSig::sendCallProceeding" << VDEBUG_END(log);

    BH225CallProceeding cp;

    cp.getQ931().BuildCallProceeding(callRef_);

    // Call proceeding specific info
    cp.m().m_callIdentifier.m_guid = callID_;

    // H.245 address
    if (getSM()->getRouterInfo()->routerMediaControl())
    {
        IPAddress h245addressPort = getSM()->getH245Address();
        IPAddress h245addressAddr = getSM()->getCallSignalAddress();

        if (h245addressPort.getPort() && h245addressAddr.getPort())
        {
            cp.m().IncludeOptionalField(H225_CallProceeding_UUIE::e_h245Address);
            cp.m().m_h245Address.SetTag(
                H225_TransportAddress::e_ipAddress);
            H225_TransportAddress_ipAddress& ipAddress =
                cp.m().m_h245Address;
            h245addressAddr.getIPAsOctetString(ipAddress.m_ip.GetPointer());
            ipAddress.m_port = h245addressPort.getPort();
        }
    }

    // Change status of the Call Control (passive) connection state machine
    if (cp.m().HasOptionalField(H225_CallProceeding_UUIE::e_h245Address))
    {
        EvtCCSentAddress evt(getSM());
        evt.start();
    }

    // Send fast start data
    if (!queue_.empty())
    {
        cp.m().IncludeOptionalField(H225_Connect_UUIE::e_fastStart);
        sendFastStart(&cp.m().m_fastStart);
    }

    // General info
    cp.m().m_protocolIdentifier.SetValue(H225ProtocolID);
    getSM()->setSiteInfo(cp.m().m_destinationInfo);

    send(cp);
}


void ConnectionCallSig::
sendAlerting(BH225Alerting& alerting)
{
    VLog log;
    VDEBUG(log) << "ConnectionCallSig::sendAlerting" << VDEBUG_END(log);

    alerting.getQ931().BuildAlerting(callRef_);

    // Alerting specific info
    alerting.m().m_callIdentifier.m_guid = callID_;

    // H.245 address
    if (getSM()->getRouterInfo()->routerMediaControl())
    {
        IPAddress h245addressPort = getSM()->getH245Address();
        IPAddress h245addressAddr = getSM()->getCallSignalAddress();

        if (h245addressPort.getPort() && h245addressAddr.getPort())
        {
            alerting.m().IncludeOptionalField(H225_Alerting_UUIE::e_h245Address);
            alerting.m().m_h245Address.SetTag(
                H225_TransportAddress::e_ipAddress);
            H225_TransportAddress_ipAddress& ipAddress =
                alerting.m().m_h245Address;
            h245addressAddr.getIPAsOctetString(ipAddress.m_ip.GetPointer());
            ipAddress.m_port = h245addressPort.getPort();
        }
    }

    // Change status of the Call Control (passive) connection state machine
    if (alerting.m().HasOptionalField(H225_Alerting_UUIE::e_h245Address))
    {
        EvtCCSentAddress evt(getSM());
        evt.start();
    }

    // Send fast start data
    if (!queue_.empty())
    {
        alerting.m().IncludeOptionalField(H225_Connect_UUIE::e_fastStart);
        sendFastStart(&alerting.m().m_fastStart);
    }

    // General info
    alerting.m().m_protocolIdentifier.SetValue(H225ProtocolID);
    getSM()->setSiteInfo(alerting.m().m_destinationInfo);

    send(alerting);
}


void ConnectionCallSig::
sendConnect(BH225Connect& connect)
{
    VLog log;
    VDEBUG(log) << "ConnectionCallSig::BH225Connect" << VDEBUG_END(log);

    connect.getQ931().BuildConnect(callRef_);
    connect.getQ931().SetDisplayName(data_.toDisplay_.c_str());

    // Connect specific info
    connect.m().m_callIdentifier.m_guid = callID_;
    connect.m().m_conferenceID = conferenceID_;

    // H.245 address
    IPAddress h245addressPort = getSM()->getH245Address();
    IPAddress h245addressAddr = getSM()->getCallSignalAddress();

    if (haveRemoteH245())
    {
        VINFO(log) << "will try to use H245 patch, no tunneling" << VINFO_END(log);
        connect.m().IncludeOptionalField(H225_Connect_UUIE::e_h245Address);
        connect.m().m_h245Address.SetTag(
            H225_TransportAddress::e_ipAddress);
        H225_TransportAddress_ipAddress& ipAddress =
            connect.m().m_h245Address;
        remoteH245_.getIPAsOctetString(ipAddress.m_ip.GetPointer());
        ipAddress.m_port = remoteH245_.getPort();
        clearH245Tunneling();
    }
    else
    {
        VINFO(log) << "no H245 patch" << VINFO_END(log);

        if (h245addressPort.getPort() && h245addressAddr.getPort())
        {
            connect.m().IncludeOptionalField(H225_Connect_UUIE::e_h245Address);
            connect.m().m_h245Address.SetTag(
                H225_TransportAddress::e_ipAddress);
            H225_TransportAddress_ipAddress& ipAddress =
                connect.m().m_h245Address;
            h245addressAddr.getIPAsOctetString(ipAddress.m_ip.GetPointer());
            ipAddress.m_port = h245addressPort.getPort();
        }

        if (h245Tunneling())
        {
            VINFO(log) << "use tunneling" << VINFO_END(log);

            // Not obvious use of EvtReceivedCCAddress:
            // (to start sending tunneled messages)
            EvtReceivedCCAddress evt(getSM());
            evt.destAddress_.setPort(0);
            evt.start();
	    }
    }

    // Change status of the Call Control (passive) connection state machine
    if (connect.m().HasOptionalField(H225_Connect_UUIE::e_h245Address))
    {
        EvtCCSentAddress evt(getSM());
        evt.start();
    }

    // Send fast start data
    if (!queue_.empty())
    {
        connect.m().IncludeOptionalField(H225_Connect_UUIE::e_fastStart);
        sendFastStart(&connect.m().m_fastStart);
    }

    // General info
    connect.m().m_protocolIdentifier.SetValue(H225ProtocolID);
    getSM()->setSiteInfo(connect.m().m_destinationInfo);

    send(connect);
}


void ConnectionCallSig::
sendReleaseComplete(int reason)
{
    VLog log;
    VDEBUG(log) << "ConnectionCallSig::sendReleaseComplete"
                << VDEBUG_END(log);

    BH225ReleaseComplete rc;

    rc.getQ931().BuildReleaseComplete(callRef_, outgoingCall_);

    rc.m().m_callIdentifier.m_guid = callID_;
    rc.m().m_protocolIdentifier.SetValue(H225ProtocolID);
    if (reason >= 0)
    {
        rc.m().IncludeOptionalField(H225_ReleaseComplete_UUIE::e_reason);
        rc.m().m_reason.SetTag(reason);
    }

    send(rc);
}


void ConnectionCallSig::
sendEmptyFacility()
{
    VLog log;
    VDEBUG(log) << "ConnectionCallSig::sendEmptyFacility"
                << VDEBUG_END(log);

    BH225Empty ef;

    ef.getQ931().BuildFacility(callRef_, outgoingCall_);

    send(ef);
}


void ConnectionCallSig::
routeImpl(EvtRouterType& evtRouter, SrcType& evtSource)
{
    VLog log;
    VDEBUG(log) << "ConnectionCallSig::routeImpl" << VDEBUG_END(log);

    onIncoming(evtSource.getQ931Base());
    processAttachments(evtSource.getQ931Base());
    evtRouter.process(*getSM());

    // If there are still some tunneling elements to send, use Empty Facility
    if (evtSource.getType() != H225Tag::e_releaseComplete &&
        !getSM()->getCallCtlConnection()->queue_.empty())
    {
        sendEmptyFacility();
    }
}


CallIdentifier ConnectionCallSig::
getCallID()
{
    H225_ConferenceIdentifier confid;
    confid = conferenceID_;
    H225_CallIdentifier cid;
    cid.m_guid = callID_;

    return CallIdentifier(confid, cid);
}


void ConnectionCallSig::
setH245(IPAddress remoteH245)
{
    remoteH245_ = remoteH245;
    haveRemoteH245_ = true;
}


bool ConnectionCallSig::
haveRemoteH245() const throw()
{
    return haveRemoteH245_;
}


IPAddress ConnectionCallSig::
getRemoteH245() const
{
    return remoteH245_;
}


IPAddress ConnectionCallSig::
getRemoteAddress() const
{
    assert(socket_);

    Sptr<IPAddress> pAddress = socket_->getRemoteIPAddress();

    IPAddress address;

    if (pAddress != 0)
    {
        address = *pAddress;
    }

    return address;
}


void ConnectionCallSig::
setOverrideH245() throw()
{
    overrideH245_ = true;
    clearH245Tunneling();
}


bool ConnectionCallSig::
overrideH245() const throw()
{
    return overrideH245_;
}


void ConnectionCallSig::
clearH245Tunneling() throw()
{
    bool old = h245tunneling_;
    h245tunneling_ = false;

    if (old)
    {
        // Try to open separate channel (only if H.245 address is known)
        EvtReceivedCCAddress evt(getSM());
        evt.destAddress_.setPort(0);
        evt.start();
    }
}


bool ConnectionCallSig::
h245Tunneling() const throw()
{
    return h245tunneling_;
}


void ConnectionCallSig::
setFastConnect() throw()
{
    fastConnect_ = true;
}


bool ConnectionCallSig::
fastConnect() const throw()
{
    return fastConnect_;
}


void ConnectionCallSig::
addFSOLC(BH245OLC& olc)
{
    queue_.push(olc);
}


void ConnectionCallSig::sendFastStart(H225_ArrayOf_PASN_OctetString* target)
{
    while (!queue_.empty())
    {
        BH245OLC fsMessage = queue_.front();

        PPER_Stream strm;
        fsMessage.Encode(strm);
        strm.CompleteEncoding();

        PASN_OctetString pos; pos = strm;

        unsigned size = target->GetSize();
        target->SetSize(size + 1);
        (*target)[size] = pos;

        queue_.pop();
    }
}


#ifndef DISABLE_WRITER

ostream& ConnectionCallSig::
writeTo(ostream& out) const
{
    out << "ConnectionCallSig: " << this
        << ", sm=" << getSM();
    return out;
}

#endif
