/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */


static const char* const H245Protocol_cxx_Version = 
    "$Id$";

#include "VLog.hxx"

#include "H245Protocol.hxx"
#include "PollFifoService.hxx"
#include "H245Defs.hxx"
#include "Call.hxx"
#include "H323CodecException.hxx"
#include "CallCtlUnexpectedException.hxx"
#include "EvtCCConnected.hxx"
#include "CallTrace.hxx"

using namespace router;
using Vocal::Logging::VLog;


H245Protocol::
H245Protocol(
    PollFifoService& service,
    Sptr<TPKTClientSocket> h245Socket,
    SesH245& h245EventSrc,
    RouterCallCtl& callCtlRouter,
    ConnectionCallCtl& callCtlconnection,
    const char*	name = 0
) 
    : TPKTClientProtocol(name ? name : "H245Protocol"),
              service_(service),
              h245Socket_(h245Socket),
              h245EventSrc_(h245EventSrc),
              callCtlRouter_(callCtlRouter),
              connection_(callCtlconnection),
              isIgnored_(false)
{
    awaitIncoming(true);
    service_.registerProtocol(*this);
    getTPKTClientSocket().setNonblocking();

    // Separate control channel opened - no more tunneling
    callCtlconnection.getSM()->getCallSigConnection()->clearH245Tunneling();
}


H245Protocol::
~H245Protocol()
{
    VLog log;
    VDEBUG(log) << "H245Protocol::~H245Protocol" << VDEBUG_END(log);

    service_.unregisterProtocol(*this);
}


const FileDescriptor& H245Protocol::
getFileDescriptor() const
{
    assert(this);
    assert(h245Socket_ != 0);

    return *h245Socket_;
}


TPKTClientSocket& H245Protocol::
getTPKTClientSocket()
{
    assert(this);
    assert(h245Socket_ != 0);

    return *h245Socket_;
}


void H245Protocol::
onOutgoingAvailable()
{
    assert(this);

    awaitOutgoing(false);
    awaitIncoming(true);
    connection_.getSM()->getFifo()->updateProtocol(*this);

    // Generate exception if connection was not opened
    getTPKTClientSocket().completeConnection();

    // Generate 'TCP Connected' event
    EvtCCConnected evt(connection_.getSM());
    evt.start();
}


void H245Protocol::
onTPKTArrived(vector<u_int8_t> & message)
{
    assert(this);

    VLog log;
    VDEBUG(log) << "H245Protocol::onTPKTArrived, size="
                << message.size()
                << VDEBUG_END(log);

    try
    {
        PPER_Stream strm(&message[0], message.size());

        H245Message msg;
        msg.DecodePER(strm);

#ifndef DISABLE_WRITER

    string  msgName;
    
    switch ( msg.GetTag() )
    {
    	case H245Message::e_request:
	{
	    H245_RequestMessage & req = msg;
	    msgName = string(req.GetTagName());
	    break;
	}
    	case H245Message::e_response:
	{
	    H245_ResponseMessage & resp = msg;
	    msgName = string(resp.GetTagName());
	    break;
	}
    	case H245Message::e_command:
	{
	    H245_CommandMessage & cmd = msg;
	    msgName = string(cmd.GetTagName());
	    break;
	}
    	case H245Message::e_indication:
	{
	    H245_IndicationMessage & ind = msg;
	    msgName = string(ind.GetTagName());
	    break;
	}
	default:
	{
	    break;
	}
    }

    CALLTRACE(log, CT_IN, getTPKTClientSocket().getRemoteAddress(), CT_H245, msgName)
    	<< "{\n  " << setprecision(2) << msg
	<< "\n}" << CALLTRACE_END(log);
#endif

        h245EventSrc_.generate(msg, &connection_);
    }
    catch(CallCtlUnexpectedException&)
    {
	// Ignore
    }
    catch(...)
    {
        VWARN(log) << "Terminating the call" << VWARN_END(log);

        // Destroy call object
        terminate();
    }
}


void H245Protocol::
onDisconnect(ConnectionBrokenException*)
    throw ( Vocal::Transport::ProtocolException )
{
    assert(this);

    VLog log;

    if (isIgnored_)
    {
        VINFO(log) << "Got TCP disconnect for call control channel, "
                   << "closing this prococol" << VINFO_END(log);
        connection_.getSM()->disable(this);
    }
    else
    {
        VWARN(log) << "Got TCP disconnect for call control channel, "
                   << "terminating the call" << VWARN_END(log);
        // TODO*: try to reestablish connection

        terminate();
    }
}


void H245Protocol::
onError(SystemException*)
    throw ( Vocal::Transport::ProtocolException )
{
    assert(this);

    VLog log;

    VWARN(log) << "Got TCP error for call control channel, "
              << "terminating the call" << VWARN_END(log);

    terminate();
}


void H245Protocol::
handleUnknownException()
{
    assert(this);

    VLog log;

    VWARN(log) << "Got unknown exception for call control channel, "
              << "terminating the call" << VWARN_END(log);

    terminate();
}


void H245Protocol::
terminate()
{
    assert(this);

    // Destroy call object
    connection_.getSM()->terminate();
}


void H245Protocol::
ignore()
{
    assert(this);

    isIgnored_ = true;
}
