/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */


static const char* const H323CallTranslator_cxx_Version = 
    "$Id: H323CallTranslator.cxx,v 1.19 2001/10/26 21:47:55 icahoon Exp $";


#include "H323CallTranslator.hxx"
#include "RouterInfo.hxx"
#include "Q931ServerProtocol.hxx"
#include "ProtocolException.hxx"
#include "TrRouterSIP.hxx"
#include "SesSip.hxx"
#include "SipEventObserver.hxx"
#include "RasEventObserver.hxx"
#include "AsyncEventObserver.hxx"
#include "TimerEventObserver.hxx"
#include "VLog.hxx"
#include "TrCallFactoryImp.hxx"
#include "SesQ931.hxx"
#include "SesH245.hxx"
#include "RouterCallSig.hxx"

#include "EvtInvite.hxx"
#include "EvtSipStatus.hxx"
#include "EvtSipAck.hxx"
#include "EvtCancel.hxx"
#include "EvtBye.hxx"
#include "EvtTrSetup.hxx"
#include "EvtCallProceeding.hxx"
#include "EvtTrAlerting.hxx"
#include "EvtTrConnect.hxx"
#include "EvtTrOLC.hxx"
#include "EvtTrOLCAck.hxx"
#include "EvtTrTCS.hxx"
#include "EvtTCSAck.hxx"
#include "EvtTrCLC.hxx"
#include "EvtCLCAck.hxx"
#include "EvtMSDetermination.hxx"
#include "EvtMSDeterminationAck.hxx"
#include "EvtMSDeterminationReject.hxx"
#include "EvtMSDeterminationRelease.hxx"
#include "EvtOLC.hxx"
#include "EvtOLCAck.hxx"
#include "EvtTrOLCReject.hxx"
#include "EvtTrRCC.hxx"
#include "EvtRCCReplied.hxx"
#include "EvtRCCAck.hxx"
#include "EvtRCCReject.hxx"
#include "EvtRCCRelease.hxx"
#include "EvtRTDRequest.hxx"
#include "EvtTCS.hxx"
#include "EvtTCSAck.hxx"
#include "EvtTCSReject.hxx"
#include "EvtTCSRelease.hxx"
#include "EvtTCSReplied.hxx"
#include "EvtReleaseComplete.hxx"
#include "EvtEndSession.hxx"
#include "EvtFacility.hxx"
#include "EvtEmpty.hxx"

using namespace Vocal;
using Vocal::Services::Service;
using Vocal::Logging::VLog;

H323CallTranslator::H323CallTranslator(
    RouterInfo      	    &	routerInfo
)
    :	PollFifoService("H323CallTranslator"),
    	routerInfo_(routerInfo),
    	gatekeeperFifo_(0),
    	sipProxyFifo_(0),
    	sipProxyOutFifo_(0),
    	q931ServerProtocol_(0),
    	trRouterSIP_(0),
    	trRouterRas_(0),
    	sesSip_(0),
        sipEventObserver_(0),
        rasEventObserver_(0),
        asyncEventObserver_(0),
        timerEventObserver_(0),
        callFactory_(0),
        sesQ931_(0),
        routerQ931_(0),
        sesH245_(0),
        routerH245_(0),
        threadPool_(1)
{
    const string    	fn("H323CallTranslator::H323CallTranslator");
    VLog    	    	log(fn);
    
    VDEBUG(log) << fn << ": created." << VDEBUG_END(log);
}


H323CallTranslator::~H323CallTranslator()
{
    const string    	fn("H323CallTranslator::~H323CallTranslator");
    VLog    	    	log(fn);
    
    VDEBUG(log) << fn << ": destroyed." << VDEBUG_END(log);
    
    clear();
}


ReturnCode
H323CallTranslator::onStart()
{
    const string    	fn("H323CallTranslator::onStart");
    VLog    	    	log(fn);
    
    VDEBUG(log) << fn << ": starting." << VDEBUG_END(log);

    ReturnCode	rc = SUCCESS;
    
    try
    {
    	// Create q931 protocol (call leg) container.
        //
        if ( !trRouterSIP_ )
        {
            trRouterSIP_ = new translator::TrRouterSIP;
        }
        if ( !trRouterRas_ )
        {
            trRouterRas_ = new translator::TrRouterRas;
        }
        if ( !routerQ931_ )
        {
            routerQ931_ = new router::RouterCallSig;
        }
        if ( !routerH245_ )
        {
            routerH245_ = new router::RouterCallCtl;
        }
        if ( !sesSip_ )
        {
            sesSip_ = new translator::SesSip;
        }
        if ( !sesQ931_ )
        {
            sesQ931_ = new router::SesQ931;
        }
        if ( !sesH245_ )
        {
            sesH245_ = new router::SesH245;
        }
        if ( !callFactory_ )
        {
            callFactory_ = new translator::TrCallFactoryImp(
                *this,
                *sesQ931_,
                *routerQ931_,
                *sesH245_,
                *routerH245_,
                *trRouterSIP_,
                *trRouterRas_,
                routerInfo_,
                gatekeeperFifo_,
                sipProxyFifo_,
                threadPool_
                );
        }

        trRouterSIP_->setFactory(callFactory_);
        trRouterSIP_->setOutFifo(sipProxyOutFifo_);

        // Create q931 server (call leg generator) protocol.
        //
        if ( !q931ServerProtocol_ )
        {
    	    q931ServerProtocol_ = new Q931ServerProtocol(
                *this,
                *callFactory_,
                *sesQ931_,
                routerInfo_
                );
        }

        registerProtocol(*q931ServerProtocol_);

        if ( !sipEventObserver_ )
        {
            sipEventObserver_ = new translator::SipEventObserver(
                *this,
                *callFactory_,
                *trRouterSIP_,
                *sesSip_);
        }

        if ( !rasEventObserver_ )
        {
            rasEventObserver_ = new translator::RasEventObserver(
                *this,
                *trRouterRas_);
        }

        if ( !asyncEventObserver_ )
        {
            asyncEventObserver_ = new translator::AsyncEventObserver(
                *this,
                *trRouterSIP_);
        }

        if ( !timerEventObserver_ )
        {
            timerEventObserver_ = new router::TimerEventObserver(*this);
        }

        // Registering events ***

        // SIP
        translator::EvtInvite* eInvite =
            new translator::EvtInvite(trRouterSIP_);
        sesSip_->add(eInvite);

        translator::EvtSipStatus* eStatus =
            new translator::EvtSipStatus(trRouterSIP_);
        sesSip_->add(eStatus);

        translator::EvtSipAck* eAck =
            new translator::EvtSipAck(trRouterSIP_);
        sesSip_->add(eAck);

        translator::EvtCancel* eCancel =
            new translator::EvtCancel(trRouterSIP_);
        sesSip_->add(eCancel);

        translator::EvtBye* eBye =
            new translator::EvtBye(trRouterSIP_);
        sesSip_->add(eBye);

        // Q.931
        translator::EvtTrSetup* eSetup =
            new translator::EvtTrSetup(routerQ931_);
        sesQ931_->add(eSetup);

        router::EvtCallProceeding* eCallProceeding =
            new router::EvtCallProceeding(routerQ931_);
        sesQ931_->add(eCallProceeding);

        translator::EvtTrAlerting* eAlerting =
            new translator::EvtTrAlerting(routerQ931_);
        sesQ931_->add(eAlerting);

        translator::EvtTrConnect* eConnect =
            new translator::EvtTrConnect(routerQ931_);
        sesQ931_->add(eConnect);

        router::EvtReleaseComplete* eReleaseComplete =
            new router::EvtReleaseComplete(routerQ931_);
        sesQ931_->add(eReleaseComplete);

        router::EvtFacility* eFacility =
            new router::EvtFacility(routerQ931_);
        sesQ931_->add(eFacility);

        router::EvtEmpty* eEmpty =
            new router::EvtEmpty(routerQ931_);
        sesQ931_->add(eEmpty);

        // H.245
        router::EvtEndSession* eEndSession =
            new router::EvtEndSession(routerH245_);
        sesH245_->add(eEndSession);

        translator::EvtTrOLC* eOLC =
            new translator::EvtTrOLC(routerH245_);
        sesH245_->add(eOLC);

        translator::EvtTrOLCAck* eOLCAck =
            new translator::EvtTrOLCAck(routerH245_);
        sesH245_->add(eOLCAck);

        translator::EvtTrTCS* eTCS =
            new translator::EvtTrTCS(routerH245_);
        sesH245_->add(eTCS);

        router::EvtTCSAck* eTCSAck =
            new router::EvtTCSAck(routerH245_);
        sesH245_->add(eTCSAck);

        translator::EvtTrCLC* eCLC =
            new translator::EvtTrCLC(routerH245_);
        sesH245_->add(eCLC);

        router::EvtCLCAck* eCLCAck =
            new router::EvtCLCAck(routerH245_);
        sesH245_->add(eCLCAck);

        router::EvtMSDetermination* eMSDetermination =
            new router::EvtMSDetermination(routerH245_);
        sesH245_->add(eMSDetermination);

        router::EvtMSDeterminationAck* eMSDeterminationAck =
            new router::EvtMSDeterminationAck(routerH245_);
        sesH245_->add(eMSDeterminationAck);

        router::EvtMSDeterminationReject* eMSDeterminationReject =
            new router::EvtMSDeterminationReject(routerH245_);
        sesH245_->add(eMSDeterminationReject);

        router::EvtMSDeterminationRelease* eMSDeterminationRelease =
            new router::EvtMSDeterminationRelease(routerH245_);
        sesH245_->add(eMSDeterminationRelease);

        translator::EvtTrOLCReject* eOLCReject =
            new translator::EvtTrOLCReject(routerH245_);
        sesH245_->add(eOLCReject);

        translator::EvtTrRCC* eRCC =
            new translator::EvtTrRCC(routerH245_);
        sesH245_->add(eRCC);

        router::EvtRCCAck* eRCCAck =
            new router::EvtRCCAck(routerH245_);
        sesH245_->add(eRCCAck);

        router::EvtRCCReject* eRCCReject =
            new router::EvtRCCReject(routerH245_);
        sesH245_->add(eRCCReject);

        router::EvtRCCRelease* eRCCRelease =
            new router::EvtRCCRelease(routerH245_);
        sesH245_->add(eRCCRelease);

        router::EvtRTDRequest* eRTDRequest =
            new router::EvtRTDRequest(routerH245_);
        sesH245_->add(eRTDRequest);

        router::EvtTCSReject* eTCSReject =
            new router::EvtTCSReject(routerH245_);
        sesH245_->add(eTCSReject);

        router::EvtTCSRelease* eTCSRelease =
            new router::EvtTCSRelease(routerH245_);
        sesH245_->add(eTCSRelease);
    }
    catch ( ... )
    {
    	rc = !SUCCESS;
    }

    return ( rc );    
}


ReturnCode
H323CallTranslator::onStop()
{
    const string    	fn("H323CallTranslator::onStop");
    VLog    	    	log(fn);
    
    VDEBUG(log) << fn << ": stopping." << VDEBUG_END(log);

    ReturnCode	rc = SUCCESS;
    
    try
    {
        if (trRouterSIP_)
        {
            trRouterSIP_->stop();
            trRouterSIP_->shutdown(); // TODO*: not needed for graceful shutdown
        }

        if (routerQ931_)
        {
            routerQ931_->stop();
            routerQ931_->shutdown(); // TODO*: not needed for graceful shutdown
        }

        clear();
    }
    catch ( ... )
    {
    	rc = !SUCCESS;
    }

    return ( rc );
}


ReturnCode	    
H323CallTranslator::onShutdown()
{
    const string    	fn("H323CallTranslator::onShutdown");
    VLog    	    	log(fn);
    
    VDEBUG(log) << fn << ": shutting down." << VDEBUG_END(log);

    try
    {
        clear();
    }
    catch ( ... )
    {
    	VWARN(log)  << fn << ": exception on shutdown ." << VWARN_END(log);
    }

    return ( SUCCESS );
}


void
H323CallTranslator::connectToGatekeeper(FifoBase< Sptr<Event> > & gkFifo)
{
    gatekeeperFifo_ = &gkFifo;
}


void
H323CallTranslator::connectToSipProxy(
    FifoBase< Sptr<Event> > & outgoingSipProxyFifo,
    FifoBase< Sptr<Event> > & incomingSipProxyFifo
)
{
    sipProxyFifo_ = &incomingSipProxyFifo;
    sipProxyOutFifo_ = &outgoingSipProxyFifo;
}


void
H323CallTranslator::clear()
{
    delete sipEventObserver_;
    sipEventObserver_ = 0;
    
    delete asyncEventObserver_;
    asyncEventObserver_ = 0;
    
    delete timerEventObserver_;
    timerEventObserver_ = 0;

    delete rasEventObserver_;
    rasEventObserver_ = 0;

    delete sesSip_;
    sesSip_ = 0;
    
    delete trRouterSIP_;
    trRouterSIP_ = 0;

    delete trRouterRas_;
    trRouterRas_ = 0;

    delete callFactory_;
    callFactory_ = 0;

    delete sesQ931_;
    sesQ931_ = 0;

    delete sesH245_;
    sesH245_ = 0;

    delete routerQ931_;
    routerQ931_ = 0;

    delete routerH245_;
    routerH245_ = 0;

    if ( q931ServerProtocol_ )
    {
    	unregisterProtocol(*q931ServerProtocol_);
    }

    delete q931ServerProtocol_;
    q931ServerProtocol_ = 0;
    threadPool_.shutdown();
}
