/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */


static const char* const TrRouterSIP_cxx_Version = 
    "$Id: TrRouterSIP.cxx,v 1.9 2001/05/09 23:15:43 vzubarev Exp $";


#include "VLog.hxx"
#include "PollFifoService.hxx"

#include "SipMsg.hxx"
#include "SipCommand.hxx"
#include "ValAssistant.h"
#include "IncomingSipEvent.hxx"
#include "OutgoingSipEvent.hxx"

#include "SIPDefs.hxx"
#include "TrCall.hxx"
#include "TrRouterSIP.hxx"
#include "TrCallFactory.hxx"

using namespace translator;
using Vocal::Logging::VLog;
using Vocal::SipEvents::OutgoingSipEvent;


TrRouterSIP::
TrRouterSIP()
    : factory_(0)
{
}


TrRouterSIP::
~TrRouterSIP()
{
    destroy();
}


void TrRouterSIP::
stop()
{
    // While there are connections
    while (!empty())
    {
        // Gracefully shutdown SM for connection
        begin()->second->getSM()->complete();
    }

    // TODO*: notify application when last object is destroyed
}


void TrRouterSIP::
shutdown()
{
    // While there are connections
    while (!empty())
    {
        // Terminate SM for connection
        begin()->second->getSM()->terminate();
    }
}


void TrRouterSIP::
destroy()
{
    // While there are connections
    while (!empty())
    {
        // Remove SM for connection
        delete begin()->second->getSM();
    }
}


void TrRouterSIP::
setOutFifo(FifoBase< Sptr<Event> >* sipProxyOutFifo)
{
    sipProxyOutFifo_ = sipProxyOutFifo;
}


void TrRouterSIP::
setFactory(TrCallFactory* factory) throw()
{
    factory_ = factory;
}


bool TrRouterSIP::
route(EvtRouterType& evtRouter, EvtSIPMsg& evtSource)
{
    VLog log;
    VDEBUG(log) << "TrRouterSIP::route" << VDEBUG_END(log);

    bool isExisting = RtSIP::route(evtRouter, evtSource);

    if (isExisting)
    {
        VDEBUG(log) << "SIP routing successfull" << VDEBUG_END(log);
        return true;
    }
    else
    {
        VDEBUG(log) << "SIP routing failed, trying to create new call"
                    << VDEBUG_END(log);

        if (factory_ &&
            evtSource.getType() == SIP_INVITE)
        {
            TrCall* call = factory_->createTrCall();
            call->setProxyFifo(sipProxyOutFifo_);
            call->getH323Call()->init();

            call->getSIPConnection()->id_ =
                evtSource.getSelMessage().getCallId();
            call->getSIPConnection()->setProxy(sipProxyOutFifo_);
            add(*(call->getSIPConnection()));

            VDEBUG(log) << "SIP connection for new call registered"
                        << VDEBUG_END(log);
            return RtSIP::route(evtRouter, evtSource);
        }
        else
        {
            VWARN(log) << "SIP routing failed, ignoring the message"
                        << VWARN_END(log);

            const SipCommand* sc =
                dynamic_cast<const SipCommand*>(&(evtSource.getSelMessage()));

            // Do not send respCallLegOrTransactionDoesNotExist for ACK
            if (sc && sc->getType() != SIP_ACK)
            {
                Sptr<StatusMsg> sm = new StatusMsg(
                    *const_cast<SipCommand*>(sc),
                    msip::respCallLegOrTransactionDoesNotExist
                    );

                // Send "not existing" reply to proxy
                Sptr<SipEvent> sipEvent= new SipEvent(0);
                sipEvent->setSipMsg(sm);
                Sptr<OutgoingSipEvent> ose = new OutgoingSipEvent(sipEvent);
                assert(evtSource.getProxy());
                evtSource.getProxy()->add(ose);

                return true;
            }

            return false;
        }
    }
}


#ifndef DISABLE_WRITER

ostream& TrRouterSIP::
writeTo(ostream& out) const
{
    out << "TrRouterSIP: " << this;
    return out;
}

#endif
