
/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */

static const char* const Heartbeat_cxx_Version = "$Id: Heartbeat.cxx,v 1.16 2001/02/07 02:33:04 hlo Exp $";

#include <cassert>
#include "Heartbeat.hxx"
#include "HeartbeatTxThread.hxx"
#include "HeartbeatRxThread.hxx"
#include "HouseKeepingThread.hxx"
#include "CAServerContainer.hxx"
#include "Data.hxx"
#include "ServerContainer.hxx"
#include "cpLog.h"
#include "callAgent.hxx"
#include "VException.hxx"


// prototype of startHeartbeating
void startHeartbeating();
void * heartbeat(void *);


/*
 * Start the heart beat thread for callAgent
 */
void
startHeartbeating()
{
    int errNum;
    pthread_t tid;

    if ((errNum = pthread_create (&tid, NULL, heartbeat, NULL)) > 0)
    {
        cpLog (LOG_ERR, "Cannot create a start heartbeat thread: %s", strerror (errNum));
    }
}

/*
 * Instantiate the callAgent heartbeat object
 */
void*
heartbeat(void *)
{
    Heartbeat caHeartbeat;
    caHeartbeat.start();
    caHeartbeat.join();

    return NULL;
}

/*
 * Default Constructor
 */
Heartbeat::Heartbeat()
        : itsSipPort(0), callAgentId(0)
{
    cpLog (LOG_DEBUG, "Construct the Heartbeat Object");

    // get the sipMyHost and sipMyPort from ca.cfg
    // - later, it will be in provisionInterface
    Data host(ca.sipMyHost), port(ca.sipMyPort);
    callAgentId = new Data(host + ":" + port);

    // sip port from ca.cfg
    itsSipPort = ca.sipMyPort;

    // construct the heartbeat transmit thread
    itsHeartbeatTxThread = new HeartbeatTxThread(itsSipPort);

    // construct the heartbeat received thread
    itsHeartbeatRxThread = new HeartbeatRxThread(
                                  CAServerContainer::instance() );

    // construct the housekeeping thread
    itsHouseKeepingThread = new HouseKeepingThread(
                                      CAServerContainer::instance() );


}

/*
 * Default destructor
 */
Heartbeat::~Heartbeat()
{}


/*
 * Copy the heartbeat object to self
 * @param rhs the heartbeat object to copy
 */
void
Heartbeat::copyRhsToThis(const Heartbeat& rhs)
{
    itsSipPort = rhs.itsSipPort;
    itsHeartbeatTxThread = rhs.itsHeartbeatTxThread;
}

/*
 * Assignment operator
 * @param rhs the heartbeat object to copy
 * @return Heartbeat& return this object
 */
Heartbeat&
Heartbeat::operator=(const Heartbeat& rhs)
{
    if (this != &rhs)
    {
        copyRhsToThis(rhs);
    }
    return *this;
}

/*
 * Start heartbeating to the multicast group
 */
void
Heartbeat::start()
{
    if (itsHeartbeatTxThread != 0)
    {
        itsHeartbeatTxThread->run();
    }

    if (itsHeartbeatRxThread != 0)
    {
        itsHeartbeatRxThread->run();
    }

    if (itsHouseKeepingThread != 0)
    {
        itsHouseKeepingThread->run();
    }

}

/*
 * Shut down the heartbeat threads
 */
void
Heartbeat::shutdown()
{
    if (itsHeartbeatTxThread != 0)
    {
        itsHeartbeatTxThread->shutdown();
    }

    if (itsHeartbeatRxThread != 0)
    {
        itsHeartbeatRxThread->shutdown();
    }

    if (itsHouseKeepingThread != 0)
    {
        itsHouseKeepingThread->shutdown();
    }

}

/*
 * Join the heartbeat threads
 */
void
Heartbeat::join()
{
    if (itsHeartbeatTxThread != 0)
    {
        itsHeartbeatTxThread->join();
    }

    if (itsHeartbeatRxThread != 0)
    {
        itsHeartbeatRxThread->join();
    }

    if (itsHouseKeepingThread != 0 )
    {
        itsHouseKeepingThread->join();
    }

}
