
/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */

static const char* const callAgent_cxx_Version = "$Id: callAgent.cxx,v 1.38 2001/02/07 02:41:01 hlo Exp $";


#include <sys/time.h>
#include <sys/types.h>
#include <signal.h>
#include <errno.h>
#include <netdb.h>

#include "global.h"

#include "callAgent.hxx"
#include "cpLog.h"
#include "parse3tuple.h"
#include "caConfigure.hxx"
#include "callProcessing.hxx"
#include "endpoint.hxx"
#include "sipProxy.hxx"
#include "SipTransceiverFilter.hxx"
#include "InviteMsg.hxx"
#include "StatusMsg.hxx"
#include "CancelMsg.hxx"
#include "AckMsg.hxx"
#include "ByeMsg.hxx"
#include "RegisterMsg.hxx"
#include "OptionsMsg.hxx"
#include "InfoMsg.hxx"

#include "Heartbeat.hxx"
#include "MessageHandler.hxx"
#include "StartupSequence.hxx"

#include "TransferMsg.hxx"

pthread_mutex_t exitMutex = PTHREAD_MUTEX_INITIALIZER;

/// Shared global data
CallAgent ca;                               /// Call Agent data

Fifo < QueueEventData* > iqs [CP_THREAD_MAX];  /// CP thread input queues

int connectCount [CP_THREAD_MAX];

Endpoint eps [ENDPOINT_MAX];                /// Endpoint data
EndpointMap epMap;                          /// Key: endpoint id

Gateway gws [GATEWAY_MAX];                  /// Gateways data
map < string, Gateway* > gwMap;                /// Key: domain name

CallData call [CALL_DATA_MAX];              /// Call data pool
pthread_mutex_t callDataMutex = PTHREAD_MUTEX_INITIALIZER;
list < CallData* > callDataList;               /// Unused call data block list

map < string, Endpoint* > dnMap;

SipTransceiverFilter* sipProxyStack;


QueueEventData qEventData[Q_EVENT_DATA_MAX];
pthread_mutex_t queueEventsMutex = PTHREAD_MUTEX_INITIALIZER;
list < QueueEventData* > qEventDataList;              /// list of free qEventData

///
QueueEventData*
newQueueEventData (void)
{
#if FENCEPOST_DEBUGGING_CODE
    static bool initializedFencepost = false;

    if (!initializedFencepost)
    {
        for (int i = 0; i < Q_EVENT_DATA_MAX; i++)
        {
            qEventData[i].blank_0 = 0;
            qEventData[i].blank_1 = 0;
            qEventData[i].blank_2 = 0;
            qEventData[i].blank_3 = 0;
            qEventData[i].blank_4 = 0;
            qEventData[i].blank_5 = 0;
            qEventData[i].blank_6 = 0;
            qEventData[i].blank_7 = 0;
            qEventData[i].blank_8 = 0;
            qEventData[i].blank_9 = 0;
        }
        initializedFencepost = 1;
    }
#endif

    QueueEventData* rc = 0;

    int error = pthread_mutex_lock( &queueEventsMutex );

    if (error)
    {
        cpLog (LOG_ERR, "Cannot lock event list");
    }
    else
    {
        list < QueueEventData* > ::iterator p = qEventDataList.begin();

        if (qEventDataList.empty())
        {
            cpLog (LOG_ERR, "Cannot allocate event data");
            cpLog (LOG_ERR, "qEventDataList size = %d", qEventDataList.size());
        }
        else
        {
            if ( (*p) == 0 )
            {
                cpLog( LOG_DEBUG, "The iterator p points to a NULL pointer !!");
                assert( (*p) != 0 );
                return ( 0 );
            }
            cpLog (LOG_DEBUG, "Allocate event data %d, qEventDataList size = %d",
                   (*p)->id,
                   qEventDataList.size());
            /// Initialize fields when it is allocated just in case we
	    /// need to look at some of the data after the block is freed
	    ///  Note: cmd is re-initialized when block is freed
            (*p)->callIdentifier[0] = 0;
            (*p)->featureId = -1;
            rc = *p;
            qEventDataList.pop_front ();
        }

        error = pthread_mutex_unlock( &queueEventsMutex );
        if (error)
        {
            cpLog (LOG_ERR, "Cannot unlock event list");
        }
    }

    assert(!((rc < &qEventData[0]) || (rc > &qEventData[Q_EVENT_DATA_MAX - 1])));

    return rc;
}    /// newQueueEventData


///
void
freeQueueEventData (QueueEventData* ed)
{
    assert(!((ed < &qEventData[0]) || (ed > &qEventData[Q_EVENT_DATA_MAX - 1])));

    int error = pthread_mutex_lock( &queueEventsMutex );
    if (error)
    {
        cpLog (LOG_ERR, "Cannot lock event list");
    }
    else
    {
        if ( ed == 0 )
        {
            cpLog( LOG_DEBUG, "The ed is a NULL pointer !!");
            assert( ed );
            return ;
        }
        /// Initialize smart pointer here so it won't hold on to the memory
        ed->cmd = 0;
        /// Initialize other fields when it is re-allocated just in case
        /// we need to look at some of the data after the block is freed
        qEventDataList.push_back (ed);
        error = pthread_mutex_unlock( &queueEventsMutex );
        if (error)
        {
            cpLog (LOG_ERR, "Cannot unlock event list");
        }
    }
}    /// freeQueueEventData


///
void
qEventDataInit (void)
{
    for (int i = 0; i < Q_EVENT_DATA_MAX; i++)
    {
        qEventData[i].id = i;
        freeQueueEventData (&qEventData[i]);
    }
    cpLog (LOG_INFO, "Number of Event data = %d (%d max)",
           qEventDataList.size(),
           Q_EVENT_DATA_MAX);
}    /// qEventDataInit


/// Initialize all call data blocks
void
callDataInit (void)
{
    for (int i = 0; i < CALL_DATA_MAX; i++)
    {
        call[i].id = i;
        callDataInit (&call[i]);
    }
    cpLog (LOG_INFO, "Number of Call Data = %d (%d max)",
           callDataList.size(),
           CALL_DATA_MAX);

}    // callDataInit


///Gets Asynchronous Messages
void
getAsyncMessage (MgcGateway* asyncStack)
{
    cpLog (LOG_DEBUG, "callAgent::getAsyncMessage");

    if ( asyncStack == 0 )
    {
        cpLog( LOG_DEBUG, "The asyncStack is a NULL pointer !!");
        assert( asyncStack != 0 );
        return ;
    }
    Sptr < MgcCommand > cmd = asyncStack->asyncReceive ();
    Sptr < MessageHandler > msgHandler = new MessageHandler;
    if ( msgHandler.getPtr() == 0 )
    {
        cpLog( LOG_DEBUG, "Out of memory !!");
        assert( msgHandler.getPtr() != 0 );
        return ;
    }

    /// processMsg will detect wildcard or individual
    msgHandler->processMsg(cmd);

} /// getAsyncMessage

/// Put SIP message into the queue
void queueInSipMessage(SipMsg &msg)
{
    SipMsg* SIPcmd = &msg;

    QueueEventData* ed = newQueueEventData ();
    if ( ed == 0 )
    {
        cpLog( LOG_DEBUG, "Ran out of QueueEventData in the pool !!");
        assert( ed != 0 );
        delete( &msg );
        return ;
    }
    ed->type = SipStackEvent;
    ed->SIPcmd = SIPcmd;
    /// move into the same iq as the other events
    iqs[0].add( ed );
}

///
int
getMaxFd (fd_set* fds)
{
    int max = 0;
    int i;
    for (i = 0; i < 129; i++)
    {
        if (FD_ISSET (i, fds))
        {
            cpLog (LOG_DEBUG, "fd %d is set", i);
            max = i + 1;
        }
    }
    return max;
}    /// getMaxFd


///
void*
receiveMessage (void* ptr)
{
    MgcGateway* asyncStack = static_cast < MgcGateway* > (ptr);
    if ( asyncStack == 0 )
    {
        cpLog( LOG_DEBUG, "The asyncStack is a NULL pointer !!");
        assert( asyncStack );
        return ( 0 );
    }

    cpLog (LOG_INFO, "Message Receiving thread started (pid %d)",
           getpid());

    fd_set fds0;
    FD_ZERO (&fds0);
    asyncStack->addToFdSet (&fds0);

    for (; ; )
    {
        getAsyncMessage (asyncStack);
    }    /// for
    return 0;
}    /// receiveMessage


///
int
agentPortIndex (int port)
{
    int idx = 0;
    for (int i = 0; i < ca.localPortCount; i++)
    {
        if (ca.localPort[i] == port)
        {
            idx = i;
            break;
        }
    }
    return idx;
}    /// agentPortIndex


///
void
addGateways (MgcGateway* asyncStack[])
{
    int i;
    for (i = 0; i < ca.gatewayCount; i++)
    {
        // tbd: break 1-to-1 gateway-address relationship so a proxy can
        //      handle multiple gateways
        gws[i].asyncStack = asyncStack [agentPortIndex (gws[i].agentPort)];
        if ( gws[i].asyncStack == 0 )
        {
            cpLog( LOG_DEBUG, "The gws[i].asyncStack is a NULL pointer !!");
            assert( gws[i].asyncStack );
            return ;
        }
        gws[i].handle = gws[i].asyncStack->addGateway (gws[i].address.c_str(),
                        gws[i].port);
        cpLog (LOG_INFO, "Gateway [%d] %s:%d localPort %d handle %d",
               i,
               gws[i].address.c_str(),
               gws[i].port,
               gws[i].agentPort,
               gws[i].handle);
    }
}    // addGateways


///
void*
asyncProcessData (void* gateway)
{
    MgcGateway* gw = static_cast < MgcGateway* > (gateway);
    if ( gw == 0 )
    {
        cpLog( LOG_DEBUG, "The gw is a NULL pointer !!");
        assert( gw );
        return ( 0 );
    }
    cpLog (LOG_ERR, "processing thread started (pid %d)\n",
           getpid());
    gw->processPendingBlock();
    return 0;
}    // process


///
void*
asyncProcess (void* gateway)
{
    MgcGateway* gw = static_cast < MgcGateway* > (gateway);
    if ( gw == 0 )
    {
        cpLog( LOG_DEBUG, "The gw is a NULL pointer !!");
        assert( gw != 0 );
        return ( 0 );
    }
    cpLog (LOG_INFO, "Message Timeout thread started (pid %d)\n",
           getpid());
    gw->asyncProcess ();
    return 0;
}    // process


///
void
startMsgTimeoutThread (MgcGateway* asyncStack)
{
    {
        int errNum;
        pthread_t tid;
        if ((errNum = pthread_create (&tid, NULL, asyncProcess, asyncStack)) > 0)
        {
            cpLog (LOG_ERR, "Cannot create asyncProcess thread: %s", strerror (errNum));
        }
    }
}    // startMsgTimeoutThread


/// Start additional message receiving threads
void
startReceiveMessageThreads (MgcGateway* asyncStack[])
{
    int errNum;
    pthread_t tid;

    for (int i = 1; i < ca.localPortCount; i++)
    {
        if ((errNum = pthread_create (&tid, NULL, receiveMessage, asyncStack[i])) > 0)
        {
            cpLog (LOG_ERR, "Cannot create message receiving thread: %s", strerror (errNum));
        }
    }
}    // startReceiveMessageThreads


///Gets SIP messages 
void* sipMessageReceive (void*)
{
    cpLog(LOG_DEBUG, "starting sipMessageReceive");

    if ( sipProxyStack == 0 )
    {
        cpLog( LOG_DEBUG, "The sipProxyStack is a NULL pointer !!");
        assert( sipProxyStack != 0 );
        return ( 0 );
    }

    for (; ; )
    {
        SipMsg* sipMsg = 0;

        // get the next SIP message
        /**
                // TODO - fix this to nex line with timeout so that exit works
          	msg = sipProxyStack->receive( 500 ); // timeout in ms
        */

        try
        {
            //      - these two are essentially the same, and does not block
            //            msg = sipProxyStack->receiveAsync();
            //            msg = sipProxyStack->receive();
            //      - use this one to block to avoid CPU intensive busy loops
            //            Blocking is ok since this thread only receives SIP
            //            messages from a single source
            Sptr < SipMsgQueue > sipReceive = sipProxyStack->receive();
            if ( !sipReceive )
            {
                continue;
            }
            Sptr < SipMsg > msg = sipReceive->back();
            if (! msg )
            {
                continue;
            }

            int method = msg->getType();
            switch (method)
            {
                case SIP_INVITE:
                {
                    Sptr < InviteMsg > inviteMsg;
                    inviteMsg.dynamicCast(msg);

                    InviteMsg* myInvite = new InviteMsg(*inviteMsg);
                    sipMsg = myInvite;
                }
                break;
                case SIP_ACK:
                {
                    Sptr < AckMsg > ackMsg;
                    ackMsg.dynamicCast(msg);

                    AckMsg* myAck = new AckMsg(*ackMsg);
                    sipMsg = myAck;
                }
                break;
                case SIP_STATUS:
                {
                    Sptr < StatusMsg > statusMsg;
                    statusMsg.dynamicCast(msg);

                    StatusMsg* myStatus = new StatusMsg(*statusMsg);
                    sipMsg = myStatus;
                }
                break;
                case SIP_BYE:
                {
                    Sptr < ByeMsg > byeMsg;
                    byeMsg.dynamicCast(msg);

                    ByeMsg* myBye = new ByeMsg(*byeMsg);
                    sipMsg = myBye;
                }
                break;
                case SIP_CANCEL:
                {
                    Sptr < CancelMsg > cancelMsg;
                    cancelMsg.dynamicCast(msg);

                    CancelMsg* myCancel = new CancelMsg(*cancelMsg);
                    sipMsg = myCancel;
                }
                break;
                case SIP_OPTIONS:
                {
                    Sptr < OptionsMsg > optionsMsg;
                    optionsMsg.dynamicCast(msg);

                    OptionsMsg* myOptions = new OptionsMsg(*optionsMsg);
                    sipMsg = myOptions;
                }
                break;
                case SIP_INFO:
                {
                    Sptr < InfoMsg > infoMsg;
                    infoMsg.dynamicCast(msg);

                    InfoMsg* myInfo = new InfoMsg(*infoMsg);
                    sipMsg = myInfo;
                }
                break;
                case SIP_REGISTER:
                {
                    Sptr < RegisterMsg > registerMsg;
                    registerMsg.dynamicCast(msg);

                    RegisterMsg* myRegister = new RegisterMsg(*registerMsg);
                    sipMsg = myRegister;
                }
                break;
                case SIP_TRANSFER:
                {
                    Sptr < TransferMsg > transferMsg;
                    transferMsg.dynamicCast(msg);

                    TransferMsg* myTransfer = new TransferMsg(*transferMsg);
                    sipMsg = myTransfer;
                }
                break;
                default:
                   cpLog(LOG_DEBUG,"Received Unknown Message Type");
                break;
            }

            if (! sipMsg )
            {
                continue;
            }

        }

        catch (void* e)
        {
            cpLog(LOG_DEBUG, "Some exception in receive");
        }

        cpLog(LOG_DEBUG, "sipMessageReceive got message");


        // put it on the FIFO queue
        if ( sipMsg )
        {
            try
            {
                queueInSipMessage(*sipMsg);
            }
            catch (void* e)
            {
                cpLog(LOG_DEBUG, "Some exception in adding to fifo");
            }

        }
    }
}


///
void startReceiveSipMessage(int sipPort)
{
    //  currently, sipPort == 5060 from ca.cfg file
    cout << "sipPort = " << sipPort << endl;
    sipProxyStack = new SipTransceiverFilter("MGCP Translator", sipPort);
    sipProxyStack->reTransOff();

    int errNum;
    pthread_t tid;

    if ((errNum = pthread_create (&tid, NULL, sipMessageReceive, NULL)) > 0)
    {
        cpLog (LOG_ERR, "Cannot create SIP Proxy thread: %s", strerror (errNum))
        ;
    }
}


///
void
caStart (void)
{
    callDataInit ();

    qEventDataInit ();

    // create asynchronous receive connections
    MgcGateway* asyncStack [RECEIVE_PORT_MAX];
    int i;
    for (i = 0; i < ca.localPortCount; i++)
    {
        asyncStack[i] = new MgcGateway (ca.localPort[i]);
    }
    addGateways (asyncStack);

    // start the heartbeat thread now
    startHeartbeating();

    for (i = 0; i < ca.localPortCount; i++)
    {
        startMsgTimeoutThread (asyncStack[i]);
    }

    //tbd:    caOAMPStart ();

    // Additional message receiving threads
    startReceiveMessageThreads (asyncStack);

    // for receiving and processing SIP message
    // currently 5060
    startReceiveSipMessage(ca.sipMyPort);

    callProcessingStart ();

    // Default message receiving thread
    receiveMessage (asyncStack[0]);
    // Don't do any useful thing after receiveMessage(), it loops forever
}   // caStart



///
void
cpStatistics (void)
{
    for (int i = 0; i < ca.threadCount; i++)
    {
        cpLog (LOG_INFO, "CP thread %d: %d Connected Calls",
               i,
               connectCount[i]);
    }
}    // cpStatistics


/// SIGINT signal handler
void
stopCallAgent (int signal)
{
    int error = pthread_mutex_lock( &exitMutex );

    //tbd: have only one thread to handle the signal(s)
    cpLog (LOG_INFO, "Call Agent stopped on signal %d", signal);

    cpStatistics ();

    exit (0);
    error = pthread_mutex_unlock( &exitMutex );
}    // stopCallAgent


int
main (const int argc, const char* argv [])
{
    signal (SIGINT, stopCallAgent);
    signal (SIGHUP, stopCallAgent);
    
    caConfigure (argc, argv);

    caStart ();

    return (0);
}    // callAgent main
